<?php
/**
 * webEdition CMS
 *
 * $Rev: 2ccf85935e5c $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-10-16 00:22 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
// remove trailing slash
if(isset($_SERVER['DOCUMENT' . '_ROOT'])){ //so zerlegt stehen lassen: Bug #6318
	$_SERVER['DOCUMENT' . '_ROOT'] = rtrim($_SERVER['DOCUMENT' . '_ROOT'], '/');
}

foreach(['HTTP_USER_AGENT', 'PHP_SELF', 'REQUEST_URI', 'QUERY_STRING', 'REDIRECT_URL'] as $cur){
	if(isset($_SERVER[$cur])){
		$_SERVER[$cur] = strtr($_SERVER[$cur], ['<' => '%3C', '>' => '%3E', '"' => '%22', '\'' => '%27', '`' => '%60']);
	}
}

//due to hoster bugs (1&1) we have to ensure servername is the called url. since http-host is not safe, we do some security additions.
if(isset($_SERVER['HTTP_HOST']) && $_SERVER['SERVER_NAME'] != $_SERVER['HTTP_HOST']){
	//some security checks
	if(strlen($_SERVER['HTTP_HOST']) < 256 && strpos($_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME'])){
		$_SERVER['SERVER_NAME'] = rawurlencode($_SERVER['HTTP_HOST']);
	}
}

// Set PHP flags
@ini_set('allow_url_fopen', '1');
@ini_set('file_uploads', '1');
@ini_set('session.use_trans_sid', '0');
@ini_set('max_input_vars', '9000');
//@ini_set("arg_separator.output","&");
//fix insecure cookies
$cookie = session_get_cookie_params();
session_set_cookie_params($cookie['lifetime'], $cookie['path'], $cookie['domain'], $cookie['secure'], true);

//prepare space for we-variables; $_SESSION['weS'] is set in we_session
if(!isset($GLOBALS['we'])){
	$GLOBALS['we'] = [];
}

if(!(defined('SYSTEM_WE_SESSION') && SYSTEM_WE_SESSION) && ini_get('session.gc_probability') != '0'){
	ini_set('session.gc_probability', '0');
}

//start autoloader!
require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we_autoload.inc.php');
//register all used tables.
we_base_request::registerTables([
	'CAPTCHADEF_TABLE' => CAPTCHADEF_TABLE,
	'CAPTCHA_TABLE' => CAPTCHA_TABLE,
	'CLEAN_UP_TABLE' => CLEAN_UP_TABLE,
	'CONTENT_TABLE' => CONTENT_TABLE,
	'DOC_TYPES_TABLE' => DOC_TYPES_TABLE,
	'ERROR_LOG_TABLE' => ERROR_LOG_TABLE,
	'FAILED_LOGINS_TABLE' => FAILED_LOGINS_TABLE,
	'FILELINK_TABLE' => FILELINK_TABLE,
	'FILE_TABLE' => FILE_TABLE,
	'FORMMAIL_BLOCK_TABLE' => FORMMAIL_BLOCK_TABLE,
	'FORMMAIL_LOG_TABLE' => FORMMAIL_LOG_TABLE,
	'HISTORY_TABLE' => HISTORY_TABLE,
	'INDEX_TABLE' => INDEX_TABLE,
	'LANGLINK_TABLE' => LANGLINK_TABLE,
	'LOCK_TABLE' => LOCK_TABLE,
	'METADATA_TABLE' => METADATA_TABLE,
	'NOTEPAD_TABLE' => NOTEPAD_TABLE,
	'PREFS_TABLE' => PREFS_TABLE,
	'PWDRESET_TABLE' => PWDRESET_TABLE,
	'RECIPIENTS_TABLE' => RECIPIENTS_TABLE,
	'SEARCH_TABLE' => SEARCH_TABLE,
	'SESSION_TABLE' => SESSION_TABLE,
	'SETTINGS_TABLE' => SETTINGS_TABLE,
	'TEMPLATES_TABLE' => TEMPLATES_TABLE,
	'TEMPORARY_DOC_TABLE' => TEMPORARY_DOC_TABLE,
	'UPDATE_LOG_TABLE' => UPDATE_LOG_TABLE,
	'VALIDATION_SERVICES_TABLE' => VALIDATION_SERVICES_TABLE,
	'VERSIONSLOG_TABLE' => VERSIONSLOG_TABLE,
	'VERSIONS_TABLE' => VERSIONS_TABLE,
]);

$GLOBALS['we']['lang'] = include_once (WE_INCLUDES_PATH . 'conf/we_conf_language.inc.php');

we_modules_info::loadConfigs();

$GLOBALS['DB_WE'] = new DB_WE();

if(!(defined('NO_SESS') || defined('FROM_WE_SHOW_DOC'))){
	$GLOBALS['we']['Charset']['Backend'] = WE_BACKENDCHARSET; //Bug 5771 schon in der Session wird ein vorläufiges Backendcharset benötigt
	we_main_session::initSession();
}

$GLOBALS['WE_LANGUAGE'] = (!empty($_SESSION['prefs']['Language']) ?
	(is_dir(WE_INCLUDES_PATH . 'we_language/' . $_SESSION['prefs']['Language']) ?
	$_SESSION['prefs']['Language'] :
	//  bugfix #4229
	($_SESSION['prefs']['Language'] = WE_LANGUAGE)) :
	WE_LANGUAGE);

define('STYLESHEET_MINIMAL', we_html_element::cssLink(LIB_DIR . 'fontLiberation/stylesheet.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(CSS_DIR . 'we_button.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(LIB_DIR . 'fontawesome/css/font-awesome.min.css', ['rel' => 'preload', 'as' => 'style']));
define('STYLESHEET', STYLESHEET_MINIMAL .
	we_html_element::cssLink(CSS_DIR . 'webEdition.css', ['rel' => 'preload', 'as' => 'style'])
);

define('JQUERY', we_html_element::cssLink(LIB_DIR . 'jquery/jquery-ui.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(LIB_DIR . 'jquery/jquery-ui.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(LIB_DIR . 'jquery/jquery-ui.structure.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(LIB_DIR . 'jquery/jquery-ui.theme.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::cssLink(LIB_DIR . 'jquery/sumoselect.css', ['rel' => 'preload', 'as' => 'style']) .
	we_html_element::jsScript(LIB_DIR . 'jquery/jquery.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::jsScript(LIB_DIR . 'jquery/jquery-ui.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::jsScript(LIB_DIR . 'jquery/jquery.sumoselect.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::jsScript(LIB_DIR . 'jquery/jquery-autosize.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::jsScript(LIB_DIR . 'jquery/i18n/datepicker-' . array_search($GLOBALS['WE_LANGUAGE'], getWELangs()) . '.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::jsScript(JS_DIR . 'startJquery.js', '', ['rel' => 'preload',	'as' => 'script']) .
	we_html_element::cssLink(CSS_DIR . 'weJquery.css', ['rel' => 'preload', 'as' => 'style'])
);

if(!isset($GLOBALS['WE_IS_DYN'])){ //only true on dynamic frontend pages
	$GLOBALS['we']['Charset']['Backend'] = (!empty($_SESSION['prefs']['BackendCharset']) ?
		$_SESSION['prefs']['BackendCharset'] : 'UTF-8');

	//send header?
	switch(isset($_REQUEST['we_cmd']) && !is_array($_REQUEST['we_cmd']) ? we_base_request::_(we_base_request::STRING, 'we_cmd', '__default__') : ''){
		case 'edit_link':
		case 'edit_linklist':
		case 'show_newsletter':
		case 'save_document':
			$header = false;
			break;
		case 'reload_hot_editpage':
		case 'reload_editpage':
			$header = (!in_array($_SESSION['weS']['EditPageNr'], [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW, we_base_constants::WE_EDITPAGE_PROPERTIES]));
			break;
		case 'switch_edit_page':
			$header = (!in_array(we_base_request::_(we_base_request::INT, 'we_cmd', -1, 1), [we_base_constants::WE_EDITPAGE_CONTENT, we_base_constants::WE_EDITPAGE_PREVIEW,
					we_base_constants::WE_EDITPAGE_PROPERTIES]));
			break;
		case 'load_editor':
			$trans = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', '__NO_TRANS__');
			$header = (!(isset($_SESSION['weS']['we_data'][$trans]) &&
				$_SESSION['weS']['we_data'][$trans][0]['Table'] === FILE_TABLE &&
				$_SESSION['weS']['EditPageNr'] == we_base_constants::WE_EDITPAGE_PREVIEW
				));
			break;
		case '__default__':
			$header = empty($GLOBALS['show_stylesheet']);
			break;
		default:
			$header = true;
	}

	if($header){
		we_html_tools::headerCtCharset('text/html', $GLOBALS['we']['Charset']['Backend']);
	}
	unset($header);
}
