<?php

/**
 * webEdition CMS
 *
 * $Rev: 4bade8b85b2d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-07-03 22:25 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_base_charsetHandler{
	private $charsets = [];
	private static $inst = null;

	const CHARSETS = [
		'UTF-8',
		'ISO-8859-1',
		'ISO-8859-2',
		'ISO-8859-3',
		'ISO-8859-4',
		'ISO-8859-5',
		'ISO-8859-6',
		'ISO-8859-7',
		'ISO-8859-8',
		'ISO-8859-9',
		'ISO-8859-10',
		'ISO-8859-11',
		'ISO-8859-12',
		'ISO-8859-13',
		'ISO-8859-14',
		'ISO-8859-15',
		'Windows-1251',
		'Windows-1252',
	];

	/**
	 * initialises with all available charsets
	 */
	private function __construct(){
		//	First ISO-8859-charsets
		$this->charsets = [
			'west_european' => [
				'national' => 'West Europe', //	Here is the name of the country in mother language
				'charset' => 'ISO-8859-1',
			],
			'central_european' => [
				'national' => 'Central Europe',
				'charset' => 'ISO-8859-2',
			],
			'south_european' => [
				'national' => 'South Europe',
				'charset' => 'ISO-8859-3',
			],
			'north_european' => [
				'national' => 'North Europe',
				'charset' => 'ISO-8859-4',
			],
			'cyrillic' => [
				'national' => '&#1077;&#1086;&#1073;&#1077;&#1089;&#1087;&#1077;&#1095;',
				'charset' => 'ISO-8859-5',
			],
			'arabic' => [
				'national' => '&#1578;&#1587;&#1580;&#1617;&#1604;&#1575;&#1604;&#1570;&#1606;',
				'charset' => 'ISO-8859-6',
			],
			'greek' => [
				'national' => 'Greek',
				'charset' => 'ISO-8859-7',
			],
			'hebrew' => [
				'national' => '&#1488;&#1497;&#1512;&#1493;&#1508;&#1492;',
				'charset' => 'ISO-8859-8',
			],
			'turkish' => [
				'national' => 'Turkish',
				'charset' => 'ISO-8859-9',
			],
			'nordic' => [
				'national' => 'Nordic',
				'charset' => 'ISO-8859-10',
			],
			'thai' => [
				'national' => 'Thai',
				'charset' => 'ISO-8859-11',
			],
			'baltic' => [
				'national' => 'baltic',
				'charset' => 'ISO-8859-13',
			],
			'keltic' => [
				'national' => 'keltic',
				'charset' => 'ISO-8859-14',
			],
			'extended_european' => [
				'national' => 'ISO-8859-15',
				'charset' => 'ISO-8859-15',
			],
			'unicode' => [
				'national' => 'Unicode',
				'charset' => 'UTF-8',
			],
			'windows_1251' => [
				'national' => 'Windows-1251',
				'charset' => 'Windows-1251',
			],
			'windows_1252' => [
				'national' => 'Windows-1252',
				'charset' => 'Windows-1252',
			],
		];
	}

	public static function inst(){
		return (self::$inst ?: (self::$inst = new self()));
	}

	/**
	 * @return array
	 * @param $availableChars array
	 * @desc This function returns an key = charset / value = charset - name(international) (name(national)))
	 */
	public function getCharsetsForTagWizzard(){
		$retArr = [];

		foreach($this->charsets as $val){
			$retArr[$val['charset']] = $val['charset'] . ' - ' . g_l('charset', '[' . $val['charset'] . ']') . ' (' . $val['national'] . ')';
		}
		return $retArr;
	}

	/**
	 * @return array|false
	 * @param string $charset
	 * @desc returns mixed (national, international, charset, when charset is known)
	 */
	public function getCharsetArrByCharset($charset){
		$charset = strtolower($charset);
		foreach($this->charsets as $val){
			if(strtolower($val['charset']) === $charset){
				return $val;
			}
		}
		return false;
	}

	/**
	 * @return array
	 * @param $availableChars array
	 * @desc This function returns an array for the property page of a webEdition document
	 */
	public function getCharsetsByArray($availableChars){
		$tmpCharArray = [];
		$retArr = [];

		foreach($availableChars as $char){
			$tmpCharArray[] = ($this->getCharsetArrByCharset($char) ?: ['charset' => $char]);
		}

		foreach($tmpCharArray as $val){
			$retArr[$val['charset']] = $val['charset'] . ' - ' . g_l('charset', '[' . $val['charset'] . ']') . ' (' . $val['national'] . ')';
		}

		return $retArr;
	}

	//FIXME: use array obove; currently this seems to complecated
	public static function getAvailCharsets(){
		return self::CHARSETS;
	}

	public static function isValidCharset($cs){
		return in_array($cs, self::CHARSETS);
	}

}
