<?php

/**
 * webEdition CMS
 *
 * $Rev: 63cf82661a98 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-04-09 01:12 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* a class for handling directories */
class we_folder extends we_contents_root{
	const ACTION_DELETE = 'delete';
	const ACTION_UNPUBLISH = 'unpublish';
	const ACTION_PUBLISH = 'publish';
	const ACTION_UNSEARCHABLE = 'unsearchable';
	const ACTION_SEARCHABLE = 'searchable';
	const ACTION_COPY_CHARSET = 'copychar';
	const ACTION_COPY_WS = 'copyws';
	const ACTION_COPY_TID = 'copytid';
	const ACTION_CATEGORY_ADD = 'add_category';
	const ACTION_CATEGORY_DELETE = 'delete_category';

	/* Flag which is set, when the file is a folder  */
	var $WorkspacePath = '';
	var $WorkspaceID = '';
	var $GreenOnly = 0;
	var $searchclassFolder;
	protected $viewType = 'list';
	protected $doclistModel = null;
	//folders are always published
	public $Published = PHP_INT_MAX;
	protected $urlMap;

	/**
	 * @var we_customer_documentFilter
	 */
	var $documentCustomerFilter = ''; // DON'T SET TO NULL !

	/* Constructor */

	function __construct(){
		$this->IsFolder = 1;
		parent::__construct();
		array_push($this->persistent_slots, 'SearchStart', 'SearchField', 'Search', 'Order', 'GreenOnly', 'IsClassFolder', 'WorkspacePath', 'WorkspaceID', 'Language', 'TriggerID', 'urlMap', 'doclistModel', 'viewType');
		if(isWE()){
			if($this->ID){
				$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO]);
			} else {
				$this->EditPageNrs = [we_base_constants::WE_EDITPAGE_PROPERTIES];
			}
		}
		$this->Table = FILE_TABLE;
		$this->ContentType = we_base_ContentTypes::FOLDER;
	}

	public function we_new($table = '', $parentID = 0, $name = ''){
		$this->ID = 0;
		$this->OldPath = '';
		if($table){
			$this->Table = $table;
			$this->ParentID = $parentID;
			$this->Filename = $name;
			$this->Text = $name;
			$this->Path = $this->getPath();
			$this->Published = time();
		}
		parent::we_new();
		$this->adjustEditPageNr();
	}

	function getPath(){
		switch($this->Table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
				return parent::getPath();
			default:
				return rtrim($this->getParentPath(), '/') . '/' . $this->Text;
		}
	}

	public function we_initSessDat(array $sessDat){
		we_contents_root::we_initSessDat($sessDat);

		if($this->Table === FILE_TABLE || $this->Table === OBJECT_FILES_TABLE){
			if(!$this->Language){
				$this->initLanguageFromParent();
			}
			if(we_base_request::_(we_base_request::BOOL, 'we_edit_weDocumentCustomerFilter')){
				$this->documentCustomerFilter = we_customer_documentFilter::getCustomerFilterFromRequest($this->ID, $this->ContentType, $this->Table);
			} else if(isset($sessDat[3])){ // init webUser from session - only for old temporary documents
				$this->documentCustomerFilter = $sessDat[3];
			}
		}
		$this->adjustEditPageNr();

		if($this->ID){
			if(!$this->doclistModel){
				$this->doclistModel = new we_doclist_model($GLOBALS['we_transaction'], $this->Table, $this->ID, $this->viewType);
			}
			$this->doclistModel->initByHttp();
		}
	}

	public function getDoclistModel(){
		return $this->doclistModel ?: ($this->ID ? new we_doclist_model(0, $this->ID, $this->viewType) : null);
	}

	/**
	 * adjust EditPageNrs for CUSTOMERFILTER AND DOCLIST
	 */
	function adjustEditPageNr(){
		if(!isWE()){
			return;
		}
		if($this->ID){
			$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_PROPERTIES, we_base_constants::WE_EDITPAGE_INFO]);
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER, we_base_perm::CUSTOMER_CAN_CHANGE_DOCS])){
			switch($this->Table){
				case FILE_TABLE:
				case OBJECT_FILES_TABLE:
					if($this->ID){
						$this->EditPageNrs = array_merge($this->EditPageNrs, [we_base_constants::WE_EDITPAGE_WEBUSER]);
					}
			}
		}
		switch($this->Table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
			case VFILE_TABLE:
				if($this->ID){
					$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_DOCLIST;
				}
		}
	}

	private function initLanguageFromParent(){
		$ParentID = $this->ParentID;
		$i = 0;
		while(!$this->Language){
			if($ParentID == 0 || $i > 20){
				$this->Language = $GLOBALS['we']['lang']['DefaultFrontendLanguage'];
				if(!$this->Language){
					$this->Language = 'de_DE';
				}
			} else {
				$this->DB_WE->query('SELECT Language, ParentID FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID = ' . intval($ParentID));
				while($this->DB_WE->next_record()){
					$ParentID = $this->DB_WE->f('ParentID');
					$this->Language = $this->DB_WE->f('Language');
				}
			}
			$i++;
		}
	}

	/**
	 * Initialize a folder by its path. Creates the folder and its parents if it does not exist yet.
	 * @param  string  $path          The path to initialize
	 * @param  string  $tblName       The table to store the folder in (default: FILE_TABLE)
	 * @return boolean                Returns true if the folder could be initialized.
	 */
	public function initByPath($path, $tblName = FILE_TABLE){
		// remove tailing '/'
		$path = rtrim($path, '/');

		// Try to retieve the given folder from the database
		$id = f('SELECT ID FROM ' . $this->DB_WE->escape($tblName) . ' WHERE Path="' . $this->DB_WE->escape($path) . '" AND IsFolder=1', '', $this->DB_WE);
		if($id != ''){
			// Folder exists: use initByID() to initialize the object.
			$this->initByID($id, $tblName);
			if($this->Table === OBJECT_FILES_TABLE){
				$this->ClassName = 'we_class_folder';
			}
		} else {
			// Folder does not exist: create it (if user has permissons to create folders).
			// Iterate the given path and create non-existent folders on-the-fly.
			$spl = explode('/', $path);
			$folderName = array_pop($spl);
			$p = [];
			$anz = count($spl);
			$last_pid = 0;
			while($spl){
				array_push($p, array_shift($spl));
				$pa = implode('/', $p);
				if($pa){
					if(($pid = f('SELECT ID FROM ' . $this->DB_WE->escape($tblName) . ' WHERE Path="' . $this->DB_WE->escape($pa) . '"', '', $this->DB_WE))){
						$last_pid = $pid;
					} else {
						$folder = ($this->Table === OBJECT_FILES_TABLE ?
							new we_class_folder() : new self());

						$folder->we_new($tblName, $last_pid, end($p));
						$folder->IsClassFolder = $last_pid == 0;
						$folder->save();
						$last_pid = $folder->ID;
					}
				}
			}

			// Eventually initialize the folder object
			$this->we_new($tblName, $last_pid, $folderName);
			$this->IsClassFolder = $last_pid == 0;
			$this->save();
		}
		return true;
	}

	protected function isSameAsParent(){
		if($this->ID){
			$pid = $this->ParentID;
			while($pid){
				if($this->ID == $pid){
					return true;
				}
				$pid = f('SELECT ParentID FROM ' . $GLOBALS['DB_WE']->escape($this->Table) . '  WHERE ID=' . intval($pid));
			}
		}
		return false;
	}

	/* saves the folder */

	public function we_save($resave = false, $skipHook = false){
		$this->setText();
		$collectionFolder = $this->Table === VFILE_TABLE;
		$objFolder = ($this->Table === OBJECT_FILES_TABLE);
		if($objFolder){
			$this->ClassName = 'we_class_folder';
		}
		$update = $this->isMoved();
		$tmp = $this->Path;

		if($update && !$objFolder && !$collectionFolder){
			if(file_exists($this->OldPath) && file_exists($this->Path)){
				t_e('Both paths exists!', $this->OldPath, $this->Path);
				return false;
			}
			//leave old dir for parent save
			$this->Path = $this->OldPath;
		}

		if(!parent::we_save($resave)){
			return false;
		}
		$this->Path = $tmp;

		if(!$this->writeFolder()){
			return false;
		}

		$this->OldPath = $this->Path;
		if($this->Table === OBJECT_TABLE){
			$f = new we_class_folder();
			$f->initByPath($this->Path, OBJECT_FILES_TABLE, true);
		}
		$this->resaveWeDocumentCustomerFilter();

		if(!$resave && $update){
			//FIXME:improve!
			we_navigation_cache::clean(true);
		}

		if(LANGLINK_SUPPORT && $this->Table === FILE_TABLE){
			$out = $this->setLanguageLink();
		}
		/* hook */
		if(!$skipHook){
			$hook = new we_hook_base('save', '', [$this, 'resave' => $resave]);
			//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}

			//check for warnings and notices
			if(($tmp = $hook->getWarningString())){
				$this->warningMsg[] = $tmp;
			}
			if(($tmp = $hook->getNoticeString())){
				$this->noticeMsg[] = $tmp;
			}
		}
		return true;
	}

	protected function saveLangLinkDocuments($withBackLink = true){
		return parent::saveLangLinkDocuments(false);
	}

	protected function setLanguageLinkBacklinks($docTable, array $langlinks){
		//don't add backlinks
	}

	public static function changeRecursiveRequest($what){
		$ok = false;
		$lang = 'grant_language';
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 1);
		// init document
		$we_dt = $_SESSION['weS']['we_data'][$we_transaction];

		$we_doc = we_contents_document::initDoc($we_dt);
		switch($what){
			case 'changeLanguageRecursive':
				$ok = $we_doc->changeLanguageRecursive();
				$lang = 'grant_language';
				break;
			case 'changeTriggerIDRecursive':
				$ok = (we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ?
					$we_doc->changeTriggerIDRecursive() :
					false);
				$lang = 'grant_tid';
				break;
		}
		$jsCmd = new we_base_jsCmd();
		if($ok){
			$jsCmd->addMsg(g_l('weClass', '[' . $lang . '][ok]'), we_base_util::WE_MESSAGE_NOTICE);
		} else {
			$jsCmd->addMsg(g_l('weClass', '[' . $lang . '][notok]'), we_base_util::WE_MESSAGE_ERROR);
		}

		echo we_html_tools::getHtmlTop('', '', '', $jsCmd->getCmds(), we_html_element::body());
	}

	private function changeLanguageRecursive(){
		$DB_WE = $this->DB_WE;

		$language = $this->Language;
		$documentTable = stripTblPrefix($this->Table);

		// Adapt tblLangLink-entries of documents and objects to the new language
		$docIds = $DB_WE->getAllq('SELECT ID FROM ' . $DB_WE->escape($this->Table) . ' WHERE Path LIKE "' . $DB_WE->escape($this->Path) . '/%" AND Language!="' . $DB_WE->escape($language) . '"', true);
		if(!$docIds){
			return;
		}
		// First: Change language of published documents, objects
		$DB_WE->query('UPDATE ' . $DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter([
				'Language' => $this->Language
			]) . ' WHERE
	ID IN (' . implode(',', $docIds) . ')');

		//update target lang, if applicable, if not, the old lang is kept. This is ok, since this operation will cause big trouble if tried to change things recursive
		$DB_WE->query('UPDATE IGNORE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
				'Locale' => $this->Language
			]) . ' WHERE DocumentTable="' . $DB_WE->escape($documentTable) . '" AND LDID IN (' . implode(',', $docIds) . ')');
	}

	private function changeTriggerIDRecursive(){
		// Change TriggerID of documents
		return $this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET TriggerID=' . intval($this->TriggerID) . ' WHERE Path LIKE "' . $this->DB_WE->escape($this->Path) . '/%" ContentType IN ("' . implode('","', [
					we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML, we_base_ContentTypes::OBJECT_FILE]) . '")');
	}

	protected function setText(){
		$this->Text = in_array($this->Table, [FILE_TABLE, TEMPLATES_TABLE]) ? $this->Filename : $this->Text;
	}

	/*
	 * not needed if blank and special chars are alloud for collection folders
	  protected function isFilenameNotValid(){
	  switch($this->Table){
	  case VFILE_TABLE:
	  $this->Filename = $this->Text;
	  break;
	  default:
	  // do nothing
	  }

	  return parent::isFilenameNotValid();
	  }
	 */

	protected function isFilenameDouble(){
		return f('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE Path="' . $this->DB_WE->escape($this->Path) . '" AND ID!=' . intval($this->ID) . ' LIMIT 1', '', $this->DB_WE);
	}

	protected function isFilenameEmpty(){
		switch($this->Table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
				return $this->Filename === '';
			default:
				return $this->Text === '';
		}
	}

	function userCanSave($ctConditionOk = false){
		switch($this->Table){
			case VFILE_TABLE:
				return we_base_permission::hasPerm(we_base_perm::WORK_COLLECTION_FOLDER_NEW) && parent::userCanSave(true);
			default:
				return parent::userCanSave($ctConditionOk);
		}
	}

	/* returns 0 because it is a directory */

	function getfilesize(){
		return 0;
	}

	/* must be called from the editor-script. Returns a filename which has to be included from the global-Script */

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			default:
				$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				return new we_editor_info($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_DOCLIST:
				return new we_editor_doclist($this, $jsCmd);
		}
	}

	function formPath($disablePath = false, $notSetHot = false, $extra = ''){
		$ws = get_ws($this->Table, true);
		if(intval($this->ParentID) == 0 && $ws){
			$this->ParentID = reset($ws);
			$this->ParentPath = id_to_path($this->ParentID, $this->Table, $this->DB_WE);
		}

		$userCanChange = we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENT_FOLDER_PATH_CHANGE) || ($this->CreatorID == $_SESSION['user']['ID']) || (!$this->ID);
		if($this->ID != 0 && $this->ParentID == 0 && $this->ParentPath === '/' && $this->Table === OBJECT_FILES_TABLE){
			$userCanChange = false;
		}
		return (!$userCanChange ? '<table class="default"><tr><td>' . $this->Path . '</td></tr>' :
			'<table class="default">
<colgroup><col style="width:20px;"/><col style="width:20px;"/><col style="width:100px;"/></colgroup>
	<tr><td style="padding-bottom:2ex;">' . $this->formInputField('', (in_array($this->Table, [FILE_TABLE, TEMPLATES_TABLE])) ? 'Filename' : 'Text', g_l('weClass', '[foldername]'), 0, 255, [
				'onchange' => "pathOfDocumentChanged(true);"]) . '</td><td></td><td></td></tr>
	<tr><td colspan="3">' . $this->formDirChooser() . '</td></tr>' .
			($this->Table === OBJECT_FILES_TABLE ? '
	<tr><td colspan="3" style="padding-top:4px;">' . $this->formTriggerDocument() . '</td></tr>
	<tr><td colspan="3">
		<table class="default"><tr><td>' . we_html_tools::alertAttentionBox(g_l('weClass', '[grant_tid][expl]') . ($this->ID ? '' : g_l('weClass', '[availableAfterSave]')), we_html_tools::TYPE_INFO, 0, false) . '</td><td>' .
			we_html_button::create_button(we_html_button::OK, 'javascript:if(_EditorFrame.getEditorIsHot()) {WE().util.showMessage(WE().consts.g_l.alert.saveFirstMessage,WE().consts.message.WE_MESSAGE_ERROR, window);' . " } else {;we_cmd('changeTriggerIDRecursive','" . $GLOBALS['we_transaction'] . "');}", '', '', ($this->ID ? false : true)) . '</td></tr>
					</table></td></tr>' :
			'') .
			($this->Table === FILE_TABLE && $this->ID && we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? '
	<tr><td style="padding-top:2ex;">' . $this->formInputField('', 'urlMap', g_l('weClass', '[urlMap]'), 0, 255, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . '</td><td></td><td></td></tr>
' : '')) .
			'</table>';
	}

	function formChangeOwners(){
		$disabledNote = ($this->ID ? '' : ' ' . g_l('weClass', '[availableAfterSave]'));

		return '<table class="default"><tr><td>' . we_html_tools::alertAttentionBox(g_l('modules_users', '[grant_owners_expl]') . $disabledNote, we_html_tools::TYPE_INFO, 390, false) . '</td><td>' .
			we_html_button::create_button(we_html_button::OK, 'javascript:if(_EditorFrame.getEditorIsHot()) {WE().util.showMessage(WE().consts.g_l.alert.input_file_name,WE().consts.message.WE_MESSAGE_ERROR, window);' . " } else {;we_cmd('users_changeR','" . $GLOBALS['we_transaction'] . "');}", '', '', !empty($disabledNote)) . '</td></tr>
					</table>';
	}

	function formChangeLanguage(){
		$disabledNote = ($this->ID ? '' : ' ' . g_l('weClass', '[availableAfterSave]'));

		return '<table class="default"><tr><td>' . we_html_tools::alertAttentionBox(g_l('weClass', '[grant_language][expl]') . $disabledNote, we_html_tools::TYPE_INFO, 390, false) . '</td><td>' .
			we_html_button::create_button(we_html_button::OK, "javascript:if(_EditorFrame.getEditorIsHot()) { WE().util.showMessage(WE().consts.g_l.alert.input_file_name,WE().consts.message.WE_MESSAGE_ERROR, window); } else {we_cmd('changeLanguageRecursive','" . $GLOBALS['we_transaction'] . "');}", '', '', !empty($disabledNote)) . '</td></tr>
</table>';
	}

	protected function formCopyDocument(){
		$idname = 'we_' . $this->Name . '_CopyID';
		$parents = [0, $this->ID];
		we_getParentIDs(FILE_TABLE, $this->ID, $parents);
		$disabledNote = ($this->ID ? '' : ' ' . g_l('weClass', '[availableAfterSave]'));
		$but = we_html_button::create_button(we_html_button::SELECT, ($this->ID ?
				"javascript:we_cmd('we_selector_directory', document.we_form.elements['" . $idname . "'].value, '" . $this->Table . "', '" . $idname . "', '', 'copyFolderCheck," . $this->ID . "," . $this->Table . "," . implode(',', $parents) . "')" :
				"javascript:WE().util.showMessage(WE().consts.g_l.alert.copy_folders_no_id,WE().consts.message.WE_MESSAGE_ERROR, window);"), '', '', !empty($disabledNote));

		return '<table class="default"><tr><td>' . we_html_tools::alertAttentionBox(g_l('weClass', '[copy_owners_expl]') . $disabledNote, we_html_tools::TYPE_INFO, 400, false) . '</td><td>' .
			we_html_element::hidden($idname, $this->CopyID) . $but . '</td></tr>
					</table>';
	}

	protected function writeFolder($pub = 0){
		if($this->Path == $this->OldPath || !$this->OldPath){
			return $this->saveToServer();
		}
		if(!$this->moveAtServer()){
			return false;
		}
		$this->modifyLinks();
		$this->modifyChildrenPath();

		$this->OldPath = $this->Path;
		return true;
	}

	protected function modifyLinks(){
		switch($this->Table){
			case FILE_TABLE:
			case TEMPLATES_TABLE:
			case OBJECT_FILES_TABLE:
				$this->DB_WE->query('UPDATE ' . $this->DB_WE->escape($this->Table) . ' SET Path=CONCAT("' . $this->DB_WE->escape($this->Path) . '",SUBSTRING(Path,' . (strlen($this->OldPath) + 1) . ')) WHERE Path LIKE "' . $this->DB_WE->escape($this->OldPath) . '/%" OR Path="' . $this->DB_WE->escape($this->OldPath) . '"');
		}
	}

	public function modifyChildrenPath(){
		@ignore_user_abort(true);
		$DB_WE = new DB_WE();
		$entries = $this->DB_WE->getAllFirstq('SELECT ID,ContentType FROM ' . $DB_WE->escape($this->Table) . ' WHERE ParentID=' . intval($this->ID), false);
		foreach($entries as $id => $ct){
			update_time_limit(30);

			if($ct){
				$we_doc = we_base_ContentTypes::getObject($ct);
				$we_doc->initByID($id, $this->Table, self::LOAD_TEMP_DB);
				$we_doc->ModifyPathInformation($this->ID);
			} else {
				t_e('No class set at entry ', $id, $this->Table);
			}
		}
		@ignore_user_abort(false);
	}

	/* for internal use */

	private function moveAtServer(){
		if($this->Table === FILE_TABLE || $this->Table === TEMPLATES_TABLE){

			// renames the folder on the local machine in the root-dir
			$path = $this->getRealPath();
			$oldPath = $this->getRealPath(true);
			if(!file_exists($path) && !file_exists($oldPath)){
				t_e('old path doesn\'t exist', $oldPath);
				return false;
			}
			if($this->Table === FILE_TABLE){
				// renames the folder on the local machine in the root-dir+site-dir
				$sitepath = $_SERVER['DOCUMENT_ROOT'] . SITE_DIR . substr($this->Path, 1);
				$siteoldPath = $_SERVER['DOCUMENT_ROOT'] . SITE_DIR . substr($this->OldPath, 1);
				if(file_exists($sitepath) && file_exists($siteoldPath)){
					t_e('old and new dir exists!', $sitepath, $siteoldPath);
					return false;
				}
				if(!file_exists($sitepath) && !file_exists($siteoldPath)){
					t_e('old directory doesn\'t exist!', $oldPath);
					return false;
				}
			}

			if(!file_exists($path) && file_exists($oldPath)){
				if(!rename($oldPath, $path)){
					return false;
				}
			}

			if($this->Table !== TEMPLATES_TABLE){
				//we are responsible for site dir!
				if(!file_exists($sitepath) && file_exists($siteoldPath)){
					if(!rename($siteoldPath, $sitepath)){
						//move back other dir!
						rename($path, $oldPath);
						return false;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Create the folders on the server in the public and the site directory.
	 * Note: for internal use
	 *
	 * @return boolean true on success, false in case of failure
	 */
	private function saveToServer(){
		$isTemplFolder = false;
		switch($this->Table){
			case TEMPLATES_TABLE:
				$isTemplFolder = true;
			case FILE_TABLE:
				$path = $this->getPath();
				// creates the folder on the local machine in the root-dir
				if(!we_base_file::createLocalFolderByPath(($isTemplFolder ? TEMPLATES_PATH : realpath(WEBEDITION_PATH . '..')) . $path)){
					return false;
				}
				if(!$isTemplFolder && $this->urlMap){
					we_base_file::makeSymbolicLink(WEBEDITION_PATH, $_SERVER['DOCUMENT_ROOT'] . $path . rtrim(WEBEDITION_DIR, '/'));
					//make sure we have a symbolic dir
					we_base_file::checkAndMakeFolder($_SERVER['DOCUMENT_ROOT'] . WE_THUMBNAIL_DIRECTORY);
					if(WE_THUMBNAIL_DIRECTORY){
						we_base_file::makeSymbolicLink($_SERVER['DOCUMENT_ROOT'] . WE_THUMBNAIL_DIRECTORY, $_SERVER['DOCUMENT_ROOT'] . $path . WE_THUMBNAIL_DIRECTORY);
					}
				}

				// creates the folder on the local machine in the root-dir+site-dir
				if(!$isTemplFolder && !we_base_file::createLocalFolderByPath($_SERVER['DOCUMENT_ROOT'] . SITE_DIR . $path)){
					return false;
				}
			default:
				return true;
		}
	}

	/**
	 * Beseitigt #Bug 3705: sorgt daf�r, das auch leere Dokumentenordner bei einem REbuild angelegt werden
	 */
	public function we_rewrite(){
		return (parent::we_rewrite() ? ($this->Table === FILE_TABLE ? $this->we_save(true) : true) : false);
	}

	/**
	 * @desc	the function modifies document EditPageNrs set
	 */
	function checkTabs(){

	}

	public static function getUrlReplacements(we_database_base $db, $onlyUrl = false, $hostMatch = false){
		static $ret = -1;
		if($ret == -1){
			$ret = ['full' => [], 'url' => [], 'full_host' => [], 'url_host' => [],];
			$db->query('SELECT Path,urlMap FROM ' . FILE_TABLE . ' WHERE urlMap!="" ORDER BY Path DESC');
			$lastRules = [];
			while($db->next_record(MYSQLI_ASSOC)){
				$host = trim(str_replace(['https://', 'http://'], '', $db->f('urlMap')), '/');
				$rep1 = '-((href\s*=|src\s*=|action\s*=|location\s*=|content\s*=|url)\s*["\'\(])(' . preg_quote($db->f('Path'), '-') . ')(/[^"\'\)]*["\'\)])-';
				$rep2 = '-^' . preg_quote($db->f('Path'), '-') . '(/.*)-';
				if($_SERVER['SERVER_NAME'] == $host){
					//this must be at the end, since duplicate replacements may need to match the original string
					$lastRules['full']['${1}${4}'] = $rep1;
					$lastRules['url']['${1}'] = $rep2;
				} else {
					$ret['full']['${1}//' . $host . '${4}'] = $rep1;
					$ret['url']['//' . $host . '${1}'] = $rep2;
				}
				$ret['full_host']['${1}' . '//' . $host . '${4}'] = $rep1;
				$ret['url_host']['//' . $host . '${1}'] = $rep2;
			}
			if($lastRules){
				$ret['full'] = array_merge($ret['full'], $lastRules['full']);
				$ret['url'] = array_merge($ret['url'], $lastRules['url']);
			}
		}
		return $ret[($onlyUrl ? 'url' : 'full') . ($hostMatch ? '_host' : '')];
	}

	public static function getUrlFromID($id){
		if(!$id){
			return '';
		}
		$replace = self::getUrlReplacements($GLOBALS['DB_WE'], true, true);
		$path = f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id));
		$cnt = 0;
		$ret = $replace ?
			preg_replace($replace, array_keys($replace), $path, 1, $cnt) :
			$path;
		return ($cnt ? getServerProtocol() . ':' : '') . $ret;
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd){
		$parts = [
			[
				'icon' => we_html_multiIconBox::PROP_PATH,
				'headline' => g_l('weClass', '[path]'),
				'html' => $this->formPath()
			]
		];

		if((count($GLOBALS['we']['lang']['FrontendLanguages']) > 1)){
			$withLanglinks = false;
			switch($this->Table){
				case FILE_TABLE:
					$withLanglinks = true;
				case OBJECT_FILES_TABLE:
					//no langlinks on objectfolders
					$parts[] = [
						'icon' => we_html_multiIconBox::PROP_LANG,
						'headline' => g_l('weClass', '[language]'),
						'html' => $this->formLangLinks($withLanglinks) .
						we_html_element::br() .
						we_html_element::div(['class' => "weMultiIconBoxHeadline"], g_l('weClass', '[grant_language][headline]')) .
						$this->formChangeLanguage(),
						'fold' => true
					];
			}
		}
		$ok = false;
		switch($this->Table){
			case FILE_TABLE:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_FOLDERS_COPY)){
					break;
				}
				$ok = true;
			case OBJECT_FILES_TABLE :
				$ok = $ok || we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_COPY);
				if(!$ok){
					break;
				}
				$parts[] = [
					'icon' => we_html_multiIconBox::PROP_COPY,
					'headline' => g_l('weClass', '[copyFolder]'),
					'html' => $this->formCopyDocument(),
				];
		}

		switch($this->Table){
			case FILE_TABLE:
			case OBJECT_FILES_TABLE:
				$parts[] = [
					'icon' => we_html_multiIconBox::PROP_USER,
					'headline' => g_l('weClass', '[owners]'),
					'html' => $this->formCreatorOwners($jsCmd) . '<br/>' .
					(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ?
					we_html_element::div(['class' => 'weMultiIconBoxHeadline'], g_l('modules_users', '[grant_owners]')) .
					$this->formChangeOwners() : ''),
					'fold' => true
				];
		}

		return we_html_multiIconBox::getHTML('PropertyPage', $parts, true);
	}

	protected function saveContentDataInDB(){
		return true;
	}

}
