<?php
/**
 * webEdition CMS
 *
 * $Rev: dbcab2e6dcdb $
 * $Author: mokraemer $
 * $Date: 2019-04-08 16:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
require_once(WE_INCLUDES_PATH . 'we_editors/we_preferences_config.inc.php');

//NOTE: only add "newConf" to entries set in $GLOBALS['configs']. All "temporary" entries should remain in main-Request-Scope

$weSuggest = &we_gui_suggest::getInstance();

abstract class we_editor_preferences{
	private static $editor_reloaded = false;
	private static $jsCmd = null;

	const TABS = [
		'ui' => ['fa-tv', ''],
//	'extensions' => 'EDIT_SETTINGS_DEF_EXT',
		'editor' => ['fa-edit', [we_base_perm::ADMINISTRATOR, we_base_perm::WORK_TEMPLATES_CAN_SEE, we_base_perm::WORK_CSS_NEW, we_base_perm::WORK_JS_NEW]],
		'seolinks' => ['fa-search', we_base_perm::ADMINISTRATOR],
		'language' => ['fa-language', we_base_perm::ADMINISTRATOR],
		'countries' => ['fa-flag-checkered', we_base_perm::ADMINISTRATOR],
		'error_handling' => ['fa-bug', we_base_perm::ADMINISTRATOR],
//	'validation' => array('fa-check', we_base_perm::ADMINISTRATOR),
		'email' => ['fa-envelope-o', we_base_perm::ADMINISTRATOR],
		//'message_reporting' => '',
		'recipients' => ['fa-envelope-o', we_base_perm::WORK_FORMMAIL_SHOW],
		'versions' => ['fa-history', we_base_perm::ADMINISTRATOR],
		'defaultAttribs' => ['fa-tags', we_base_perm::ADMINISTRATOR],
		'advanced' => ['fa-wrench', we_base_perm::ADMINISTRATOR],
		'system' => ['fa-cogs', we_base_perm::ADMINISTRATOR],
		//'cache' => ['fa-hdd-o', we_base_perm::ADMINISTRATOR],
		'security' => ['fa-key', we_base_perm::ADMINISTRATOR],
		'proxy' => ['fa-bomb', we_base_perm::ADMINISTRATOR],
		'modules' => ['fa-cubes', we_base_perm::ADMINISTRATOR],
	];

	/**
	 * This function returns the HTML code of a dialog.
	 *
	 * @param          string                                  $name
	 * @param          string                                  $title
	 * @param          array                                   $content
	 * @param          int                                     $expand             (optional)
	 * @param          string                                  $show_text          (optional)
	 * @param          string                                  $hide_text          (optional)
	 * @param          bool                                    $cookie             (optional)
	 * @param          string                                  $JS                 (optional)
	 *
	 * @return         string
	 */
	private static function create_dialog($name, array $content, $expand = -1, $show_text = '', $hide_text = ''){
		$output = ($expand != -1 ? we_html_element::jsScript(JS_DIR . 'multiIconBox.js') : '');

		// Return HTML code of dialog
		return $output . we_html_multiIconBox::getHTML($name, $content, true, '', $expand, $show_text, $hide_text);
	}

	/**
	 * This functions return either the saved option or the changed one.
	 *
	 * @param          string                                  $settingvalue
	 *
	 * @see            return_value()
	 *
	 * @return         array|string|int
	 */
	private static function get_value($settingname){
		$all = explode('-', $settingname);
		$settingname = $all[0];
		switch($settingname){
			case 'specify_jeditor_colors':
				return (isset($_SESSION['prefs'][$settingname]) ? $_SESSION['prefs'][$settingname] : 1);

			case 'seem_start_type':
				if(($_SESSION['prefs']['seem_start_type'] === 'document' || $_SESSION['prefs']['seem_start_type'] === 'object') && $_SESSION['prefs']['seem_start_file'] == 0){
					return 'cockpit';
				}
				return $_SESSION['prefs']['seem_start_type'];

			case 'locale_locales':
				return getWeFrontendLanguagesForBackend();

			case 'locale_default':
				return $GLOBALS['we']['lang']['DefaultFrontendLanguage'];

			case 'proxy_proxy':
				// Check for settings file
				if(file_exists(WEBEDITION_PATH . 'liveUpdate/includes/proxysettings.inc.php')){
					require_once(WEBEDITION_PATH . 'liveUpdate/includes/proxysettings.inc.php');
				}
				return defined('WE_PROXYHOST');

			case 'message_reporting':
				return (!empty($_SESSION['prefs']['message_reporting'])) ? $_SESSION['prefs']['message_reporting'] : (we_base_util::WE_MESSAGE_ERROR + we_base_util::WE_MESSAGE_WARNING + we_base_util::WE_MESSAGE_NOTICE);

			default:
				if(isset($GLOBALS['configs']['user'][$settingname])){
					if(isset($all[1])){
						//handle subkey
						$tmp = we_unserialize(isset($_SESSION['prefs'][$settingname]) ? $_SESSION['prefs'][$settingname] : $GLOBALS['configs']['user'][$settingname][0]);
						return isset($tmp[$all[1]]) ? $tmp[$all[1]] : 0;
					}
					return (isset($_SESSION['prefs'][$settingname]) ? $_SESSION['prefs'][$settingname] : $GLOBALS['configs']['user'][$settingname][0]);
				}

				//if not found in global_config or other config - simply return '' - this should not happen - should we return something more error-specific?
				return defined($settingname) ?
					constant($settingname) :
					(isset($GLOBALS['configs']['global'][$settingname]) ? $GLOBALS['configs']['global'][$settingname][2] :
					(isset($GLOBALS['configs']['other'][$settingname]) ? $GLOBALS['configs']['other'][$settingname][1] :
					'')
					);
		}
	}

	private static function getYesNoSelect($setting){
		$select = new we_html_select(['name' => 'newconf[' . $setting . ']', 'class' => 'weSelect']);
		$select->addOption(0, g_l('prefs', '[no]'));
		$select->addOption(1, g_l('prefs', '[yes]'));
		$select->selectOption(self::get_value($setting) ? 1 : 0);

		return $select->getHtml();
	}

	private static function getTrueFalseSelect($setting){
		$select = new we_html_select(['name' => 'newconf[' . $setting . ']', 'class' => 'weSelect']);
		$select->addOption(0, 'false');
		$select->addOption(1, 'true');
		$select->selectOption(self::get_value($setting) ? 1 : 0);

		return $select->getHtml();
	}

	/**
	 * This functions saves an option in the current session.
	 *
	 * @param          string                                  $settingvalue
	 * @param          string                                  $settingname
	 *
	 * @see            save_all_values
	 * @see            we_base_preferences::changeSourceCode()
	 *
	 * @return         void
	 */
	private static function remember_value($settingvalue, $settingname, $comment = ''){
		$DB_WE = $GLOBALS['DB_WE'];
		if(isset($GLOBALS['configs']['user'][$settingname]) && $settingvalue == null){ //checkboxes -> unchecked - all other values are set by the form
			$settingvalue = 0;
		}

		//check for user-setting
		switch($settingname){
			default:
				if(isset($GLOBALS['configs']['user'][$settingname])){
					$_SESSION['prefs'][$settingname] = ($settingvalue == null ? 0 : $settingvalue);
				} else {
					$file = &$GLOBALS['config_files']['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;
			case 'seem_start_file'://don't do anything here
				return;
			case 'seem_start_type':
				switch($settingvalue){
					case 'document':
						$tmp = $_SESSION['prefs']['seem_start_file'] = we_base_request::_(we_base_request::INT, 'seem_start_document');
						$_SESSION['prefs'][$settingname] = ($tmp ? $settingvalue : 'cockpit');
						break;
					case 'object':
						$tmp = $_SESSION['prefs']['seem_start_file'] = we_base_request::_(we_base_request::INT, 'seem_start_object');
						$_SESSION['prefs'][$settingname] = ($tmp ? $settingvalue : 'cockpit');
						break;
					default:
						$_SESSION['prefs'][$settingname] = $settingvalue;
						break;
				}
				return;
			case 'sizeOpt':
				if($settingvalue == 0){
					$_SESSION['prefs']['weWidth'] = 0;
					$_SESSION['prefs']['weHeight'] = 0;
					$_SESSION['prefs']['sizeOpt'] = 0;
				} else if(($settingvalue == 1) && we_base_request::_(we_base_request::INT, 'newconf', false, 'weWidth') !== false && we_base_request::_(we_base_request::INT, 'newconf', false, 'weHeight') !== false){
					$_SESSION['prefs']['sizeOpt'] = 1;
				}

				return;

			case 'weWidth':
				if($_SESSION['prefs']['sizeOpt'] == 1){
					$generate_java_script = ($_SESSION['prefs']['weWidth'] != $settingvalue);

					$_SESSION['prefs']['weWidth'] = $settingvalue;

					if($generate_java_script){
						$height = we_base_request::_(we_base_request::INT, 'newconf', 0, "weHeight");
						self::$jsCmd->addCmd('setNewWESize', $settingvalue, $height);
					}
				}
				return;

			case 'weHeight':
				if($_SESSION['prefs']['sizeOpt'] == 1){
					$_SESSION['prefs'][$settingname] = $settingvalue;
				}
				return;

			case 'editorFont':
				if(intval($settingvalue) == 0){
					$_SESSION['prefs']['editorFontname'] = 'none';
					$_SESSION['prefs']['editorFontsize'] = -1;
					$_SESSION['prefs']['editorFont'] = 0;
				} else if(($settingvalue == 1) && we_base_request::_(we_base_request::STRING, 'newconf', '', 'editorFontname') && we_base_request::_(we_base_request::INT, 'newconf', 0, 'editorFontsize')){
					$_SESSION['prefs']['editorFont'] = 1;
				}

				if(!self::$editor_reloaded){
					self::$editor_reloaded = true;

					// editor font has changed - mark all editors to reload!
					self::$jsCmd->addCmd('reloadUsedEditors');
				}
				return;

			case 'editorCodecompletion':
				$_SESSION['prefs'][$settingname] = is_array($settingvalue) ? we_serialize($settingvalue, SERIALIZE_JSON) : '';
				return;
			case 'editorFontname':
			case 'editorFontsize':
				if($_SESSION['prefs']['editorFont'] == 1){
					$_SESSION['prefs'][$settingname] = $settingvalue;
				}

				return;

			case 'editorTooltipFont':
				if(intval($settingvalue) == 0){
					$_SESSION['prefs']['editorTooltipFontname'] = 'none';
					$_SESSION['prefs']['editorTooltipFontsize'] = -1;
					$_SESSION['prefs']['editorTooltipFont'] = 0;
				} else if(($settingvalue == 1) && we_base_request::_(we_base_request::STRING, 'newconf', '', 'editorTooltipFontname') && we_base_request::_(we_base_request::INT, 'newconf', 0, 'editorTooltipFontsize')){
					$_SESSION['prefs']['editorTooltipFont'] = 1;
				}

				if(!self::$editor_reloaded){
					self::$editor_reloaded = true;

					// editor tooltip font has changed - mark all editors to reload!
					self::$jsCmd->addCmd('reloadUsedEditors');
				}

				return;

			case 'Language': //Handle both
				$_SESSION['prefs'][$settingname] = $settingvalue;
				$_SESSION['prefs']['BackendCharset'] = we_base_request::_(we_base_request::STRING, 'newconf', '', 'BackendCharset');


				if($settingvalue != $GLOBALS['WE_LANGUAGE'] || $_SESSION['prefs']['BackendCharset'] != $GLOBALS['we']['Charset']['Backend']){

					// complete webEdition reload: anpassen nach Wegfall der Frames
					self::$jsCmd->addCmd('reloadUsedEditors', 1);
				}

			case 'locale_locales':
				return;
			case 'locale_default':
				if(($loc = we_base_request::_(we_base_request::STRING, 'newconf', '', 'locale_locales')) && ($def = we_base_request::_(we_base_request::STRING, 'newconf', '', 'locale_default'))){
					$locales = explode(',', $loc);
					we_base_preferences::we_writeLanguageConfig($def, $locales);
					//remove langs not used
					$GLOBALS['DB_WE']->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE Locale NOT IN ("' . implode('","', $locales) . '")');
				}
				return;

			case 'WE_COUNTRIES_TOP':
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, implode(',', array_keys(we_base_request::_(we_base_request::INT, 'newconf', 0, 'countries'), 2)), true, $comment);
				return;

			case 'WE_COUNTRIES_SHOWN':
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, implode(',', array_keys(we_base_request::_(we_base_request::INT, 'newconf', 0, 'countries'), 1)), true, $comment);
				return;
			case 'SYSTEM_WE_SESSION_TIME':
				//check, that a session lasts at least 90 seconds, due to we-pings, this is sufficient for WE - other is up to the user.
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, max($settingvalue, 90), true, $comment);
				return;

			case 'WE_SEEM':
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				if(intval($settingvalue) == constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, ($settingvalue == 1 ? 0 : 1), true, $comment);
				}
				return;

			case 'WE_LOGIN_HIDEWESTATUS':
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				if($settingvalue != constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;
			case 'WE_LOGIN_WEWINDOW':
				if(constant($settingname) != $settingvalue){
					$file = &$GLOBALS['config_files']['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			case 'SIDEBAR_DISABLED':
				$file = &$GLOBALS['config_files']['conf_global']['content'];
				if($settingvalue != SIDEBAR_DISABLED){
					$file = we_base_preferences::changeSourceCode('define', $file, 'SIDEBAR_DISABLED', $settingvalue, true, $comment);
				}

				$sidebar_show_on_startup = we_base_request::_(we_base_request::BOOL, 'newconf', false, 'SIDEBAR_SHOW_ON_STARTUP');
				if(SIDEBAR_SHOW_ON_STARTUP != $sidebar_show_on_startup){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_SHOW_ON_STARTUP]', $sidebar_show_on_startup);
				}

				$sidebar_document = we_base_request::_(we_base_request::INT, 'newconf', 0, 'SIDEBAR_DEFAULT_DOCUMENT');
				if(SIDEBAR_DEFAULT_DOCUMENT != $sidebar_document){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_DEFAULT_DOCUMENT]', $sidebar_document);
				}

				$sidebar_width = we_base_request::_(we_base_request::INT, 'newconf', 0, 'SIDEBAR_DEFAULT_WIDTH');
				if(SIDEBAR_DEFAULT_WIDTH != $sidebar_width){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_DEFAULT_WIDTH]', $sidebar_width);
				}

				return;

			case 'DEFAULT_STATIC_EXT':
			case 'DEFAULT_DYNAMIC_EXT':
			case 'DEFAULT_HTML_EXT':
				if(constant($settingname) != $settingvalue){
					$file = &$GLOBALS['config_files']['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			//FORMMAIL RECIPIENTS
			case 'formmail_values':
				if($settingvalue){
					$recipients = explode('<##>', $settingvalue);
					if($recipients){
						foreach($recipients as $recipient){
							$single_recipient = explode('<#>', $recipient);

							if(isset($single_recipient[0]) && ($single_recipient[0] === '#')){
								if(!empty($single_recipient[1])){
									$DB_WE->query('INSERT INTO ' . RECIPIENTS_TABLE . ' (Email) VALUES("' . $DB_WE->escape($single_recipient[1]) . '")');
								}
							} else {
								if(!empty($single_recipient[1]) && !empty($single_recipient[0])){
									$DB_WE->query('UPDATE ' . RECIPIENTS_TABLE . ' SET Email="' . $DB_WE->escape($single_recipient[1]) . '" WHERE ID=' . intval($single_recipient[0]));
								}
							}
						}
					}
				}

				return;

			case 'formmail_deleted':
				if($settingvalue){
					$formmail_deleted = explode(',', $settingvalue);
					foreach($formmail_deleted as $del){
						$DB_WE->query('DELETE FROM ' . RECIPIENTS_TABLE . ' WHERE ID=' . intval($del));
					}
				}
				return;

			case 'active_integrated_modules':
				if(!we_base_preferences::userIsAllowed('active_integrated_modules')){
					break;
				}
				$futureModules = we_base_request::_(we_base_request::STRING, 'newconf', [], 'active_integrated_modules');
				$active = we_modules_info::getUserEnabledModules();
				$inactive = we_modules_info::getAllInactiveModules();
				$delNonEmpty = we_base_request::_(we_base_request::BOOL, 'removeNonEmptyTables');

				foreach($active as $cur){//remove tables if empty
					if(!in_array($cur, $futureModules)){//remove tables if empty
						$tables = we_modules_info::getModuleField($cur, 'tables');
						if(!$tables){
							continue;
						}
						foreach($tables as $name => $table){
							if($GLOBALS['DB_WE']->isTabExist($table) && ($delNonEmpty || f('SELECT COUNT(1) FROM ' . $table) == 0)){
								$GLOBALS['DB_WE']->delTable($table);
							}
						}
					}
				}

				foreach($inactive as $cur){//add new tables
					if(in_array($cur, $futureModules)){//remove tables if empty
						$tables = we_modules_info::getModuleField($cur, 'tables');
						if(!$tables){
							continue;
						}
						foreach($tables as $name => $table){
							we_base_updater::replayUpdateDB(stripTblPrefix($table) . '.sql');
						}
					}
				}
				$GLOBALS['config_files']['conf_active_integrated_modules']['content'] = '<?php
$GLOBALS[\'_we_active_integrated_modules\'] = [
\'' . implode("',\n'", $futureModules) . '\'
];';
				return;

			case 'useproxy':
				if($settingvalue == 1){
					// Create/overwrite proxy settings file
					$host = we_base_request::_(we_base_request::STRING, 'newconf', '', "proxyhost");
					$port = we_base_request::_(we_base_request::INT, 'newconf', '', "proxyport");
					$user = we_base_request::_(we_base_request::STRING, 'newconf', '', "proxyuser");
					$pass = str_replace('"', '', we_base_request::_(we_base_request::RAW_CHECKED, 'newconf', '', 'proxypass'));
					$pass = ($pass === we_customer_customer::NOPWD_CHANGE ? (defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '') : $pass);
					we_base_preferences::setConfigContent('proxysettings', '<?php
	define(\'WE_PROXYHOST\', "' . $host . '");
	define(\'WE_PROXYPORT\', ' . $port . ');
	define(\'WE_PROXYUSER\', "' . $user . '");
	define(\'WE_PROXYPASSWORD\', "' . $pass . '");'
					);
				} else {
					// Delete proxy settings file
					we_base_file::deleteLocalFile(WEBEDITION_PATH . 'liveUpdate/includes/proxysettings.inc.php');
					we_base_preferences::unsetConfig('proxysettings');
				}
				return;

			case 'proxyhost':
			case 'proxyport':
			case 'proxyuser':
			case 'proxypass':
				return;
			case 'SMTP_PASSWORD':
				if($settingvalue !== we_customer_customer::NOPWD_CHANGE){
					$file = &$GLOBALS['config_files']['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			// ADVANCED
			case 'DB_CONNECT':
				$file = &$GLOBALS['config_files']['conf_conf']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue);
				return;

			case 'DB_SET_CHARSET':
				$file = &$GLOBALS['config_files']['conf_conf']['content'];

				if(!defined($settingname) || $settingvalue != constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			case 'useauth':
				$file = &$GLOBALS['config_files']['conf_conf']['content'];
				if($settingvalue == 1){
					// enable
					if(!(defined('HTTP_USERNAME')) || !(defined('HTTP_PASSWORD'))){
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', 'myUsername', false);
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', 'myPassword', false);
					}

					$un = defined('HTTP_USERNAME') ? HTTP_USERNAME : '';
					$pw = defined('HTTP_PASSWORD') ? HTTP_PASSWORD : '';
					$un1 = we_base_request::_(we_base_request::STRING, 'newconf', '', 'HTTP_USERNAME');
					$pw1 = we_base_request::_(we_base_request::STRING, 'newconf', '', 'HTTP_PASSWORD');
					if($un != $un1 || $pw1 != we_customer_customer::NOPWD_CHANGE){

						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', $un1);
						$file = ($pw1 === we_customer_customer::NOPWD_CHANGE ? $file : we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', $pw1));
					}
				} else {
					// disable
					if(defined('HTTP_USERNAME') || defined('HTTP_PASSWORD')){
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', 'myUsername', false);
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', 'myPassword', false);
					}
				}
				return;

			case 'HTTP_USERNAME':
			case 'HTTP_PASSWORD':
				return;

			//ERROR HANDLING
			case 'WE_ERROR_HANDLER':
			case 'WE_ERROR_NOTICES':
			case 'WE_ERROR_DEPRECATED':
			case 'WE_ERROR_WARNINGS':
			case 'WE_ERROR_ERRORS':
			case 'WE_ERROR_SHOW':
			case 'WE_ERROR_LOG':
				$file = &$GLOBALS['config_files']['conf_global']['content'];

				if($settingvalue != constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			case 'WE_ERROR_MAIL':
				$file = &$GLOBALS['config_files']['conf_global']['content'];

				if(!$settingvalue && WE_ERROR_MAIL){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL', 0, true, $comment);
				} else if($settingvalue && !WE_ERROR_MAIL){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL', 1, true, $comment);
				}
				return;

			case 'WE_ERROR_MAIL_ADDRESS':
				$file = &$GLOBALS['config_files']['conf_global']['content'];

				if(WE_ERROR_MAIL_ADDRESS != $settingvalue){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL_ADDRESS', $settingvalue, true, $comment);
				}
				return;

			case 'ERROR_DOCUMENT_NO_OBJECTFILE':
				if(!defined($settingname) || constant($settingname) != $settingvalue){
					$file = &$GLOBALS['config_files']['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;
		}
	}

	/**
	 * This functions saves all options.
	 *
	 * @see            remember_value()
	 *
	 * @return         void
	 */
	private static function save_all_values(){
		we_base_preferences::loadConfigs();
		//set config to latest version
		$_REQUEST['newconf']['CONF_SAVED_VERSION'] = WE_SVNREV;
		// Second, change sourcecodes of the configfiles
		foreach($GLOBALS['configs'] as $name => $conf){
			foreach($conf as $key => $default){
				switch($name){
					case 'conf':
					case 'global'://no settings in session
						if(we_base_preferences::userIsAllowed($key)){
							self::remember_value(we_base_request::_($default[1], 'newconf', null, $key), $key, $default[0]);
						}
						break;
					case 'user':
						self::remember_value(we_base_request::_($default[0], 'newconf', null, $key), $key);
						break;
					default:
						if(we_base_preferences::userIsAllowed($key)){
							self::remember_value(we_base_request::_($default[0], 'newconf', null, $key), $key);
						}
						break;
				}
			}
		}
		if(isset($_SESSION['weS']['versions']) && isset($_SESSION['weS']['versions']['logPrefs'])){
			$_SESSION['weS']['versions']['logPrefsChanged'] = [];
			foreach(array_keys($_SESSION['weS']['versions']['logPrefs']) as $k){
				if(isset($_REQUEST['newconf'][$k])){
					if($_SESSION['weS']['versions']['logPrefs'][$k] != $_REQUEST['newconf'][$k]){
						$_SESSION['weS']['versions']['logPrefsChanged'][$k] = $_REQUEST['newconf'][$k];
					}
				} elseif($_SESSION['weS']['versions']['logPrefs'][$k] != ''){
					$_SESSION['weS']['versions']['logPrefsChanged'][$k] = '';
				}
			}

			if(($_SESSION['weS']['versions']['logPrefsChanged'])){
				$versionslog = new we_versions_log();
				$versionslog->saveVersionsLog($_SESSION['weS']['versions']['logPrefsChanged'], we_versions_log::VERSIONS_PREFS);
			}
			unset($_SESSION['weS']['versions']['logPrefs']);
			unset($_SESSION['weS']['versions']['logPrefsChanged']);
		}

		//SAVE CHANGES
		// Third save all changes of the config files
		we_base_preferences::saveConfigs();
	}

	/**
	 * This builds every single dialog (of a tab).
	 *
	 * @param          $selected_setting                       string              (optional)
	 *
	 * @see            render_dialog()
	 *
	 * @return         string
	 */
	private static function build_dialog($selected_setting = 'ui'){
		global $DB_WE;
		$weSuggest = & we_gui_suggest::getInstance();

		switch($selected_setting){
			case 'save':
				return self::create_dialog('', [['headline' => '', 'html' => g_l('prefs', '[save]'),]]);

			case 'saved'://SAVED SUCCESSFULLY DIALOG
				return self::create_dialog('', [['headline' => '', 'html' => g_l('prefs', '[saved]'),]]);

			case 'ui':
				//LANGUAGE
				$settings = [];

				//	Look which languages are installed ...
				$language_directory = dir(WE_INCLUDES_PATH . 'we_language');

				while(false !== ($entry = $language_directory->read())){
					if($entry != '.' && $entry != '..'){
						if(is_dir(WE_INCLUDES_PATH . 'we_language/' . $entry)){
							$language[$entry] = $entry;
						}
					}
				}
				global $languages;

				if(!empty($language)){ // Build language select box
					$languages = new we_html_select(['name' => 'newconf[Language]', 'class' => 'weSelect', 'onchange' => "document.getElementById('langnote').style.display='block'"]);
					foreach($language as $key => $value){
						$languages->addOption($key, $value);
					}
					$languages->selectOption(self::get_value('Language'));
					// Lang notice
					$langNote = '<div id="langnote" style="padding: 5px; background-color: rgb(221, 221, 221); width: 190px; display:none">
<table style="width:100%">
<tbody>
<tr>
<td style="padding-right: 10px;vertical-align:top">
	<span class="fa-stack fa-lg" style="font-size: 14px;color:#007de3;"><i class="fa fa-circle fa-stack-2x" ></i><i class="fa fa-info fa-stack-1x fa-inverse"></i></span>
</td>
<td class="middlefont">' . g_l('prefs', '[language_notice]') . '
</td>
</tr>
</tbody>
</table>
</div>';
					// Build dialog
					$settings[] = ['headline' => g_l('prefs', '[choose_language]'), 'html' => $languages->getHtml() . '<br/><br/>' . $langNote, 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true];
				} else { // Just one Language Installed, no select box needed
					foreach($language as $key => $value){
						$languages = $value;
					}
					// Build dialog
					$settings[] = ['headline' => g_l('prefs', '[choose_language]'), 'html' => $languages, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true];
				}

				$BackendCharset = new we_html_select(['name' => 'newconf[BackendCharset]', 'class' => 'weSelect', 'onchange' => "document.getElementById('langnote').style.display='block'"]);
				$c = we_base_charsetHandler::getAvailCharsets();
				foreach($c as $char){
					$BackendCharset->addOption($char, $char);
				}
				$BackendCharset->selectOption(self::get_value('BackendCharset'));
				$settings[] = ['headline' => g_l('prefs', '[choose_backendcharset]'), 'html' => $BackendCharset->getHtml() . '<br/><br/>' . $langNote, 'space' => we_html_multiIconBox::SPACE_BIG];


				// DEFAULT CHARSET
				if(we_base_preferences::userIsAllowed('DEFAULT_CHARSET')){
					$charsets = we_base_charsetHandler::inst()->getCharsetsForTagWizzard();
					$charset = $GLOBALS['we']['Charset']['Backend'];
					$GLOBALS['weDefaultCharset'] = self::get_value('DEFAULT_CHARSET');
					$defaultCharset = we_html_tools::textInput('newconf[DEFAULT_CHARSET]', $GLOBALS['weDefaultCharset'], 255, [], 'text', 100);
					$defaultCharsetChooser = we_html_tools::select('DefaultCharsetSelect', $charsets, 1, $GLOBALS['weDefaultCharset'], false, ['onchange' => "document.forms[0].elements['newconf[DEFAULT_CHARSET]'].value=this.options[this.selectedIndex].value;"], true, 100, '', false);
					$DEFAULT_CHARSET = '<table class="default"><tr><td>' . $defaultCharset . '</td><td>' . $defaultCharsetChooser . '</td></tr></table>';

					$settings[] = ['headline' => g_l('prefs', '[default_charset]'),
						'space' => we_html_multiIconBox::SPACE_BIG,
						'html' => $DEFAULT_CHARSET
					];
				}

				//AMOUNT COLUMNS IN COCKPIT
				$cockpit_amount_columns = new we_html_select(['name' => 'newconf[cockpit_amount_columns]', 'class' => 'weSelect']);
				for($i = 1; $i <= 10; $i++){
					$cockpit_amount_columns->addOption($i, $i);
				}
				$cockpit_amount_columns->selectOption(self::get_value('cockpit_amount_columns'));
				$settings[] = ['headline' => g_l('prefs', '[cockpit_amount_columns]'), 'html' => $cockpit_amount_columns->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];

				/*				 * **************************************************************
				 * SEEM
				 * *************************************************************** */

				if(we_base_preferences::userIsAllowed('WE_SEEM')){
					// Build maximize window
					$seem_disabler = we_html_forms::checkbox(1, self::get_value('WE_SEEM') == 0 ? 1 : 0, 'newconf[WE_SEEM]', g_l('prefs', '[seem_deactivate]'));

					// Build dialog if user has permission
					$settings[] = ['headline' => g_l('prefs', '[seem]'), 'html' => $seem_disabler, 'space' => we_html_multiIconBox::SPACE_BIG];
				}

				// SEEM start document
				if(we_base_preferences::userIsAllowed('seem_start_type')){

					// Cockpit
					$document_path = $object_path = '';
					$document_id = $object_id = 0;

					switch(self::get_value('seem_start_type')){
						default:
							$seem_start_type = 0;
							break;
						case 'cockpit':
							$_SESSION['prefs']['seem_start_file'] = 0;
							$_SESSION['prefs']['seem_start_weapp'] = '';
							$seem_start_type = 'cockpit';
							break;

						// Object
						case 'object':
							$seem_start_type = 'object';
							if(self::get_value('seem_start_file') != 0){
								$object_id = self::get_value('seem_start_file');
								$get_object_paths = getPathsFromTable(OBJECT_FILES_TABLE, $GLOBALS['DB_WE'], we_base_constants::FILE_ONLY, $object_id);

								if(isset($get_object_paths[$object_id])){ //	seeMode start file exists
									$object_path = $get_object_paths[$object_id];
								}
							}
							break;
						case 'weapp':
							$seem_start_type = 'weapp';
							if(self::get_value('seem_start_weapp') != ''){
								$seem_start_weapp = self::get_value('seem_start_weapp');
							}

							break;
						// Document
						case 'document':
							$seem_start_type = 'document';
							if(self::get_value('seem_start_file') != 0){
								$document_id = self::get_value('seem_start_file');
								$get_document_paths = getPathsFromTable(FILE_TABLE, $GLOBALS['DB_WE'], we_base_constants::FILE_ONLY, $document_id);

								if(isset($get_document_paths[$document_id])){ //	seeMode start file exists
									$document_path = $get_document_paths[$document_id];
								}
							}
							break;
					}

					$start_type = new we_html_select(['name' => 'newconf[seem_start_type]', 'class' => 'weSelect', 'id' => 'seem_start_type', 'onchange' => "show_seem_chooser(this.value);"]);

					$showStartType = false;
					$permitedStartTypes = [''];
					$start_type->addOption(0, '-');
					$seem_cockpit_selectordummy = "<div id='selectordummy' style='height:24px;'>&nbsp;</div>";
					if(we_base_permission::hasPerm(we_base_perm::WORK_QUICKSTART_CAN_SEE)){
						$start_type->addOption('cockpit', g_l('prefs', '[seem_start_type_cockpit]'));
						$showStartType = true;
						$permitedStartTypes[] = 'cockpit';
					}

					$seem_document_chooser = '';
					if(we_base_permission::hasPerm(we_base_perm::WORK_DOCUMENTS_CAN_SEE)){
						$start_type->addOption('document', g_l('prefs', '[seem_start_type_document]'));
						$showStartType = true;
						// Build SEEM select start document chooser

						$weSuggest->setAcId('Doc');
						$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML, we_base_ContentTypes::IMAGE]);
						$weSuggest->setInput('seem_start_document_name', $document_path, [], self::get_value('seem_start_file'));
						$weSuggest->setMaxResults(20);
						$weSuggest->setRequired(true);
						$weSuggest->setResult('seem_start_document', $document_id);
						$weSuggest->setSelector(we_gui_suggest::DocSelector);
						$weSuggest->setWidth(150);
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));

						$seem_document_chooser = we_html_element::span(['id' => 'seem_start_document', 'style' => 'display:none'], $weSuggest->getHTML());
						$permitedStartTypes[] = 'document';
					}
					$seem_object_chooser = '';
					if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm(we_base_perm::OBJECTFILES_CAN_SEE)){
						$start_type->addOption('object', g_l('prefs', '[seem_start_type_object]'));
						$showStartType = true;
						// Build SEEM select start object chooser

						$weSuggest->setAcId('Obj');
						$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE]);
						$weSuggest->setInput('seem_start_object_name', $object_path, [], self::get_value('seem_start_file'));
						$weSuggest->setMaxResults(20);
						$weSuggest->setRequired(true);
						$weSuggest->setResult('seem_start_object', $object_id);
						$weSuggest->setSelector(we_gui_suggest::DocSelector);
						$weSuggest->setTable(OBJECT_FILES_TABLE);
						$weSuggest->setWidth(150);
						$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, false));

						$seem_object_chooser = we_html_element::span(['id' => 'seem_start_object', 'style' => 'display:none'], $weSuggest->getHTML());
						$permitedStartTypes[] = 'object';
					}
					/* $start_weapp = new we_html_select(['name' => 'newconf[seem_start_weapp]', 'class' => 'weSelect', 'id' => 'seem_start_weapp']);

					  $seem_weapp_chooser = '';
					  if($start_weapp->getOptionNum()){
					  $start_type->addOption('weapp', g_l('prefs', '[seem_start_type_weapp]'));
					  if(!empty($seem_start_weapp)){
					  $start_weapp->selectOption($seem_start_weapp);
					  }
					  $weAPPSelector = $start_weapp->getHtml();
					  $seem_weapp_chooser = we_html_element::span(['id' => 'seem_start_weapp', 'style' => 'display:none'], $weAPPSelector);
					  $permitedStartTypes[] = 'weapp';
					  } */

					// Build final HTML code
					if($showStartType){
						if(in_array($seem_start_type, $permitedStartTypes)){
							$start_type->selectOption($seem_start_type);
						} else {
							$seem_start_type = $permitedStartTypes[0];
						}
						$seem_html = new we_html_table(['class' => 'default defaultfont'], 2, 1);
						$seem_html->setColContent(0, 0, $start_type->getHtml());
						$seem_html->setCol(1, 0, ['style' => 'padding-top:5px;'], $seem_cockpit_selectordummy . $seem_document_chooser . $seem_object_chooser);
						$settings[] = ['headline' => g_l('prefs', '[seem_startdocument]'), 'html' => $seem_html->getHtml(),
							'space' => we_html_multiIconBox::SPACE_BIG];
						self::$jsCmd->addCmd('show_seem_chooser', $seem_start_type);
					}

					// Build dialog if user has permission
				}
				$val = self::get_value('message_reporting');

				$html = "<input type=\"hidden\" id=\"message_reporting\" name=\"newconf[message_reporting]\" value=\"$val\" />" . we_html_forms::checkbox(we_base_util::WE_MESSAGE_ERROR, 1, "message_reporting_errors", g_l('prefs', '[message_reporting][show_errors]'), false, '', "handle_message_reporting_click();", true) . "<br/>" .
					we_html_forms::checkbox(we_base_util::WE_MESSAGE_WARNING, $val & we_base_util::WE_MESSAGE_WARNING, "message_reporting_warnings", g_l('prefs', '[message_reporting][show_warnings]'), false, '', "handle_message_reporting_click();") . "<br/>" .
					we_html_forms::checkbox(we_base_util::WE_MESSAGE_NOTICE, $val & we_base_util::WE_MESSAGE_NOTICE, "message_reporting_notices", g_l('prefs', '[message_reporting][show_notices]'), false, '', "handle_message_reporting_click();");

				$settings[] = ['headline' => g_l('prefs', '[message_reporting][headline]') . we_html_tools::alertAttentionBox(g_l('prefs', '[message_reporting][information]', false, true), we_html_tools::TYPE_HELP, false, false),
					'html' => $html, 'space' => we_html_multiIconBox::SPACE_BIG];

				/*				 * *******************************************************
				 * Sidebar
				 * ******************************************************* */
				if(we_base_preferences::userIsAllowed('SIDEBAR_DISABLED')){
					// Settings
					$sidebar_disable = self::get_value('SIDEBAR_DISABLED');
					$sidebar_show = ($sidebar_disable) ? 'none' : 'block';

					$sidebar_id = self::get_value('SIDEBAR_DEFAULT_DOCUMENT');
					$sidebar_paths = getPathsFromTable(FILE_TABLE, $GLOBALS['DB_WE'], we_base_constants::FILE_ONLY, $sidebar_id);
					$sidebar_path = '';
					if(isset($sidebar_paths[$sidebar_id])){
						$sidebar_path = $sidebar_paths[$sidebar_id];
					}

					// Enable / disable sidebar
					$sidebar_disabler = we_html_forms::checkbox(1, $sidebar_disable, 'newconf[SIDEBAR_DISABLED]', g_l('prefs', '[sidebar_deactivate]'), false, '', "document.getElementById('sidebar_options').style.display=(this.checked?'none':'block');");

					// Show on Startup
					$sidebar_show_on_startup = we_html_forms::checkbox(1, self::get_value('SIDEBAR_SHOW_ON_STARTUP'), 'newconf[SIDEBAR_SHOW_ON_STARTUP]', g_l('prefs', '[sidebar_show_on_startup]'), false, '', '');

					// Sidebar width
					$sidebar_width = we_html_tools::textInput('newconf[SIDEBAR_DEFAULT_WIDTH]', self::get_value('SIDEBAR_DEFAULT_WIDTH'), 255, ['onchange' => "if ( isNaN( this.value ) ||  parseInt(this.value) < 100 ) { this.value=100; };"], 'number', 90);
					$sidebar_width_chooser = we_html_tools::select('tmp_sidebar_width', [0 => '', 100 => 100, 150 => 150, 200 => 200, 250 => 250, 300 => 300, 350 => 350, 400 => 400], 1, '', false, [
							'onchange' => "document.forms[0].elements['newconf[SIDEBAR_DEFAULT_WIDTH]'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;"], true, 100);

					// Sidebar document
					$weSuggest->setAcId('SidebarDoc');
					$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT]);
					$weSuggest->setInput('ui_sidebar_file_name', $sidebar_path);
					$weSuggest->setMaxResults(20);
					$weSuggest->setResult('newconf[SIDEBAR_DEFAULT_DOCUMENT]', $sidebar_id);
					$weSuggest->setSelector(we_gui_suggest::DocSelector);
					$weSuggest->setWidth(150);
					$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:selectSidebarDoc()'));

					// build html
					$sidebar_html1 = new we_html_table(['class' => 'default'], 1, 1);

					$sidebar_html1->setColContent(0, 0, $sidebar_disabler);

					// build html
					$sidebar_html2 = new we_html_table(['class' => 'default defaultfont', 'id' => 'sidebar_options', 'style' => 'display:' . $sidebar_show], 8, 2);

					$sidebar_html2->setCol(0, 0, ['colspan' => 2, 'height' => 10], '');
					$sidebar_html2->setCol(1, 0, ['colspan' => 2, 'height' => 10], $sidebar_show_on_startup);
					$sidebar_html2->setCol(2, 0, ['colspan' => 2, 'height' => 10], '');
					$sidebar_html2->setCol(3, 0, ['colspan' => 2], g_l('prefs', '[sidebar_width]'));
					$sidebar_html2->setColContent(4, 0, $sidebar_width);
					$sidebar_html2->setCol(4, 1, ['style' => 'padding-left:10px;'], $sidebar_width_chooser);
					$sidebar_html2->setCol(5, 0, ['colspan' => 2, 'height' => 10], '');
					$sidebar_html2->setCol(6, 0, ['colspan' => 2], g_l('prefs', '[sidebar_document]'));
					$sidebar_html2->setCol(7, 0, ['colspan' => 2], $weSuggest->getHTML());

					// Build dialog if user has permission
					$settings[] = ['headline' => g_l('prefs', '[sidebar]'), 'html' => $sidebar_html1->getHtml() . $sidebar_html2->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];
				}


				// TREE

				$tree_count = self::get_value('default_tree_count');
				$file_tree_count = new we_html_select(['name' => 'newconf[default_tree_count]', 'class' => 'weSelect']);
				$file_tree_count->addOption(0, g_l('prefs', '[all]'));


				for($i = 10; $i < 51; $i += 10){
					$file_tree_count->addOption($i, $i);
				}

				for($i = 100; $i < 501; $i += 100){
					$file_tree_count->addOption($i, $i);
				}

				if(!$file_tree_count->selectOption($tree_count)){
					$file_tree_count->addOption($tree_count, $tree_count);
					// Set selected extension
					$file_tree_count->selectOption($tree_count);
				}

				$settings[] = ['headline' => g_l('prefs', '[tree_title]') . we_html_tools::alertAttentionBox(g_l('prefs', '[tree_count_description]'), we_html_tools::TYPE_HELP),
					'html' => $file_tree_count->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];


				//WINDOW DIMENSIONS

				if(self::get_value('sizeOpt') == 0){
					$window_specify = false;
					$window_max = true;
				} else {
					$window_specify = true;
					$window_max = false;
				}

				// Build maximize window
				$window_max_code = we_html_forms::radiobutton(0, self::get_value('sizeOpt') == 0, 'newconf[sizeOpt]', g_l('prefs', '[maximize]'), true, '', "document.getElementsByName('newconf[weWidth]')[0].disabled = true;document.getElementsByName('newconf[weHeight]')[0].disabled = true;");

				// Build specify window dimension
				$window_specify_code = we_html_forms::radiobutton(1, !(self::get_value('sizeOpt') == 0), 'newconf[sizeOpt]', g_l('prefs', '[specify]'), true, '', "document.getElementsByName('newconf[weWidth]')[0].disabled = false;document.getElementsByName('newconf[weHeight]')[0].disabled = false;");

				// Create specify window dimension input
				$window_specify_table = new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;margin-left:50px;'], 1, 3);
				$window_specify_table->setColContent(0, 0, we_html_tools::textInput('newconf[weWidth]', (self::get_value('sizeOpt') ? self::get_value('weWidth') : ''), 4, [
						'disabled' => (self::get_value('sizeOpt') == 0 ? 'disabled' : "")], "number", 60));
				$window_specify_table->setCol(0, 1, ['style' => 'padding:0 .5em;vertical-align: middle;'], 'x');
				$window_specify_table->setColContent(0, 2, we_html_tools::textInput('newconf[weHeight]', (self::get_value('sizeOpt') ? self::get_value('weHeight') : ''), 4, [
						'disabled' => (self::get_value('sizeOpt') == 0 ? 'disabled' : "")], "number", 60));

				// Build apply current window dimension
				$window_current_dimension_table = we_html_button::create_button('apply_current_dimension', "javascript:document.getElementsByName('newconf[sizeOpt]')[1].checked = true;document.getElementsByName('newconf[weWidth]')[0].disabled = false;document.getElementsByName('newconf[weHeight]')[0].disabled = false;document.getElementsByName('newconf[weWidth]')[0].value = parent.opener.top.window.outerWidth;document.getElementsByName('newconf[weHeight]')[0].value = parent.opener.top.window.outerHeight;");

				// Build final HTML code
				$window_html = new we_html_table(['class' => 'default withSpace'], 3, 1);
				$window_html->setColContent(0, 0, $window_max_code);
				$window_html->setColContent(1, 0, $window_specify_code . $window_specify_table->getHtml());
				$window_html->setCol(2, 0, ['style' => 'padding-left:50px;'], $window_current_dimension_table);

				// Build dialog
				$settings[] = ['headline' => g_l('prefs', '[dimension]'), 'html' => $window_html->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];
				return self::create_dialog('', $settings, -1);

			case 'defaultAttribs':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}

				/**
				 * inlineedit setting
				 */
				// Build select box

				$commands_default_tmp = we_html_tools::select('tmp_commands', we_wysiwyg_editor::getEditorCommands(false), 1, '', false, ['onchange' => "var elem=document.getElementById('commands_default_id'); var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]);
				$COMMANDS_DEFAULT = we_html_tools::textInput('newconf[COMMANDS_DEFAULT]', self::get_value('COMMANDS_DEFAULT'), '', ['id' => "commands_default_id"], 'text', 225, 0, '');

				$CSSAPPLYTO_DEFAULT = new we_html_select(['name' => 'newconf[CSSAPPLYTO_DEFAULT]', 'class' => 'weSelect']);
				$CSSAPPLYTO_DEFAULT->addOption('all', 'all');
				$CSSAPPLYTO_DEFAULT->addOption('around', 'around');
				$CSSAPPLYTO_DEFAULT->addOption('wysiwyg', 'wysiwyg');
				$CSSAPPLYTO_DEFAULT->selectOption(self::get_value('CSSAPPLYTO_DEFAULT') ?: 'around');

				$weSuggest->setAcId('doc2');
				$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
				$weSuggest->setInput('imagestartid_default_text', (IMAGESTARTID_DEFAULT ? id_to_path(IMAGESTARTID_DEFAULT) : ''));
				$weSuggest->setMaxResults(20);
				$weSuggest->setResult('newconf[IMAGESTARTID_DEFAULT]', (IMAGESTARTID_DEFAULT ?: 0));
				$weSuggest->setSelector(we_gui_suggest::DirSelector);
				$weSuggest->setWidth(226);
				$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory', document.forms[0].elements['newconf[IMAGESTARTID_DEFAULT]'].value,WE().consts.tables.FILE_TABLE, 'newconf[IMAGESTARTID_DEFAULT]','imagestartid_default_text')"));
				$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[IMAGESTARTID_DEFAULT]'].value = 0;document.forms[0].elements.imagestartid_default_text.value=''"));

				$settings = [
					['headline' => g_l('prefs', '[xhtml_default]'), 'html' => self::getTrueFalseSelect('XHTML_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[inlineedit_default]', false, true), 'html' => self::getTrueFalseSelect('INLINEEDIT_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[imagestartid_default]', false, true), 'html' => $weSuggest->getHTML(), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[commands_default]', false, true), 'html' => '<div>' . $commands_default_tmp . '</div><div style="margin-top:4px">' . $COMMANDS_DEFAULT . '</div>',
						'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[removefirstparagraph_default]', false, true), 'html' => self::getTrueFalseSelect('REMOVEFIRSTPARAGRAPH_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[showinputs_default]', false, true), 'html' => self::getTrueFalseSelect('SHOWINPUTS_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[hidenameattribinweimg_default]'), 'html' => self::getYesNoSelect('HIDENAMEATTRIBINWEIMG_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[hidenameattribinweform_default]'), 'html' => self::getYesNoSelect('HIDENAMEATTRIBINWEFORM_DEFAULT'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[replaceacronym]'), 'html' => self::getYesNoSelect('REPLACEACRONYM'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[cssapplyto_default]', false, true), 'html' => $CSSAPPLYTO_DEFAULT->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG],
				];

				return self::create_dialog('', $settings, -1);

			case 'countries':
				if(!we_base_preferences::userIsAllowed('WE_COUNTRIES_DEFAULT')){
					break;
				}

				$countries_default = we_html_tools::textInput('newconf[WE_COUNTRIES_DEFAULT]', self::get_value('WE_COUNTRIES_DEFAULT'), '', [], 'text', 225);

				$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
				$langcode = array_search($lang[0], getWELangs());
				$countrycode = array_search($langcode, getWECountries());
				$supported = we_base_country::getTranslationList(we_base_country::TERRITORY, $langcode);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
				asort($supported, SORT_LOCALE_STRING);
				setlocale(LC_ALL, $oldLocale);
				$countries_top = explode(',', self::get_value('WE_COUNTRIES_TOP'));
				$countries_shown = explode(',', self::get_value('WE_COUNTRIES_SHOWN'));
				$tabC = new we_html_table(['style' => 'border:1px solid black', 'class' => 'defaultfont'], 1, 4);
				$i = 0;
				$tabC->setCol($i, 0, ['class' => 'bold'], g_l('prefs', '[countries_country]'));
				$tabC->setCol($i, 1, ['class' => 'bold'], g_l('prefs', '[countries_top]'));
				$tabC->setCol($i, 2, ['class' => 'bold'], g_l('prefs', '[countries_show]'));
				$tabC->setCol($i, 3, ['class' => 'bold'], g_l('prefs', '[countries_noshow]'));
				foreach($supported as $countrycode => $country){
					$i++;
					$tabC->addRow();
					$tabC->setColContent($i, 0, CheckAndConvertISObackend($country));
					$tabC->setColContent($i, 1, '<input type="radio" name="newconf[countries][' . $countrycode . ']" value="2" ' . (in_array($countrycode, $countries_top) ? 'checked="checked"' : '') . ' > ');
					$tabC->setColContent($i, 2, '<input type="radio" name="newconf[countries][' . $countrycode . ']" value="1" ' . (in_array($countrycode, $countries_shown) ? 'checked="checked"' : '') . ' > ');
					$tabC->setColContent($i, 3, '<input type="radio" name="newconf[countries][' . $countrycode . ']" value="0" ' . (!in_array($countrycode, $countries_top) && !in_array($countrycode, $countries_shown) ? 'checked' : '') . ' > ');
				}

				$settings = [
					['headline' => g_l('prefs', '[countries_headline]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[countries_information]', false, true), we_html_tools::TYPE_INFO, 450, false),
						'noline' => true],
					['headline' => g_l('prefs', '[countries_default]'), 'html' => $countries_default, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => '', 'html' => $tabC->getHtml(), 'noline' => true],
				];
				// Build dialog element if user has permission
				return self::create_dialog('', $settings);

			case 'language':
				if(!we_base_preferences::userIsAllowed('locale_default') && we_base_preferences::userIsAllowed('locale_locales')){
					break;
				}
				$default = self::get_value('locale_default');
				$locales = self::get_value('locale_locales');

				self::$jsCmd->addCmd('initLocale', $default);

				$hidden_fields = we_html_element::hidden('newconf[locale_default]', $default, 'locale_default') .
					we_html_element::hidden('newconf[locale_locales]', implode(',', array_keys($locales)), 'locale_locales');

				//Locales
				$select_box = new we_html_select(['class' => 'weSelect', 'name' => 'locale_temp_locales', 'size' => 10, 'id' => 'locale_temp_locales', 'style' => 'width: 340px']);
				$select_box->addOptions($locales);

				$enabled_buttons = (count($locales) > 0);


				// Create edit list
				$editlist_table = new we_html_table(['class' => 'default'], 1, 2);

				// Buttons
				$default = we_html_button::create_button('default', 'javascript:defaultLocale()', '', '', !$enabled_buttons);
				$delete = we_html_button::create_button(we_html_button::DELETE, 'javascript:deleteLocale()');

				$editlist_table->setCol(0, 0, ['style' => 'padding-right:10px;'], $hidden_fields . $select_box->getHtml());
				$editlist_table->setCol(0, 1, ['style' => 'vertical-align:top'], $default . $delete);

				// Add Locales
				// Languages
				$Languages = g_l('languages', '');
				$TopLanguages = [
					'~de' => $Languages['de'],
					'~nl' => $Languages['nl'],
					'~en' => $Languages['en'],
					'~fi' => $Languages['fi'],
					'~fr' => $Languages['fr'],
					'~pl' => $Languages['pl'],
					'~ru' => $Languages['ru'],
					'~es' => $Languages['es'],
				];
				asort($Languages);
				asort($TopLanguages);
				$TopLanguages[''] = '---';
				$Languages = array_merge($TopLanguages, $Languages);

				$languages = new we_html_select(['name' => 'newconf[locale_language]', 'id' => 'locale_language', 'style' => 'width: 139px', 'class' => 'weSelect']);
				$languages->addOptions($Languages);

				// Countries
				$Countries = g_l('countries', '');
				$TopCountries = [
					'~DE' => $Countries['DE'],
					'~CH' => $Countries['CH'],
					'~AT' => $Countries['AT'],
					'~NL' => $Countries['NL'],
					'~GB' => $Countries['GB'],
					'~US' => $Countries['US'],
					'~FI' => $Countries['FI'],
					'~FR' => $Countries['FR'],
					'~PL' => $Countries['PL'],
					'~RU' => $Countries['RU'],
					'~ES' => $Countries['ES'],
				];
				asort($Countries);
				asort($TopCountries);
				$TopCountries['~'] = '---';
				$Countries = array_merge(['' => ''], $TopCountries, $Countries);

				$countries = new we_html_select(['name' => 'newconf[locale_country]', 'id' => 'locale_country', 'style' => 'width: 139px', 'class' => 'weSelect']);
				$countries->addOptions($Countries);

				// Button
				$add_button = we_html_button::create_button(we_html_button::ADD, 'javascript:addLocale()');

				// Build final HTML code
				$add_html = g_l('prefs', '[locale_languages]') . '<br/>' .
					$languages->getHtml() . '<br/><br/>' .
					g_l('prefs', '[locale_countries]') . '<br/>' .
					$countries->getHtml() . '<br/><br/>' .
					$add_button;

				$settings = [
					['headline' => g_l('prefs', '[tab][language]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[locale_information]', false, true), we_html_tools::TYPE_INFO, 450, false, 50) . we_html_element::br() . $editlist_table->getHtml(),
						'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[locale_add]'), 'html' => $add_html, 'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[langlink_headline]'), 'noline' => true],
					['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[langlink_information]', false, true), we_html_tools::TYPE_INFO, 450, false, 50),
						'space' => we_html_multiIconBox::SPACE_MED2,
						'noline' => true],
					['headline' => g_l('prefs', '[langlink_support]'), 'html' => self::getTrueFalseSelect('LANGLINK_SUPPORT'), 'space' => we_html_multiIconBox::SPACE_MED2, 'noline' => true],
				];

				return self::create_dialog('', $settings);

			case 'editor':
				//EDITOR PLUGIN

				$attr = ' style="width:150px;"';

				$template_editor_mode = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorMode]', 'onchange' => 'displayEditorOptions(this.options[this.options.selectedIndex].value);']);
				$template_editor_mode->addOption('textarea', g_l('prefs', '[editor_plaintext]'));
				$template_editor_mode->addOption('codemirror2', g_l('prefs', '[editor_javascript2]'));
				//$template_editor_mode->addOption('java', g_l('prefs', '[editor_java]'));
				$template_editor_mode->selectOption(self::get_value('editorMode'));

				/**
				 * Editor font settings
				 */
				$template_fonts = ['Arial',
					'Andale Mono',
					'Consolas',
					'Courier New',
					'DejaVu Sans Mono',
					'Droid Sans Mono',
					'FreeMono',
					'Helvetica',
					'Inconsolata',
					'Letter Gothic',
					'Liberation Mono',
					'Menlo',
					'Monaco',
					'Mono',
					'Nimbus Mono L',
					'Tahoma',
					'Verdana',
					'serif',
					'sans-serif'];
				$template_font_sizes = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 24, 32, 48, 72];

				$template_editor_font_specify = (self::get_value('editorFontname') != '' && self::get_value('editorFontname') != 'none');
				$template_editor_font_size_specify = (self::get_value('editorFontsize') != '' && self::get_value('editorFontsize') != -1);

				// Build specify font
				$template_editor_font_specify_code = we_html_forms::checkbox(1, $template_editor_font_specify, 'newconf[editorFont]', g_l('prefs', '[specify]'), true, 'defaultfont', "if (document.getElementsByName('newconf[editorFont]')[0].checked) { document.getElementsByName('newconf[editorFontname]')[0].disabled = false;document.getElementsByName('newconf[editorFontsize]')[0].disabled = false; } else { document.getElementsByName('newconf[editorFontname]')[0].disabled = true;document.getElementsByName('newconf[editorFontsize]')[0].disabled = true; }");

				$template_editor_font_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorFontname]', 'style' => 'width: 135px;', ($template_editor_font_specify ? 'enabled' : 'disabled') => ($template_editor_font_specify ? 'enabled' : 'disabled')]);

				/* 			$colorsDisabled = true;

				  $template_editor_fontcolor_selector = getColorInput('newconf[editorFontcolor]', self::get_value('editorFontcolor'), $colorsDisabled);
				  $template_editor_we_tag_fontcolor_selector = getColorInput('newconf[editorWeTagFontcolor]', self::get_value('editorWeTagFontcolor'), $colorsDisabled);
				  $template_editor_we_attribute_fontcolor_selector = getColorInput('newconf[editorWeAttributeFontcolor]', self::get_value('editorWeAttributeFontcolor'), $colorsDisabled);
				  $template_editor_html_tag_fontcolor_selector = getColorInput('newconf[editorHTMLTagFontcolor]', self::get_value('editorHTMLTagFontcolor'), $colorsDisabled);
				  $template_editor_html_attribute_fontcolor_selector = getColorInput('newconf[editorHTMLAttributeFontcolor]', self::get_value('editorHTMLAttributeFontcolor'), $colorsDisabled);
				  $template_editor_pi_tag_fontcolor_selector = getColorInput('newconf[editorPiTagFontcolor]', self::get_value('editorPiTagFontcolor'), $colorsDisabled);
				  $template_editor_comment_fontcolor_selector = getColorInput('newconf[editorCommentFontcolor]', self::get_value('editorCommentFontcolor'), $colorsDisabled);
				 */
				foreach($template_fonts as $font){
					$template_editor_font_select_box->addOption($font, $font);
				}
				$template_editor_font_select_box->selectOption($template_editor_font_specify ? self::get_value('editorFontname') : 'Courier New');

				$template_editor_font_sizes_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorFontsize]', 'style' => 'width: 135px;', ($template_editor_font_size_specify ? 'enabled' : 'disabled') => ($template_editor_font_size_specify ? 'enabled' : 'disabled')]);
				foreach($template_font_sizes as $key => $sz){
					$template_editor_font_sizes_select_box->addOption($sz, $sz);
				}
				$template_editor_font_sizes_select_box->selectOption($template_editor_font_specify ? $template_font_sizes[$key] : 11);

				$template_editor_font_sizes_select_box->selectOption(self::get_value('editorFontsize'));


				$template_editor_font_specify_table = '<table style="margin:0 0 20px 50px;" class="default defaultfont">
	<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontname]') . '</td><td>' . $template_editor_font_select_box->getHtml() . '</td></tr>
	<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontsize]') . '</td><td>' . $template_editor_font_sizes_select_box->getHtml() . '</td></tr>
</table>';
				/*
				  $template_editor_font_color_checkbox = we_html_forms::checkboxWithHidden(self::get_value('specify_jeditor_colors'), "newconf[specify_jeditor_colors]", g_l('prefs', '[editor_font_colors]'), false, "defaultfont", "setEditorColorsDisabled(!this.checked);");
				  $attr = ($colorsDisabled ? $attr_dis : $attr);
				  $template_editor_font_color_table = '<table id="editorColorTable" style="margin: 10px 0 0 50px;" class="default">
				  <tr><td id="label_editorFontcolor" ' . $attr . '>' . g_l('prefs', '[editor_normal_font_color]') . '</td><td>' . $template_editor_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorWeTagFontcolor"' . $attr . '>' . g_l('prefs', '[editor_we_tag_font_color]') . '</td><td>' . $template_editor_we_tag_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorWeAttributeFontcolor"' . $attr . '>' . g_l('prefs', '[editor_we_attribute_font_color]') . '</td><td>' . $template_editor_we_attribute_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorHTMLTagFontcolor"' . $attr . '>' . g_l('prefs', '[editor_html_tag_font_color]') . '</td><td>' . $template_editor_html_tag_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorHTMLAttributeFontcolor"' . $attr . '>' . g_l('prefs', '[editor_html_attribute_font_color]') . '</td><td>' . $template_editor_html_attribute_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorPiTagFontcolor"' . $attr . '>' . g_l('prefs', '[editor_pi_tag_font_color]') . '</td><td>' . $template_editor_pi_tag_fontcolor_selector . '</td></tr>
				  <tr><td id="label_editorCommentFontcolor"' . $attr . '>' . g_l('prefs', '[editor_comment_font_color]') . '</td><td>' . $template_editor_comment_fontcolor_selector . '</td></tr>
				  </table>';
				 */
				$template_editor_theme = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorTheme]']);
				foreach(glob(WE_LIB_PATH . 'CodeMirror/theme/*.css') as $filename){
					$theme = str_replace(['.css', WE_LIB_PATH . 'CodeMirror/theme/'], '', $filename);
					$template_editor_theme->addOption($theme, $theme);
				}
				$template_editor_theme->selectOption(self::get_value('editorTheme'));

				//Build activation of line numbers
				$template_editor_linenumbers_code = we_html_forms::checkbox(1, self::get_value('editorLinenumbers'), 'newconf[editorLinenumbers]', g_l('prefs', '[editor_enable]'), true, 'defaultfont', '');
				$template_editor_highlightLine_code = we_html_forms::checkbox(1, self::get_value('editorHighlightCurrentLine'), 'newconf[editorHighlightCurrentLine]', g_l('prefs', '[editor_enable]'), true, 'defaultfont', '');

				//Build activation of code completion
				$template_editor_codecompletion_code = we_html_element::hiddens([
						'newconf[editorCodecompletion][WE]' => self::get_value('editorCodecompletion-WE'),
						'newconf[editorCodecompletion][htmlTag]' => self::get_value('editorCodecompletion-htmlTag'),
						'newconf[editorCodecompletion][htmlDefAttr]' => self::get_value('editorCodecompletion-htmlDefAttr'),
						'newconf[editorCodecompletion][htmlAttr]' => self::get_value('editorCodecompletion-htmlAttr'),
						'newconf[editorCodecompletion][htmlJSAttr]' => self::get_value('editorCodecompletion-htmlJSAttr'),
						'newconf[editorCodecompletion][html5Tag]' => self::get_value('editorCodecompletion-html5Tag'),
						'newconf[editorCodecompletion][html5Attr]' => self::get_value('editorCodecompletion-html5Attr')
					]) .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-WE'), 'editorCodecompletion0', 'WE-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][WE]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-htmlTag'), 'editorCodecompletion1', 'HTML-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlTag]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-htmlDefAttr'), 'editorCodecompletion2', 'HTML-Default-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlDefAttr]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-htmlAttr'), 'editorCodecompletion3', 'HTML-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlAttr]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-htmlJSAttr'), 'editorCodecompletion4', 'HTML-JS-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlJSAttr]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-html5Tag'), 'editorCodecompletion5', 'HTML5-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][html5Tag]\');') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorCodecompletion-html5Attr'), 'editorCodecompletion6', 'HTML5-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][html5Attr]\');');


				$template_editor_tabstop_code = '<table class="default defaultfont">
				<tr><td colspan="2">' .
					we_html_forms::checkbox(1, self::get_value('editorShowTab'), 'editorShowTab', g_l('prefs', '[editor_tabstop]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorShowTab]\');') .
					we_html_element::hidden('newconf[editorShowTab]', self::get_value('editorShowTab')) .
					'</td></tr>
				<tr><td style="width:200px;">' . g_l('prefs', '[editor_tabSize]') . '</td><td>' . we_html_tools::textInput("newconf[editorTabSize]", self::get_value("editorTabSize"), 2, [
						], "number", 135) . '</td></tr>
				<tr><td colspan="2">' .
					we_html_forms::checkbox(1, self::get_value('editorAutoIndent'), 'editorAutoIndent', g_l('prefs', '[editor_autoindent]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorAutoIndent]\');') .
					we_html_element::hidden('newconf[editorAutoIndent]', self::get_value('editorAutoIndent')) .
					'</td></tr>
				<tr><td colspan="2">' .
					we_html_forms::checkbox(1, self::get_value('editorIndentSpaces'), 'editorIndentSpaces', g_l('prefs', '[editor][indentSpaces]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorIndentSpaces]\');') .
					we_html_element::hidden('newconf[editorIndentSpaces]', self::get_value('editorIndentSpaces')) .
					'</td></tr>
				<tr><td colspan="2">' .
					we_html_forms::checkbox(1, self::get_value('editorShowSpaces'), 'editorShowSpaces', g_l('prefs', '[editor][showSpaces]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorShowSpaces]\');') .
					we_html_element::hidden('newconf[editorShowSpaces]', self::get_value('editorShowSpaces')) .
					'</td></tr>
			</table>';

				$template_editor_Wrap_code = we_html_forms::checkbox(1, self::get_value('editorWrap'), 'editorWrap', g_l('prefs', '[editor_enable]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorWrap]\');') .
					we_html_element::hidden('newconf[editorWrap]', self::get_value('editorWrap'));


				$template_editor_tooltips_code = we_html_forms::checkbox(1, self::get_value('editorTooltips'), 'newconf[editorTooltips]', g_l('prefs', '[editorTooltips]'), true, '', '') .
					'<br/>' .
					we_html_forms::checkbox(1, self::get_value('editorTooltipsIDs'), 'newconf[editorTooltipsIDs]', g_l('prefs', '[editorTooltipsIDs]'), true, '', '');

				$template_editor_tooltip_font_specify = (self::get_value('editorTooltipFontname') != '' && self::get_value('editorTooltipFontname') != 'none');
				$template_editor_tooltip_font_size_specify = (self::get_value('editorTooltipFontsize') != '' && self::get_value('editorTooltipFontsize') != -1);

				// Build specify font
				$template_editor_tooltip_font_specify_code = we_html_forms::checkbox(1, $template_editor_tooltip_font_specify, 'newconf[editorTooltipFont]', g_l('prefs', '[specify]'), true, '', 'if (document.getElementsByName(\'newconf[editorTooltipFont]\')[0].checked) { document.getElementsByName(\'newconf[editorTooltipFontname]\')[0].disabled = false;document.getElementsByName(\'newconf[editorTooltipFontsize]\')[0].disabled = false; } else { document.getElementsByName(\'newconf[editorTooltipFontname]\')[0].disabled = true;document.getElementsByName(\'newconf[editorTooltipFontsize]\')[0].disabled = true; }');

				$template_editor_tooltip_font_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorTooltipFontname]', 'style' => 'width: 135px;',
					($template_editor_tooltip_font_specify ? 'enabled' : 'disabled') => ($template_editor_tooltip_font_specify ? 'enabled' : 'disabled')]);

				foreach($template_fonts as $font){
					$template_editor_tooltip_font_select_box->addOption($font, $font);
				}
				$template_editor_tooltip_font_select_box->selectOption($template_editor_tooltip_font_specify ? self::get_value('editorTooltipFontname') : 'Tahoma');

				$template_editor_tooltip_font_sizes_select_box = new we_html_select(['class' => 'weSelect editor editor_codemirror2', 'name' => 'newconf[editorTooltipFontsize]',
					'style' => 'width: 135px;', ($template_editor_tooltip_font_size_specify ? 'enabled' : 'disabled') => ($template_editor_tooltip_font_size_specify ? 'enabled' : 'disabled')]);
				$template_toolfont_sizes = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20];
				foreach($template_toolfont_sizes as $sz){
					$template_editor_tooltip_font_sizes_select_box->addOption($sz, $sz);
				}
				$template_editor_tooltip_font_sizes_select_box->selectOption($template_editor_tooltip_font_specify ? self::get_value("editor_tooltip_font_size") : 11);
				$template_editor_tooltip_font_specify_table = '<table style="margin:0 0 20px 50px;" class="default defaultfont">
				<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontname]') . '</td><td>' . $template_editor_tooltip_font_select_box->getHtml() . '</td></tr>
				<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontsize]') . '</td><td>' . $template_editor_tooltip_font_sizes_select_box->getHtml() . '</td></tr>
			</table>';

				//Build activation of integration of documentation
				$template_editor_autoClose = we_html_forms::checkbox(1, self::get_value('editorDocuintegration'), 'newconf[editorDocuintegration]', g_l('prefs', '[editor_enable]'), true, '', '') .
//remove fonts not available
					we_html_element::jsScript(LIB_DIR . 'fontdetect/fontdetect.js', 'checkFonts()');
				$settings = [
					//['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[editor_information]'), we_html_tools::TYPE_INFO, 480, false),],
					['headline' => g_l('prefs', '[editor_mode]'), 'html' => $template_editor_mode->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2 editor_textarea', 'headline' => g_l('prefs', '[editor_font]'), 'html' => $template_editor_font_specify_code . $template_editor_font_specify_table,
						'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_theme]'), 'html' => $template_editor_theme->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED2],
//				array('class' => 'editor editor_java', 'headline' => g_l('prefs', '[editor_highlight_colors]'), 'html' => $template_editor_font_color_checkbox . $template_editor_font_color_table, 'space' => we_html_multiIconBox::SPACE_MED2),
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_linenumbers]'), 'html' => $template_editor_linenumbers_code, 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_highlightLine]'), 'html' => $template_editor_highlightLine_code, 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2 editor_textarea', 'headline' => g_l('global', '[wrapcheck]'), 'html' => $template_editor_Wrap_code, 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2 editor_textarea', 'headline' => g_l('prefs', '	[editor][indent]'), 'html' => $template_editor_tabstop_code, 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_completion]'), 'html' => $template_editor_codecompletion_code, 'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_tooltips]'), 'html' => $template_editor_tooltips_code . '<br/>' . $template_editor_tooltip_font_specify_code . '<br/>' . $template_editor_tooltip_font_specify_table,
						'space' => we_html_multiIconBox::SPACE_MED2],
					['class' => 'editor editor_codemirror2', 'headline' => g_l('prefs', '[editor_autoCloseTags]'), 'html' => $template_editor_autoClose, 'space' => we_html_multiIconBox::SPACE_MED2],
					//array('class'=>'editor editor_codemirror2','headline' => g_l('prefs', '[editor_docuclick]'), 'html' => $template_editor_docuintegration_code, 'space' => we_html_multiIconBox::SPACE_MED2),
				];

				return self::create_dialog("settings_editor_predefined", $settings, count($settings), g_l('prefs', '[show_predefined]'), g_l('prefs', '[hide_predefined]'));

			case 'recipients':
				if(!we_base_preferences::userIsAllowed('FORMMAIL_VIAWEDOC')){
					break;
				}
				$settings = [];
				//FORMMAIL RECIPIENTS
				if(we_base_preferences::userIsAllowed('FORMMAIL_BLOCK')){
					// Build dialog if user has permission
					$settings[] = ['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[formmail_information]', false, true), we_html_tools::TYPE_INFO, 450, false),];

					/**
					 * Recipients list
					 */
					$select_box = new we_html_select(['class' => "weSelect", 'name' => "we_recipient", 'size' => 10, 'style' => "width: 340px;height:100px", 'ondblclick' => "edit_recipient();"]);

					$enabled_buttons = false;

					foreach($DB_WE->getAllq('SELECT ID,Email FROM ' . RECIPIENTS_TABLE . ' ORDER BY Email') as $entry){
						$enabled_buttons = true;
						$select_box->addOption($entry['ID'], $entry['Email']);
					}

					// Create needed hidden fields
					$hidden_fields = we_html_element::hiddens(['newconf[formmail_values]' => '',
							'newconf[formmail_deleted]' => '']);

					// Create edit list
					$editlist_table = new we_html_table(['class' => 'default'], 2, 3);

					$editlist_table->setCol(0, 0, ['style' => 'padding-right:10px;'], $hidden_fields . $select_box->getHtml());
					$editlist_table->setCol(0, 2, ['style' => 'vertical-align:top;'], we_html_button::create_button(we_html_button::ADD, "javascript:add_recipient();") . we_html_button::create_button(we_html_button::EDIT, "javascript:edit_recipient();", '', '', !$enabled_buttons, false) . we_html_button::create_button(we_html_button::DELETE, "javascript:delete_recipient();", '', '', !$enabled_buttons, false));

					// Build dialog if user has permission
					$settings[] = ['headline' => '', 'html' => $editlist_table->getHtml(),];
				}

				// formmail stuff


				if(we_base_preferences::userIsAllowed('FORMMAIL_CONFIRM')){
					$formmail_confirm = new we_html_select(['name' => 'newconf[FORMMAIL_CONFIRM]', 'style' => "width:88px;", 'class' => "weSelect"]);
					$formmail_confirm->addOption(1, g_l('prefs', '[on]'));
					$formmail_confirm->addOption(0, g_l('prefs', '[off]'));
					$formmail_confirm->selectOption(self::get_value("FORMMAIL_CONFIRM") ? 1 : 0);

					$settings[] = ['html' => $formmail_confirm->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[formmailConfirm]')];

					$formmail_log = new we_html_select(['name' => "newconf[FORMMAIL_LOG]", 'onchange' => "formmailLogOnOff()", 'style' => "width:88px;", 'class' => "weSelect"]);
					$formmail_log->addOption(1, g_l('prefs', '[yes]'));
					$formmail_log->addOption(0, g_l('prefs', '[no]'));
					$formmail_log->selectOption(self::get_value("FORMMAIL_LOG") ? 1 : 0);

					$html = '<table class="default">
							<tr>
								<td>' . $formmail_log->getHtml() . '</td>
								<td style="padding-left:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('show_formmail_log')") . '</td>
							</tr>
						</table>';
					$settings[] = ['html' => $html, 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[logFormmailRequests]'), 'noline' => true];

					$isDisabled = (self::get_value('FORMMAIL_LOG') == 0);


					$formmail_emptylog = new we_html_select(['name' => "newconf[FORMMAIL_EMPTYLOG]", 'style' => "width:88px;", 'class' => "weSelect"]);
					if($isDisabled){
						$formmail_emptylog->setAttribute("disabled", "disabled");
					}
					$formmail_emptylog->addOption(-1, g_l('prefs', '[never]'));
					$formmail_emptylog->addOption(we_base_util::SECONDS_DAY, g_l('prefs', '[1_day]'));
					$formmail_emptylog->addOption(we_base_util::SECONDS_DAY * 2, sprintf(g_l('prefs', '[more_days]'), 2));
					$formmail_emptylog->addOption(we_base_util::SECONDS_DAY * 4, sprintf(g_l('prefs', '[more_days]'), 4));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK, g_l('prefs', '[1_week]'));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK * 2, sprintf(g_l('prefs', '[more_weeks]'), 2));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK * 4, sprintf(g_l('prefs', '[more_weeks]'), 4));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK * 8, sprintf(g_l('prefs', '[more_weeks]'), 8));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK * 16, sprintf(g_l('prefs', '[more_weeks]'), 16));
					$formmail_emptylog->addOption(we_base_util::SECONDS_WEEK * 32, sprintf(g_l('prefs', '[more_weeks]'), 32));

					$formmail_emptylog->selectOption(self::get_value("FORMMAIL_EMPTYLOG"));


					$settings[] = ['html' => $formmail_emptylog->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[deleteEntriesOlder]')];

					// formmail only via we doc //
					$formmail_ViaWeDoc = new we_html_select(['name' => "newconf[FORMMAIL_VIAWEDOC]", 'style' => "width:88px;", 'class' => "weSelect"]);
					$formmail_ViaWeDoc->addOption(1, g_l('prefs', '[yes]'));
					$formmail_ViaWeDoc->addOption(0, g_l('prefs', '[no]'));
					$formmail_ViaWeDoc->selectOption((self::get_value("FORMMAIL_VIAWEDOC") ? 1 : 0));

					$settings[] = ['html' => $formmail_ViaWeDoc->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[formmailViaWeDoc]')];

					// limit formmail requests //
					$formmail_block = new we_html_select(['name' => "newconf[FORMMAIL_BLOCK]", 'onchange' => "formmailBlockOnOff()", 'style' => "width:88px;", 'class' => "weSelect"]);
					if($isDisabled){
						$formmail_block->setAttribute("disabled", "disabled");
					}
					$formmail_block->addOption(1, g_l('prefs', '[yes]'));
					$formmail_block->addOption(0, g_l('prefs', '[no]'));
					$formmail_block->selectOption(self::get_value("FORMMAIL_BLOCK") ? 1 : 0);

					$html = '<table class="default">
							<tr>
								<td>' . $formmail_block->getHtml() . '</td>
								<td style="padding-left:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('show_formmail_block_log')") . '</td>
							</tr>
						</table>';

					$settings[] = ['html' => $html, 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[blockFormmail]'), 'noline' => true];

					$isDisabled = $isDisabled || (self::get_value("FORMMAIL_BLOCK") == 0);

					// table is IE fix. Without table IE has a gap on the left of the input
					$formmail_trials = '<table class="default"><tr><td>' .
						we_html_tools::textInput("newconf[FORMMAIL_TRIALS]", self::get_value("FORMMAIL_TRIALS"), '', [], "text", 88, 0, '', $isDisabled) .
						'</td></tr></table>';

					$settings[] = ['html' => $formmail_trials, 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[formmailTrials]'), 'noline' => true];

					if(!$isDisabled){
						$isDisabled = (self::get_value("FORMMAIL_BLOCK") == 0);
					}

					$formmail_span = new we_html_select(['name' => "newconf[FORMMAIL_SPAN]", 'style' => "width:88px;", 'class' => "weSelect"]);
					if($isDisabled){
						$formmail_span->setAttribute("disabled", "disabled");
					}
					$formmail_span->addOption(60, g_l('prefs', '[1_minute]'));
					$formmail_span->addOption(120, sprintf(g_l('prefs', '[more_minutes]'), 2));
					$formmail_span->addOption(180, sprintf(g_l('prefs', '[more_minutes]'), 3));
					$formmail_span->addOption(300, sprintf(g_l('prefs', '[more_minutes]'), 5));
					$formmail_span->addOption(600, sprintf(g_l('prefs', '[more_minutes]'), 10));
					$formmail_span->addOption(1200, sprintf(g_l('prefs', '[more_minutes]'), 20));
					$formmail_span->addOption(1800, sprintf(g_l('prefs', '[more_minutes]'), 30));
					$formmail_span->addOption(2700, sprintf(g_l('prefs', '[more_minutes]'), 45));
					$formmail_span->addOption(3600, g_l('prefs', '[1_hour]'));
					$formmail_span->addOption(7200, sprintf(g_l('prefs', '[more_hours]'), 2));
					$formmail_span->addOption(14400, sprintf(g_l('prefs', '[more_hours]'), 4));
					$formmail_span->addOption(28800, sprintf(g_l('prefs', '[more_hours]'), 8));
					$formmail_span->addOption(we_base_util::SECONDS_DAY, sprintf(g_l('prefs', '[more_hours]'), 24));

					$formmail_span->selectOption(self::get_value("FORMMAIL_SPAN"));


					$settings[] = ['html' => $formmail_span->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[formmailSpan]'), 'noline' => true];
					$formmail_blocktime = new we_html_select(['name' => "newconf[FORMMAIL_BLOCKTIME]", 'style' => "width:88px;", 'class' => "weSelect"]);
					if($isDisabled){
						$formmail_blocktime->setAttribute("disabled", "disabled");
					}
					$formmail_blocktime->addOption(60, g_l('prefs', '[1_minute]'));
					$formmail_blocktime->addOption(120, sprintf(g_l('prefs', '[more_minutes]'), 2));
					$formmail_blocktime->addOption(180, sprintf(g_l('prefs', '[more_minutes]'), 3));
					$formmail_blocktime->addOption(300, sprintf(g_l('prefs', '[more_minutes]'), 5));
					$formmail_blocktime->addOption(600, sprintf(g_l('prefs', '[more_minutes]'), 10));
					$formmail_blocktime->addOption(1200, sprintf(g_l('prefs', '[more_minutes]'), 20));
					$formmail_blocktime->addOption(1800, sprintf(g_l('prefs', '[more_minutes]'), 30));
					$formmail_blocktime->addOption(2700, sprintf(g_l('prefs', '[more_minutes]'), 45));
					$formmail_blocktime->addOption(3600, g_l('prefs', '[1_hour]'));
					$formmail_blocktime->addOption(7200, sprintf(g_l('prefs', '[more_hours]'), 2));
					$formmail_blocktime->addOption(14400, sprintf(g_l('prefs', '[more_hours]'), 4));
					$formmail_blocktime->addOption(28800, sprintf(g_l('prefs', '[more_hours]'), 8));
					$formmail_blocktime->addOption(we_base_util::SECONDS_DAY, sprintf(g_l('prefs', '[more_hours]'), 24));
					$formmail_blocktime->addOption(-1, g_l('prefs', '[ever]'));

					$formmail_blocktime->selectOption(self::get_value("FORMMAIL_BLOCKTIME"));


					$settings[] = ['html' => $formmail_blocktime->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'headline' => g_l('prefs', '[blockFor]'), 'noline' => true];
				}

				return self::create_dialog('', $settings, -1);

			case 'modules':
				if(!we_base_preferences::userIsAllowed('active_integrated_modules')){
					break;
				}
				$modInfos = we_modules_info::getAllModules();

				$html = '';
				$tmp = [];
				foreach($modInfos as $modInfo){
					if(empty($modInfo['alwaysActive'])){
						$tmp[$modInfo['text']] = $modInfo;
					}
				}
				ksort($tmp);
				foreach($tmp as $modInfo){
					$onclick = (empty($modInfo['childmodule']) ? '' :
						"if(!this.checked){document.getElementById('newconf[active_integrated_modules][" . $modInfo['childmodule'] . "]').checked=false;}") .
						(empty($modInfo['dependson']) ? '' :
						"if(this.checked){document.getElementById('newconf[active_integrated_modules][" . $modInfo['dependson'] . "]').checked=true;}"
						);

					$html .= we_html_forms::checkbox($modInfo['name'], we_modules_info::isActive($modInfo['name']), "newconf[active_integrated_modules][" . $modInfo['name'] . "]", $modInfo['text'], false, '', $onclick) . '<br/>';
				}
				$html .= we_html_tools::alertAttentionBox(we_html_forms::checkbox('1', false, 'removeNonEmptyTables', g_l('prefs', '[module_activation][removeNonEmptyTables]')), we_html_tools::TYPE_ALERT, '60em', false);
				$settings = [
					['headline' => g_l('prefs', '[module_activation][headline]') . we_html_tools::alertAttentionBox(g_l('prefs', '[module_activation][information]', false, true), we_html_tools::TYPE_HELP, false, false)
						, 'html' => $html, 'space' => we_html_multiIconBox::SPACE_BIG]
				];

				return self::create_dialog('', $settings, -1);

			case 'proxy':
				if(!we_base_preferences::userIsAllowed('useproxy')){
					break;
				}
				/**
				 * Proxy server
				 */
				// Check Proxy settings  ...
				$proxy = self::get_value('proxy_proxy');

				$use_proxy = we_html_forms::checkbox(1, $proxy, 'newconf[useproxy]', g_l('prefs', '[useproxy]', false, true), false, '', "set_state();");
				$proxyaddr = we_html_tools::textInput('newconf[proxyhost]', self::get_value('WE_PROXYHOST'), '', [], "text", 225, 0, '', !$proxy);
				$proxyport = we_html_tools::textInput('newconf[proxyport]', self::get_value('WE_PROXYPORT'), '', [], "text", 225, 0, '', !$proxy);
				$proxyuser = we_html_tools::textInput('newconf[proxyuser]', self::get_value('WE_PROXYUSER'), '', ['autocomplete' => 'off'], "text", 225, 0, '', !$proxy);
				$proxypass = we_html_tools::textInput('newconf[proxypass]', we_customer_customer::NOPWD_CHANGE, '', ['autocomplete' => 'off'], 'password', 225, 0, '', !$proxy);

				// Build dialog if user has permission

				$settings = [
					['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[proxy_information]', false, true), we_html_tools::TYPE_INFO, 450, false), 'noline' => true],
					['headline' => g_l('prefs', '[tab][proxy]'), 'html' => $use_proxy, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[proxyaddr]'), 'html' => $proxyaddr, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[proxyport]'), 'html' => $proxyport, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[proxyuser]'), 'html' => $proxyuser, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[proxypass]'), 'html' => $proxypass, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
				];
				// Build dialog element if user has permission
				return self::create_dialog("", $settings, -1);


			case 'advanced':
				/*				 * *******************************************************************
				 * ATTRIBS
				 * ******************************************************************* */
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}

				$WYSIWYG_TYPE = new we_html_select(['name' => "newconf[WYSIWYG_TYPE]", 'class' => "weSelect"]);
				$options = ['tinyMCE' => 'tinyMCE', 'tinyMCE_beta' => 'tinyMCE4 (Apha - not for productive use!)'];
				foreach($options as $key => $val){
					$WYSIWYG_TYPE->addOption($key, $val);
				}
				$WYSIWYG_TYPE->selectOption(self::get_value("WYSIWYG_TYPE"));
				$settings[] = ['headline' => g_l('prefs', '[wysiwyg_type]'), 'html' => $WYSIWYG_TYPE->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];


				$we_max_upload_size = '<table class="default"><tr><td>' .
					we_html_tools::textInput("newconf[FILE_UPLOAD_MAX_UPLOAD_SIZE]", self::get_value("FILE_UPLOAD_MAX_UPLOAD_SIZE"), '', ['onkeypress' => "return WE().util.IsDigit(event);"], "number", 60) . ' MB</td><td style="padding-left:20px;" class="small">' .
					g_l('prefs', '[upload][we_max_size_hint]') .
					'</td></tr></table>';

				$settings[] = ['headline' => g_l('prefs', '[upload][we_max_size]'), 'html' => $we_max_upload_size, 'space' => we_html_multiIconBox::SPACE_BIG];

				$img = new we_html_select(['name' => "newconf[FILE_UPLOAD_IMG_MAX_SIZE]", 'class' => "weSelect"]);
				$op = we_fileupload_ui_base::$scaleProps;
				$op[''] = g_l('siteimport', '[nolimit]');
				$img->addOptions($op);
				$img->selectOption(self::get_value("FILE_UPLOAD_IMG_MAX_SIZE"));

				$we_max_img_upload_size = $img->getHtml() . ' px ';

				$settings[] = ['headline' => g_l('prefs', '[upload][we_max_img_size]'), 'html' => $we_max_img_upload_size, 'space' => we_html_multiIconBox::SPACE_BIG];



				/* $WYSIWYG_TYPE_FRONTEND = new we_html_select(array('name' => "newconf[WYSIWYG_TYPE_FRONTEND]", 'class' => "weSelect"));
				  $options = array('tinyMCE' => 'tinyMCE', 'default' => 'webEdition Editor (deprecated))');
				  foreach($options as $key => $val){
				  $WYSIWYG_TYPE_FRONTEND->addOption($key, $val);
				  }
				  $WYSIWYG_TYPE_FRONTEND->selectOption(self::get_value("WYSIWYG_TYPE_FRONTEND"));
				  $settings[] = array('headline' => "Editor für textareas im Frontend", 'html' => $WYSIWYG_TYPE_FRONTEND->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG);
				 */
				$we_doctype_workspace_behavior = self::get_value("WE_DOCTYPE_WORKSPACE_BEHAVIOR");
				$we_doctype_workspace_behavior_table = '<table class="default"><tr><td>' .
					we_html_forms::radiobutton(0, (!$we_doctype_workspace_behavior), "newconf[WE_DOCTYPE_WORKSPACE_BEHAVIOR]", g_l('prefs', '[we_doctype_workspace_behavior_0]'), true, '', '', false, g_l('prefs', '[we_doctype_workspace_behavior_hint0]'), 0, 430) .
					'</td></tr><tr><td style="padding-top:10px;">' .
					we_html_forms::radiobutton(1, $we_doctype_workspace_behavior, "newconf[WE_DOCTYPE_WORKSPACE_BEHAVIOR]", g_l('prefs', '[we_doctype_workspace_behavior_1]'), true, '', '', false, g_l('prefs', '[we_doctype_workspace_behavior_hint1]'), 0, 430) .
					'</td></tr></table>';

				$settings[] = ['headline' => g_l('prefs', '[we_doctype_workspace_behavior]'), 'html' => $we_doctype_workspace_behavior_table, 'space' => we_html_multiIconBox::SPACE_BIG];


				// Build select box
				$NAVIGATION_ENTRIES_FROM_DOCUMENT = new we_html_select(['name' => "newconf[NAVIGATION_ENTRIES_FROM_DOCUMENT]", 'class' => "weSelect"]);
				for($i = 0; $i < 2; $i++){
					$NAVIGATION_ENTRIES_FROM_DOCUMENT->addOption($i, g_l('prefs', $i == 0 ? '[navigation_entries_from_document_folder]' : '[navigation_entries_from_document_item]'));
				}
				$NAVIGATION_ENTRIES_FROM_DOCUMENT->selectOption(self::get_value("NAVIGATION_ENTRIES_FROM_DOCUMENT") ? 1 : 0);
				$settings[] = ['headline' => g_l('prefs', '[navigation_entries_from_document]'), 'html' => $NAVIGATION_ENTRIES_FROM_DOCUMENT->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];


				$NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH = new we_html_select(['name' => "newconf[NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH]", 'class' => "weSelect"]);
				$NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH->addOption(0, g_l('prefs', '[no]'));
				$NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH->addOption(1, g_l('prefs', '[yes]'));
				$NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH->selectOption(self::get_value("NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH") ? 1 : 0);
				$settings[] = ['headline' => g_l('prefs', '[navigation_rules_continue]'), 'html' => $NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG];

				return self::create_dialog("", $settings, -1);

			case 'system':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}
				//FILE EXTENSIONS
				// Get webEdition extensions
				$we_extensions = we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT);

				// Build static webEdition extensions select box
				$static_we_extensions = new we_html_select(['name' => 'newconf[DEFAULT_STATIC_EXT]', 'class' => 'weSelect']);
				$dynamic_we_extensions = new we_html_select(['name' => 'newconf[DEFAULT_DYNAMIC_EXT]', 'class' => 'weSelect']);
				foreach($we_extensions as $value){
					$static_we_extensions->addOption($value, $value);
					$dynamic_we_extensions->addOption($value, $value);
				}
				$static_we_extensions->selectOption(self::get_value('DEFAULT_STATIC_EXT'));
				$dynamic_we_extensions->selectOption(self::get_value('DEFAULT_DYNAMIC_EXT'));

				$we_extensions_html = g_l('prefs', '[static]') . we_html_element::br() . $static_we_extensions->getHtml() . we_html_element::br() . we_html_element::br() . g_l('prefs', '[dynamic]') . we_html_element::br() . $dynamic_we_extensions->getHtml();

				// HTML extensions
				$html_extensions = we_base_ContentTypes::getExtension(we_base_ContentTypes::HTML);

				// Build static webEdition extensions select box
				$static_html_extensions = new we_html_select(['name' => 'newconf[DEFAULT_HTML_EXT]', 'class' => 'weSelect']);
				foreach($html_extensions as $value){
					$static_html_extensions->addOption($value, $value);
				}
				$static_html_extensions->selectOption(self::get_value('DEFAULT_HTML_EXT'));

				$html_extensions_html = g_l('prefs', '[html]') . '<br/>' . $static_html_extensions->getHtml();


				//	array('headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[extensions_information]'), we_html_tools::TYPE_INFO, 450, false), ),



				$we_new_folder_mod = '<table class="default"><tr><td>' .
					we_html_tools::textInput("newconf[WE_NEW_FOLDER_MOD]", self::get_value("WE_NEW_FOLDER_MOD"), 3, ['onkeypress' => "return WE().util.IsDigit(event);"], "text", 60) . '</td><td style="padding-left:20px;" class="small">' .
					g_l('prefs', '[we_new_folder_mod_hint]') .
					'</td></tr></table>';

				// Build db select box
				$db_connect = new we_html_select(['name' => 'newconf[DB_CONNECT]', 'class' => 'weSelect']);
				if(class_exists('mysqli', false)){
					$db_connect->addOption('mysqli_connect', 'mysqli_connect');
					$db_connect->addOption('mysqli_pconnect', 'mysqli_pconnect');
				}
				if(function_exists('mysql_connect')){ //only allow old connection method if new is not available
					$db_connect->addOption('connect', 'connect (deprecated)');
					$db_connect->addOption('pconnect', 'pconnect (deprecated)');
				}
				$db_connect->selectOption(DB_CONNECT);

				// Build db charset select box
				$html_db_charset_warning = we_html_tools::alertAttentionBox(g_l('prefs', '[db_set_charset_warning]'), we_html_tools::TYPE_ALERT, false);

				$db_set_charset = new we_html_select(['name' => 'newconf[DB_SET_CHARSET]', 'class' => 'weSelect']);

				$GLOBALS['DB_WE']->query('SHOW CHARACTER SET');

				$charsets = [''];
				while($GLOBALS['DB_WE']->next_record()){
					$charsets[] = $GLOBALS['DB_WE']->f('Charset');
				}
				sort($charsets);
				foreach($charsets as $charset){
					$db_set_charset->addOption($charset, $charset);
				}

				if(defined('DB_SET_CHARSET') && DB_SET_CHARSET != ''){
					$db_set_charset->selectOption(DB_SET_CHARSET);
				} else {
					$tmp = $GLOBALS['DB_WE']->getCurrentCharset();
					if($tmp){
						$db_set_charset->selectOption($tmp);
						$file = &$GLOBALS['config_files']['conf_global']['content'];
						$file = we_base_preferences::changeSourceCode('define', $file, 'DB_SET_CHARSET', $tmp);
					}
				}

				// Check authentication settings  ...
				$auth = self::get_value("HTTP_USERNAME");
				$auth_user = self::get_value("HTTP_USERNAME");
				$auth_pass = self::get_value("HTTP_PASSWORD");

				// Build dialog if user has permission
				$use_auth = we_html_element::hidden('newconf[useauth]', $auth) .
					we_html_forms::checkbox(1, $auth, 'useauthEnabler', g_l('prefs', '[useauth]', false, true), false, '', "set_state_auth();");

				/**
				 * User name
				 */
				$authuser = we_html_tools::textInput('newconf[HTTP_USERNAME]', $auth_user, '', ['autocomplete' => 'off'], "text", 225, 0, '', !$auth);
				$authpass = we_html_tools::textInput('newconf[HTTP_PASSWORD]', we_customer_customer::NOPWD_CHANGE, '', ['autocomplete' => 'off'], 'password', 225, 0, '', !$auth);


				if(we_base_imageEdit::gd_version() > 0){ //  gd lib ist installiert
					$but = we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'newconf[WE_THUMBNAIL_DIRECTORY]', WE().consts.contentTypes.FOLDER, document.forms[0].elements['newconf[WE_THUMBNAIL_DIRECTORY]'].value, '')") : "";
					$inp = we_html_tools::textInput("newconf[WE_THUMBNAIL_DIRECTORY]", self::get_value("WE_THUMBNAIL_DIRECTORY"), '', [], 'text', 125);
					$thumbnail_dir = $inp . $but;
				} else { //  gd lib ist nicht installiert
					$but = we_base_permission::hasPerm(we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT) ? we_html_button::create_button(we_html_button::SELECT, "#", '', '', true) : "";
					$inp = we_html_tools::textInput("newconf[WE_THUMBNAIL_DIRECTORY]", self::get_value("WE_THUMBNAIL_DIRECTORY"), '', [], 'text', 125, 0, '', true);
					$thumbnail_dir = $inp . $but . '<br/>' . g_l('thumbnails', '[add_description_nogdlib]');
				}

				//  select if hooks can be executed
				$EXECUTE_HOOKS = new we_html_select(['name' => "newconf[EXECUTE_HOOKS]", 'class' => "weSelect"]);
				$EXECUTE_HOOKS->addOption(0, g_l('prefs', '[no]'));
				$EXECUTE_HOOKS->addOption(1, g_l('prefs', '[yes]'));

				$EXECUTE_HOOKS->selectOption(self::get_value("EXECUTE_HOOKS") ? 1 : 0);

				$hooksHtml = $EXECUTE_HOOKS->getHtml();

				$useSession = new we_html_select(['name' => "newconf[SYSTEM_WE_SESSION]", 'class' => "weSelect", 'onchange' => 'WE().util.showMessage(\'' . g_l('prefs', '[session][crypt][alert]') . '\', WE().consts.message.WE_MESSAGE_ERROR);']);
				$useSession->addOption(0, g_l('prefs', '[no]'));
				$useSession->addOption(1, g_l('prefs', '[yes]'));
				$useSession->selectOption(self::get_value("SYSTEM_WE_SESSION") ? 1 : 0);

				$sessionTime = '<table class="default"><tr><td>' .
					we_html_tools::textInput("newconf[SYSTEM_WE_SESSION_TIME]", abs(self::get_value("SYSTEM_WE_SESSION_TIME")), '', ['onkeypress' => "return WE().util.IsDigit(event);"], "number", 60) . '</td><td style="padding-left:20px;" class="small">s</td></tr></table>';

				$cryptSession = new we_html_select(['name' => 'newconf[SYSTEM_WE_SESSION_CRYPT]', 'class' => "weSelect", 'onchange' => 'WE().util.showMessage(\'' . g_l('prefs', '[session][crypt][alert]') . '\', WE().consts.message.WE_MESSAGE_ERROR);']);
				$cryptSession->addOption(0, g_l('prefs', '[no]'));
				$cryptSession->addOption(1, 'Transparent');
				$cryptSession->addOption(2, 'Cookie');
				$cryptSession->selectOption(self::get_value('SYSTEM_WE_SESSION_CRYPT'));

				$sessionHtml = $useSession->getHtml();


				$we_windowtypes = [0 => g_l('prefs', '[login][windowtypeboth]'),
					1 => g_l('prefs', '[login][windowtypepopup]'),
					2 => g_l('prefs', '[login][windowtypesame]')];
				$we_windowtypeselect = new we_html_select(['name' => 'newconf[WE_LOGIN_WEWINDOW]', 'class' => 'weSelect']);
				foreach($we_windowtypes as $key => $value){
					$we_windowtypeselect->addOption($key, $value);
				}
				$we_windowtypeselect->selectOption(self::get_value('WE_LOGIN_WEWINDOW'));
				$loginWE = we_html_forms::checkbox(1, self::get_value('WE_LOGIN_HIDEWESTATUS') == 1 ? 1 : 0, 'newconf[WE_LOGIN_HIDEWESTATUS]', g_l('prefs', '[login][deactivateWEstatus]')) .
					we_html_element::br() .
					g_l('prefs', '[login][windowtypes]') .
					we_html_element::br() .
					$we_windowtypeselect->getHtml();

				$_Schedtrigger_setting = new we_html_select(['name' => 'newconf[SCHEDULER_TRIGGER]', 'class' => "weSelect"]);
				$_Schedtrigger_setting->addOption(SCHEDULER_TRIGGER_PREDOC, g_l('prefs', '[we_scheduler_trigger][preDoc]')); //pre
				$_Schedtrigger_setting->addOption(SCHEDULER_TRIGGER_POSTDOC, g_l('prefs', '[we_scheduler_trigger][postDoc]')); //post
				$_Schedtrigger_setting->addOption(SCHEDULER_TRIGGER_CRON, g_l('prefs', '[we_scheduler_trigger][cron]')); //cron
				$_Schedtrigger_setting->selectOption(self::get_value('SCHEDULER_TRIGGER'));

				$settings = [
					['headline' => g_l('prefs', '[we_extensions]'), 'html' => $we_extensions_html, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[html_extensions]'), 'html' => $html_extensions_html, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[we_new_folder_mod]'), 'html' => $we_new_folder_mod, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[db_connect]', false, true), 'html' => $db_connect->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[db_set_charset]'), 'html' => $db_set_charset->getHtml() . we_html_tools::alertAttentionBox(g_l('prefs', '[db_set_charset_information]', false, true), we_html_tools::TYPE_HELP) . $html_db_charset_warning,
						'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[auth]'), 'html' => $use_auth, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[authuser]'), 'html' => $authuser, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[authpass]'), 'html' => $authpass, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[login][login]'), 'html' => $loginWE, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[thumbnail_dir]'), 'html' => $thumbnail_dir, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[hooks]'), 'html' => $hooksHtml . we_html_tools::alertAttentionBox(g_l('prefs', '[hooks_information]', false, true), we_html_tools::TYPE_HELP),
						'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[we_scheduler_trigger][head]'),
						'html' => we_modules_info::isActive(we_modules_info::SCHEDULER) ? $_Schedtrigger_setting->getHtml() . we_html_tools::alertAttentionBox(g_l('prefs', '[we_scheduler_trigger][description]', false, true), we_html_tools::TYPE_HELP) : '-',
						'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[session][title]'), 'html' => $sessionHtml . we_html_tools::alertAttentionBox(g_l('prefs', '[session][information]', false, true), we_html_tools::TYPE_HELP),
						'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[session][time]'), 'html' => $sessionTime, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[session][crypt][title]'), 'html' => $cryptSession->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG],
				];
				// Build dialog element if user has permission
				return self::create_dialog("", $settings, -1);

			case 'seolinks':
				/*				 * *******************************************************************
				 * ATTRIBS
				 * ******************************************************************* */
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}
				// Build dialog if user has permission

				$navigation_directoryindex_names = we_html_tools::textInput("newconf[NAVIGATION_DIRECTORYINDEX_NAMES]", self::get_value("NAVIGATION_DIRECTORYINDEX_NAMES"), '', [
						], 'text', 225);

				$weSuggest->setAcId('doc2');
				$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML]);
				$weSuggest->setInput('error_document_no_objectfile_text', ( ERROR_DOCUMENT_NO_OBJECTFILE ? id_to_path(ERROR_DOCUMENT_NO_OBJECTFILE) : ''));
				$weSuggest->setMaxResults(20);
				$weSuggest->setResult('newconf[ERROR_DOCUMENT_NO_OBJECTFILE]', ( ERROR_DOCUMENT_NO_OBJECTFILE ?: 0));
				$weSuggest->setSelector(we_gui_suggest::DocSelector);
				$weSuggest->setWidth(300);
				$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.forms[0].elements['newconf[ERROR_DOCUMENT_NO_OBJECTFILE]'].value,WE().consts.tables.FILE_TABLE, 'newconf[ERROR_DOCUMENT_NO_OBJECTFILE]','error_document_no_objectfile_text','','','', '" . we_base_ContentTypes::WEDOCUMENT . ',' . we_base_ContentTypes::HTML . "', 1)"));
				$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[ERROR_DOCUMENT_NO_OBJECTFILE]'].value = 0;document.forms[0].elements.error_document_no_objectfile_text.value = ''"));

				$settings = [
					['headline' => g_l('prefs', '[general_directoryindex_hide]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[navigation_directoryindex_description]'), we_html_tools::TYPE_INFO, 480),
						'noline' => true],
					['headline' => g_l('prefs', '[navigation_directoryindex_hide]'), 'html' => self::getTrueFalseSelect('NAVIGATION_DIRECTORYINDEX_HIDE'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[wysiwyglinks_directoryindex_hide]'), 'html' => self::getTrueFalseSelect('WYSIWYGLINKS_DIRECTORYINDEX_HIDE'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[navigation_directoryindex_names]'), 'html' => $navigation_directoryindex_names, 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['html' => we_html_tools::alertAttentionBox(g_l('prefs', '[general_directoryindex_hide_description]'), we_html_tools::TYPE_INFO, 480), 'noline' => true],
					['headline' => g_l('prefs', '[taglinks_directoryindex_hide]'), 'html' => self::getTrueFalseSelect('TAGLINKS_DIRECTORYINDEX_HIDE'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[general_objectseourls]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[objectseourls_lowercase_description]'), we_html_tools::TYPE_HELP),
						'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[objectseourls_lowercase]'), 'html' => self::getTrueFalseSelect('OBJECTSEOURLS_LOWERCASE'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[navigation_objectseourls]'), 'html' => self::getTrueFalseSelect('NAVIGATION_OBJECTSEOURLS'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[wysiwyglinks_objectseourls]'), 'html' => self::getTrueFalseSelect('WYSIWYGLINKS_OBJECTSEOURLS'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['html' => we_html_tools::alertAttentionBox(g_l('prefs', '[general_objectseourls_description]'), we_html_tools::TYPE_INFO, 480), 'noline' => true],
					['headline' => g_l('prefs', '[taglinks_objectseourls]'), 'html' => self::getTrueFalseSelect('TAGLINKS_OBJECTSEOURLS'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[urlencode_objectseourls]'), 'html' => self::getTrueFalseSelect('URLENCODE_OBJECTSEOURLS'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[general_seoinside]'), 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[general_seoinside_description]'), we_html_tools::TYPE_HELP),
						'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[seoinside_hideineditmode]'), 'html' => self::getTrueFalseSelect('SEOINSIDE_HIDEINEDITMODE'), 'space' => we_html_multiIconBox::SPACE_BIG,
						'noline' => true],
					['headline' => g_l('prefs', '[seoinside_hideinwebedition]'), 'html' => self::getTrueFalseSelect('SEOINSIDE_HIDEINWEBEDITION'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[error_no_object_found]'), 'html' => $weSuggest->getHTML(), 'space' => we_html_multiIconBox::SPACE_BIG, 'noline' => true],
					['headline' => g_l('prefs', '[suppress404code]'), 'html' => self::getTrueFalseSelect('SUPPRESS404CODE'), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[force404redirect]'), 'html' => self::getTrueFalseSelect('FORCE404REDIRECT') . we_html_tools::alertAttentionBox(g_l('prefs', '[force404redirect_description]'), we_html_tools::TYPE_HELP),
						'space' => we_html_multiIconBox::SPACE_BIG],
				];
				return self::create_dialog('', $settings, -1, '', '');

			case 'error_handling':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}

				/**
				 * Error handler
				 */
				$foldAt = 4;

				// Create checkboxes
				$we_error_handler = we_html_forms::checkbox(1, self::get_value('WE_ERROR_HANDLER'), 'newconf[WE_ERROR_HANDLER]', g_l('prefs', '[error_use_handler]', false, true), false, '', 'set_state_error_handler();');

				// Error types
				// Create checkboxes
				$error_handling_table = new we_html_table(['class' => 'default'], 8, 1);

				$error_handling_table->setColContent(0, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_ERRORS'), 'newconf[WE_ERROR_ERRORS]', g_l('prefs', '[error_errors]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));
				$error_handling_table->setColContent(2, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_WARNINGS'), 'newconf[WE_ERROR_WARNINGS]', g_l('prefs', '[error_warnings]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));
				$error_handling_table->setColContent(4, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_NOTICES'), 'newconf[WE_ERROR_NOTICES]', g_l('prefs', '[error_notices]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));
				$error_handling_table->setColContent(6, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_DEPRECATED'), 'newconf[WE_ERROR_DEPRECATED]', g_l('prefs', '[error_deprecated]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));

				// Create checkboxes
				$error_display_table = new we_html_table(['class' => 'default defaultfont'], 8, 1);
				$error_display_table->setCol(0, 0, ['style' => 'padding-left: 25px;'], we_html_tools::alertAttentionBox(g_l('prefs', '[error_notices_warning]'), we_html_tools::TYPE_ALERT, 260));

				$error_display_table->setColContent(1, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_SHOW'), 'newconf[WE_ERROR_SHOW]', g_l('prefs', '[error_display]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));
				$error_display_table->setColContent(3, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_LOG'), 'newconf[WE_ERROR_LOG]', g_l('prefs', '[error_log]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));
				$error_display_table->setColContent(5, 0, we_html_forms::checkbox(1, self::get_value('WE_ERROR_MAIL'), 'newconf[WE_ERROR_MAIL]', g_l('prefs', '[error_mail]'), false, '', '', !self::get_value('WE_ERROR_HANDLER')));

				// Create specify mail address input
				$error_mail_specify_table = new we_html_table(['class' => 'default defaultfont', 'style' => 'margin-left:25px;'], 1, 4);

				$error_mail_specify_table->setColContent(0, 1, g_l('prefs', '[error_mail_address]') . ': ');
				$error_mail_specify_table->setCol(0, 2, ['style' => 'text-align:left'], we_html_tools::textInput('newconf[WE_ERROR_MAIL_ADDRESS]', self::get_value('WE_ERROR_MAIL_ADDRESS'), 100, [
						'placeholder' => "mail@example"], 'email', 195));

				$error_display_table->setColContent(7, 0, $error_mail_specify_table->getHtml());

				$settings = [
					//array('headline' => g_l('prefs', '[templates]'), 'html' => $template_error_handling_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG),
					['headline' => g_l('prefs', '[tab][error_handling]'), 'html' => $we_error_handler, 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[error_types]'), 'html' => $error_handling_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG],
					['headline' => g_l('prefs', '[error_displaying]'), 'html' => $error_display_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_BIG],
				];

				return self::create_dialog('settings_error_expert', $settings, $foldAt, g_l('prefs', '[show_expert]'), g_l('prefs', '[hide_expert]'));

			case 'security':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					return '';
				}
				$row = 0;
				$customer_table = new we_html_table(['class' => 'default defaultfont', 'id' => 'customer_table'], 9, 10);
				$customer_table->setCol($row, 0, ['width' => '20px'], '');
				$customer_table->setCol($row, 1, ['colspan' => 5], g_l('prefs', '[security][customer][disableLogins]') . ':');
				$customer_table->setCol($row, 6, ['width' => 300]);
				$customer_table->setColContent( ++$row, 1, g_l('prefs', '[security][customer][sameIP]'));
				$customer_table->setCol($row, 2, ['width' => '20px']);
				$customer_table->setColContent($row, 3, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_IP]', self::get_value('SECURITY_LIMIT_CUSTOMER_IP'), 3, [], 'number', 50));
				$customer_table->setCol($row, 4, ['style' => 'width:2em;text-align:center'], '/');
				$customer_table->setColContent($row, 5, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_IP_HOURS]', self::get_value('SECURITY_LIMIT_CUSTOMER_IP_HOURS'), 3, [
						], 'number', 50));
				$customer_table->setColContent($row, 6, 'h');

				$customer_table->setColContent( ++$row, 1, g_l('prefs', '[security][customer][sameUser]'));
				$customer_table->setColContent($row, 3, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_NAME]', self::get_value('SECURITY_LIMIT_CUSTOMER_NAME'), 3, [
						], 'number', 50));
				$customer_table->setCol($row, 4, ['style' => 'text-align:center;'], '/');
				$customer_table->setColContent($row, 5, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_NAME_HOURS]', self::get_value('SECURITY_LIMIT_CUSTOMER_NAME_HOURS'), 3, [
						], 'number', 50));
				$customer_table->setColContent($row, 6, 'h');

				$customer_table->setColContent( ++$row, 1, g_l('prefs', '[security][customer][errorPage]'));

				$weSuggest->setAcId('SECURITY_LIMIT_CUSTOMER_REDIRECT_doc');
				$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML]);
				$weSuggest->setInput('SECURITY_LIMIT_CUSTOMER_REDIRECT_text', (SECURITY_LIMIT_CUSTOMER_REDIRECT ? id_to_path(SECURITY_LIMIT_CUSTOMER_REDIRECT) : ''));
				$weSuggest->setMaxResults(20);
				$weSuggest->setResult('newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]', ( SECURITY_LIMIT_CUSTOMER_REDIRECT ?: 0));
				$weSuggest->setSelector(we_gui_suggest::DocSelector);
				$weSuggest->setWidth(250);
				$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document', document.forms[0].elements['newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]'].value,WE().consts.tables.FILE_TABLE, 'newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]','SECURITY_LIMIT_CUSTOMER_REDIRECT_text','','','', '" . we_base_ContentTypes::WEDOCUMENT . "," . we_base_ContentTypes::HTML . "', 1)"));
				$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]'].value = 0;document.forms[0].elements.SECURITY_LIMIT_CUSTOMER_REDIRECT_text.value = ''"));

				$customer_table->setCol($row, 3, ['colspan' => 5], $weSuggest->getHTML());

				$customer_table->setColContent( ++$row, 1, g_l('prefs', '[security][customer][slowDownLogin]'));
				$customer_table->setColContent($row, 3, we_html_tools::textInput('newconf[SECURITY_DELAY_FAILED_LOGIN]', self::get_value('SECURITY_DELAY_FAILED_LOGIN'), 3, [
						], 'number', 50));
				$customer_table->setColContent($row, 4, 's');

				$customer_table->setColContent( ++$row, 1, g_l('prefs', '[security][customer][deleteSession]'));

				$customer_table->setColContent($row, 3, we_html_tools::select('newconf[SECURITY_DELETE_SESSION]', [g_l('prefs', '[no]'), g_l('prefs', '[yes]')], 1, self::get_value('SECURITY_DELETE_SESSION')));

				$encryption = new we_html_select(['name' => 'newconf[SECURITY_ENCRYPTION_TYPE_PASSWORD]', 'class' => 'weSelect']);
				//$encryption->addOption(we_customer_customer::ENCRYPT_NONE, g_l('prefs', '[security][encryption][type][0]'));
				if(function_exists('openssl_encrypt') && (array_search('blowfish', openssl_get_cipher_methods(true)) !== false)){
					$encryption->addOption(we_customer_customer::ENCRYPT_SYMMETRIC, g_l('prefs', '[security][encryption][type][1]'));
				}

				$encryption->addOption(we_customer_customer::ENCRYPT_HASH, g_l('prefs', '[security][encryption][type][2]'), []);
				$encryption->selectOption(self::get_value('SECURITY_ENCRYPTION_TYPE_PASSWORD'));


				$encryptinfo = we_html_tools::alertAttentionBox(g_l('prefs', '[security][encryption][hint]'), we_html_tools::TYPE_ALERT, false);
				$cryptkey = self::get_value('SECURITY_ENCRYPTION_KEY');
				$encryptionKey = we_html_tools::textInput('newconf[SECURITY_ENCRYPTION_KEY]', ($cryptkey ?: bin2hex(we_customer_customer::cryptGetIV(56))), 112, [], 'text', '20em') . ' (hex)'; //+Button vorhandene Passwörter convertieren

				$storeSessionPassword = new we_html_select(['name' => 'newconf[SECURITY_SESSION_PASSWORD]', 'class' => 'weSelect']);
				$storeSessionPassword->addOption(we_customer_customer::REMOVE_PASSWORD, g_l('prefs', '[security][storeSessionPassword][type][0]'));
				$storeSessionPassword->addOption(we_customer_customer::STORE_PASSWORD, g_l('prefs', '[security][storeSessionPassword][type][1]'));
				$storeSessionPassword->addOption(we_customer_customer::STORE_DBPASSWORD, g_l('prefs', '[security][storeSessionPassword][type][2]'));
				$storeSessionPassword->addOption(we_customer_customer::STORE_PASSWORD + we_customer_customer::STORE_DBPASSWORD, g_l('prefs', '[security][storeSessionPassword][type][3]'));
				$storeSessionPassword->selectOption(self::get_value('SECURITY_SESSION_PASSWORD'));

				$settings = [
					['headline' => g_l('perms_customer', '[perm_group_title]'), 'html' => $customer_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[security][encryption][title]'), 'html' => $encryption->getHtml() . $encryptinfo, 'space' => we_html_multiIconBox::SPACE_MED2,
						'noline' => true],
					['headline' => g_l('prefs', '[security][encryption][symmetricKey]'), 'html' => $encryptionKey, 'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[security][storeSessionPassword][title]'), 'html' => $storeSessionPassword->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[security][userPassRegex][title]'), 'html' => we_html_tools::textInput('newconf[SECURITY_USER_PASS_REGEX]', self::get_value('SECURITY_USER_PASS_REGEX'), 100, [
							], 'text', '20em'),
						'space' => we_html_multiIconBox::SPACE_MED2],
					['headline' => g_l('prefs', '[security][userPassRegex][desc]'), 'html' => we_html_tools::textInput('newconf[SECURITY_USER_PASS_DESC]', self::get_value('SECURITY_USER_PASS_DESC'), 100, [
							], 'text', '20em'),
						'space' => we_html_multiIconBox::SPACE_MED2],
				];
				return self::create_dialog('settings_security', $settings);

			case 'email':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}

				$emailSelect = we_html_tools::select('newconf[WE_MAILER]', ['php' => g_l('prefs', '[mailer_php]'), 'smtp' => g_l('prefs', '[mailer_smtp]')], 1, self::get_value('WE_MAILER'), false, [
						'onchange' => "var el = document.getElementById('smtp_table').style;var el2=document.getElementById('auth_table').style;var elAuth=document.getElementsByName('newconf[SMTP_AUTH]')[0]; if(this.value=='smtp'){ el.display='block'; el2.display=(elAuth.checked?'block':'none');}else{ el.display='none';el2.display='none';}"], true, 300);

				$smtp_table = new we_html_table(['class' => 'default defaultfont', 'id' => 'smtp_table', 'width' => 300, 'style' => 'display: ' . ((self::get_value('WE_MAILER') === 'php') ? 'none' : 'block') . ';'], 4, 2);
				$smtp_table->setCol(0, 0, ['style' => 'padding-right:10px;'], g_l('prefs', '[smtp_server]'));
				$smtp_table->setCol(0, 1, ['style' => 'text-align:right'], we_html_tools::textInput('newconf[SMTP_SERVER]', self::get_value('SMTP_SERVER'), 180, ['placeholder' => 'mail.example.com'], 'text', 180));
				$smtp_table->setColContent(1, 0, g_l('prefs', '[smtp_port]'));
				$smtp_table->setCol(1, 1, ['style' => 'text-align:right'], we_html_tools::textInput('newconf[SMTP_PORT]', self::get_value('SMTP_PORT'), 180, [], 'number', 180));
				$smtp_table->setCol(2, 0, ['style' => 'padding-right:10px;'], g_l('prefs', '[smtp_mailFrom]'));
				$smtp_table->setCol(2, 1, ['style' => 'text-align:right'], we_html_tools::textInput('newconf[SMTP_MAIL_SENDER]', self::get_value('SMTP_MAIL_SENDER'), 180, [
						'placeholder' => 'admin@mail.example.com'], 'email', 180));


				$encryptSelect = we_html_tools::select('newconf[SMTP_ENCRYPTION]', ['' => g_l('prefs', '[smtp_encryption_none]'), 'ssl' => g_l('prefs', '[smtp_encryption_ssl]'),
						'tls' => g_l('prefs', '[smtp_encryption_tls]')], 1, self::get_value('SMTP_ENCRYPTION'), false, [], true, 180);

				$smtp_table->setColContent(3, 0, g_l('prefs', '[smtp_encryption]'));
				$smtp_table->setCol(3, 1, ['style' => 'text-align:right'], $encryptSelect);
				$smtp_table->setCol(4, 0, ['colspan' => 3], we_html_forms::checkbox(1, self::get_value('SMTP_AUTH'), 'newconf[SMTP_AUTH]', g_l('prefs', '[smtp_auth]'), false, '', "var el2 = document.getElementById('auth_table').style; if(this.checked) el2.display='block'; else el2.display='none';"));

				$auth_table = new we_html_table(['class' => 'default defaultfont', 'id' => 'auth_table', 'width' => 200, 'style' => 'display: ' . ((self::get_value('SMTP_AUTH') == 1) ? 'block' : 'none') . ';'], 2, 2);
				$auth_table->setColContent(0, 0, g_l('prefs', '[smtp_username]'));
				$auth_table->setCol(0, 1, ['style' => 'text-align:right'], we_html_tools::textInput('newconf[SMTP_USERNAME]', self::get_value('SMTP_USERNAME'), 105, ['placeholder' => g_l('prefs', '[smtp_username]')], 'text', 180));
				$auth_table->setColContent(1, 0, g_l('prefs', '[smtp_password]'));
				$auth_table->setCol(1, 1, ['style' => 'text-align:right'], we_html_tools::textInput('newconf[SMTP_PASSWORD]', self::get_value('SMTP_PASSWORD') ? we_customer_customer::NOPWD_CHANGE : '', 105, [
						'placeholder' => g_l('prefs', '[smtp_password]')], 'password', 180));

				$settings = [
					['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[mailer_information]', false, true), we_html_tools::TYPE_INFO, 450, false),],
					['headline' => g_l('prefs', '[mailer_type]'), 'html' => $emailSelect, 'space' => we_html_multiIconBox::SPACE_MED, 'noline' => true],
					['headline' => '', 'html' => $smtp_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED, 'noline' => true],
					['headline' => '', 'html' => $auth_table->getHtml(), 'space' => we_html_multiIconBox::SPACE_MED, 'noline' => true],
				];


				return self::create_dialog('settings_email', $settings);

			case 'versions':
				if(!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
					break;
				}

				$versionsPrefs = [
					'ctypes' => [
						we_base_ContentTypes::IMAGE => 'VERSIONING_IMAGE',
						we_base_ContentTypes::HTML => 'VERSIONING_TEXT_HTML',
						we_base_ContentTypes::WEDOCUMENT => 'VERSIONING_TEXT_WEBEDITION',
						we_base_ContentTypes::JS => 'VERSIONING_TEXT_JS',
						we_base_ContentTypes::CSS => 'VERSIONING_TEXT_CSS',
						we_base_ContentTypes::TEXT => 'VERSIONING_TEXT_PLAIN',
						we_base_ContentTypes::HTACCESS => 'VERSIONING_TEXT_HTACCESS',
						we_base_ContentTypes::TEMPLATE => 'VERSIONING_TEXT_WETMPL',
						we_base_ContentTypes::FLASH => 'VERSIONING_FLASH',
						we_base_ContentTypes::VIDEO => 'VERSIONING_VIDEO',
						we_base_ContentTypes::AUDIO => 'VERSIONING_AUDIO',
						we_base_ContentTypes::PDF => 'VERSIONING_PDF',
						we_base_ContentTypes::APPLICATION => 'VERSIONING_SONSTIGE',
						we_base_ContentTypes::XML => 'VERSIONING_TEXT_XML',
						we_base_ContentTypes::OBJECT_FILE => 'VERSIONING_OBJECT',
					],
					'other' => [
						'VERSIONS_TIME_DAYS' => 'VERSIONS_TIME_DAYS',
						'VERSIONS_TIME_WEEKS' => 'VERSIONS_TIME_WEEKS',
						'VERSIONS_TIME_YEARS' => 'VERSIONS_TIME_YEARS',
						'VERSIONS_ANZAHL' => 'VERSIONS_ANZAHL',
						'VERSIONS_CREATE' => 'VERSIONS_CREATE',
						'VERSIONS_CREATE_TMPL' => 'VERSIONS_CREATE_TMPL',
						'VERSIONS_TIME_DAYS_TMPL' => 'VERSIONS_TIME_DAYS_TMPL',
						'VERSIONS_TIME_WEEKS_TMPL' => 'VERSIONS_TIME_WEEKS_TMPL',
						'VERSIONS_TIME_YEARS_TMPL' => 'VERSIONS_TIME_YEARS_TMPL',
						'VERSIONS_ANZAHL_TMPL' => 'VERSIONS_ANZAHL_TMPL'
					]
				];

				$_SESSION['weS']['versions']['logPrefs'] = [];
				foreach($versionsPrefs as $v){
					foreach($v as $val){
						$_SESSION['weS']['versions']['logPrefs'][$val] = self::get_value($val);
					}
				}

				$checkboxes = we_html_forms::checkbox(1, false, 'version_all', g_l('prefs', '[version_all]'), false, '', 'checkAll(this);') . '<br/>';

				foreach($versionsPrefs['ctypes'] as $k => $v){
					$checkboxes .= we_html_forms::checkbox(1, self::get_value($v), 'newconf[' . $v . ']', g_l('contentTypes', '[' . $k . ']'), false, '', 'checkAllRevert(this);') . '<br/>';
				}

				$versions_time_days = new we_html_select(['name' => 'newconf[VERSIONS_TIME_DAYS]', 'class' => 'weSelect']);

				$versions_time_days->addOption(-1, '');
				$versions_time_days->addOption(we_base_util::SECONDS_DAY, g_l('prefs', '[1_day]'));
				for($x = 2; $x <= 31; $x++){
					$versions_time_days->addOption(($x * we_base_util::SECONDS_DAY), sprintf(g_l('prefs', '[more_days]'), $x));
				}
				$versions_time_days->selectOption(self::get_value('VERSIONS_TIME_DAYS'));


				$versions_time_weeks = new we_html_select(['name' => 'newconf[VERSIONS_TIME_WEEKS]', 'class' => 'weSelect']);
				$versions_time_weeks->addOption(-1, '');
				$versions_time_weeks->addOption(we_base_util::SECONDS_WEEK, g_l('prefs', '[1_week]'));
				for($x = 2; $x <= 52; $x++){
					$versions_time_weeks->addOption(($x * we_base_util::SECONDS_WEEK), sprintf(g_l('prefs', '[more_weeks]'), $x));
				}
				$versions_time_weeks->selectOption(self::get_value('VERSIONS_TIME_WEEKS'));


				$versions_time_years = new we_html_select(['name' => 'newconf[VERSIONS_TIME_YEARS]', 'class' => 'weSelect']);
				$versions_time_years->addOption(-1, '');
				$versions_time_years->addOption(we_base_util::SECONDS_YEAR, g_l('prefs', '[1_year]'));
				for($x = 2; $x <= 10; $x++){
					$versions_time_years->addOption(($x * we_base_util::SECONDS_YEAR), sprintf(g_l('prefs', '[more_years]'), $x));
				}
				$versions_time_years->selectOption(self::get_value('VERSIONS_TIME_YEARS'));
				$versions_anzahl = we_html_tools::textInput('newconf[VERSIONS_ANZAHL]', self::get_value('VERSIONS_ANZAHL'), 5, [], 'number', 50, 0, '');

				$versions_create_publishing = we_html_forms::radiobutton(1, (self::get_value('VERSIONS_CREATE') == 1), 'newconf[VERSIONS_CREATE]', g_l('prefs', '[versions_create_publishing]'), true, '', '', false, '');
				$versions_create_always = we_html_forms::radiobutton(0, (self::get_value('VERSIONS_CREATE') == 0), 'newconf[VERSIONS_CREATE]', g_l('prefs', '[versions_create_always]'), true, '', '', false, '');

				$versions_time_days_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_DAYS_TMPL]', 'class' => 'weSelect']);

				$versions_time_days_tmpl->addOption(-1, '');
				$versions_time_days_tmpl->addOption(we_base_util::SECONDS_DAY, g_l('prefs', '[1_day]'));
				for($x = 2; $x <= 31; $x++){
					$versions_time_days_tmpl->addOption(($x * we_base_util::SECONDS_DAY), sprintf(g_l('prefs', '[more_days]'), $x));
				}
				$versions_time_days_tmpl->selectOption(self::get_value('VERSIONS_TIME_DAYS_TMPL'));


				$versions_time_weeks_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_WEEKS_TMPL]', 'class' => 'weSelect']);
				$versions_time_weeks_tmpl->addOption(-1, '');
				$versions_time_weeks_tmpl->addOption(we_base_util::SECONDS_WEEK, g_l('prefs', '[1_week]'));
				for($x = 2; $x <= 52; $x++){
					$versions_time_weeks_tmpl->addOption(($x * we_base_util::SECONDS_WEEK), sprintf(g_l('prefs', '[more_weeks]'), $x));
				}
				$versions_time_weeks_tmpl->selectOption(self::get_value('VERSIONS_TIME_WEEKS_TMPL'));

				$versions_time_years_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_YEARS_TMPL]', 'class' => 'weSelect']);
				$versions_time_years_tmpl->addOption(-1, '');
				$versions_time_years_tmpl->addOption(we_base_util::SECONDS_YEAR, g_l('prefs', '[1_year]'));
				for($x = 2; $x <= 10; $x++){
					$versions_time_years_tmpl->addOption(($x * we_base_util::SECONDS_YEAR), sprintf(g_l('prefs', '[more_years]'), $x));
				}
				$versions_time_years_tmpl->selectOption(self::get_value('VERSIONS_TIME_YEARS_TMPL'));
				$versions_anzahl_tmpl = we_html_tools::textInput('newconf[VERSIONS_ANZAHL_TMPL]', self::get_value('VERSIONS_ANZAHL_TMPL'), 5, [], 'number', 50, 0, '');
				$versions_create_tmpl_publishing = we_html_forms::radiobutton(1, (self::get_value('VERSIONS_CREATE_TMPL') == 1), 'newconf[VERSIONS_CREATE_TMPL]', g_l('prefs', '[versions_create_tmpl_publishing]'), true, '', '', false, '');
				$versions_create_tmpl_always = we_html_forms::radiobutton(0, (self::get_value('VERSIONS_CREATE_TMPL') == 0), 'newconf[VERSIONS_CREATE_TMPL]', g_l('prefs', '[versions_create_tmpl_always]'), true, '', '', false, '');

				return self::create_dialog('', [
						['headline' => g_l('prefs', '[ContentType]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_activate_text]'), we_html_tools::TYPE_HELP),
							'space' => we_html_multiIconBox::SPACE_BIG,
							'html' => $checkboxes
						],
						['html' => $versions_time_days->getHtml() . ' ' . $versions_time_weeks->getHtml() . ' ' . $versions_time_years->getHtml(),
							'space' => we_html_multiIconBox::SPACE_BIG,
							'headline' => g_l('prefs', '[versioning_time]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_time_text]'), we_html_tools::TYPE_HELP)
						],
						['headline' => g_l('prefs', '[versioning_anzahl]'),
							'html' => $versions_anzahl,
							'space' => we_html_multiIconBox::SPACE_BIG
						],
						['headline' => g_l('prefs', '[versioning_create]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_create_text]'), we_html_tools::TYPE_HELP, false, false),
							'html' => $versions_create_publishing . '<br/>' . $versions_create_always,
							'space' => we_html_multiIconBox::SPACE_BIG
						],
						['html' => we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_templates_text]'), we_html_tools::TYPE_INFO, 470, false),
							'noline' => true,
						],
						['html' => $versions_time_days_tmpl->getHtml() . ' ' . $versions_time_weeks_tmpl->getHtml() . ' ' . $versions_time_years_tmpl->getHtml(),
							'space' => we_html_multiIconBox::SPACE_BIG,
							'noline' => true,
							'headline' => g_l('prefs', '[versioning_time]')
						],
						['headline' => g_l('prefs', '[versioning_anzahl]'),
							'html' => $versions_anzahl_tmpl,
							'noline' => true,
							'space' => we_html_multiIconBox::SPACE_BIG
						],
						['headline' => g_l('prefs', '[versioning_create]'),
							'html' => $versions_create_tmpl_publishing . '<br/>' . $versions_create_tmpl_always,
							'space' => we_html_multiIconBox::SPACE_BIG
						],
						/* ['headline' => g_l('prefs', '[versioning_wizard]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_wizard_text]'), we_html_tools::TYPE_HELP),
						  'html' => '<div style="float:left;">' . we_html_button::create_button('openVersionWizard', 'javascript:openVersionWizard()') . '</div>',
						  'space' => we_html_multiIconBox::SPACE_BIG
						  ], */
						], -1, '', '');
		}

		return 'No rights.';
	}

	/**
	 * This functions renders the complete dialog.
	 *
	 * @return         string
	 */
	private static function render_dialog(){
		$tabname = we_base_request::_(we_base_request::STRING, 'tabname', 'setting_ui');

		// Check configuration file for all needed variables => since included in startup, nothing should change
		we_base_preferences::check_global_config();
		$tabs = array_keys(self::TABS);
		$tabs[] = 'save';
		$ret = '';

		foreach($tabs as $tab){
			$ret .= we_html_element::div(['id' => 'setting_' . $tab, 'style' => ($tabname === 'setting_' . $tab ? '' : 'display: none;')], self::build_dialog($tab));
		}
		return $ret;
	}

	private static function saveSettings(){
		$acError = false;
		$acErrorMsg = '';

		// check seemode start document | object
		switch(we_base_request::_(we_base_request::STRING, 'seem_start_type')){
			case 'document':
				$doc = we_base_request::_(we_base_request::INT, 'seem_start_document', 0);
				if(!$doc){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				} else {
					$folder = f('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . intval($doc));
					if($folder === '' || $folder == 1){
						$acError = true;
						$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
					}
				}
				break;
			case 'weapp':
				$app = we_base_request::_(we_base_request::STRING, 'newconf', '', 'seem_start_weapp');
				if(!$app){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				}
				break;
			case 'object':
				$obj = we_base_request::_(we_base_request::INT, 'seem_start_object', 0);
				if(!$obj){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				} else {
					$folder = f('SELECT IsFolder FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($obj));
					if($folder === '' || $folder == 1){
						$acError = true;
						$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
					}
				}
				break;
		}
		// check sidebar document
		if(!we_base_request::_(we_base_request::BOOL, 'newconf', false, 'SIDEBAR_DISABLED') && we_base_request::_(we_base_request::FILE, 'ui_sidebar_file_name')){
			$folder = f('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::_(we_base_request::INT, 'newconf', 0, 'SIDEBAR_DEFAULT_DOCUMENT'));
			if($folder === '' || $folder == 1){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[sidebar]') . ' / ' . g_l('prefs', '[sidebar_document]'), g_l('prefs', '[tab][ui]')) . "\\n";
			}
		}
		// check doc for error on none existing objects
		if(we_base_request::_(we_base_request::FILE, 'error_document_no_objectfile_text')){
			$folder = f('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::_(we_base_request::INT, 'newconf', 0, 'ERROR_DOCUMENT_NO_OBJECTFILE'));
			if($folder === '' || $folder == 1){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[error_no_object_found]'), g_l('prefs', '[tab][error_handling]')) . "\\n";
			}
		}
		// check if versioning number is correct
		if(($cnt = we_base_request::_(we_base_request::INT, 'newconf', 0, 'VERSIONS_ANZAHL'))){
			if(!(abs($cnt) == $cnt && $cnt > 0)){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[versioning_anzahl]'), g_l('prefs', '[tab][versions]')) . "\\n";
			}
		}

		return [true, $acError, $acErrorMsg];
	}

	public static function show(){
		self::$jsCmd = new we_base_jsCmd();
// Check if we need to save settings
		list($doSave, $acError, $acErrorMsg) = (we_base_request::_(we_base_request::BOOL, 'save_settings') ? self::saveSettings() : [false, false, '']);


		if($doSave && !$acError){
			self::save_all_values();
			self::$jsCmd->addMsg(g_l('prefs', '[saved]'), we_base_util::WE_MESSAGE_NOTICE);
			//	self::$jsCmd->addMsg(g_l('prefs', '[error_mail_not_saved]'), we_base_util::WE_MESSAGE_ERROR);

			self::$jsCmd->addCmd('updatePrefs');
			$body = we_html_element::body(['class' => 'weDialogBody'], self::build_dialog('saved')) . '</html>';
		} else {
			$form = we_html_element::form(['onSubmit' => 'return false;', 'name' => 'we_form', 'method' => 'post', 'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=editor_preferences'], we_html_element::hidden('save_settings', 0) . self::render_dialog());

			if($acError){
				self::$jsCmd->addMsg(g_l('alert', '[field_in_tab_notvalid_pre]') . "\\n\\n" . $acErrorMsg . "\\n" . g_l('alert', '[field_in_tab_notvalid_post]'), we_base_util::WE_MESSAGE_ERROR);
			}

			$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'startPrefs();'], $form);
		}


		echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'preferences.js') . self::$jsCmd->getCmds(), $body);
	}

}
