<?php

/**
 * webEdition CMS
 *
 * $Rev: a2003b43d6bf $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-06-21 12:06 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_html_multiIconBox{
	const SPACE_SMALL = 'small';
	const SPACE_ICON = 'icon';
	const SPACE_MED = 'med';
	const SPACE_MED2 = 'med2';
	const SPACE_BIG = 'big';
	const PROP_ATTRIB = '<i class="fa fa-cog"></i>';
	const PROP_AUTONAME = '<i class="fa fa-font"></i>';
	const PROP_BANNER = '<i class="fa fa-flag"></i>';
	const PROP_CATEGORIES = '<i class="fa fa-tags"></i>';
	const PROP_CHARSET = '<i class="fa fa-paragraph"></i>';
	const PROP_CLASS = '<i class="fa fa-code"></i>';
	const PROP_CONTENT = '<i class="fa fa-inbox"></i>';
	const PROP_COPY = '<i class="fa fa-copy"></i>';
	const PROP_CUSTOMER = '<i class="fa fa-users"></i>';
	const PROP_DISPLAY = '<i class="fa fa-eye"></i>';
	const PROP_DOC = '<i class="fa fa-file-text"></i>';
	const PROP_FILTER = '<i class="fa fa-filter"></i>';
	const PROP_IMAGE = '<i class="fa fa-image"></i>';
	const PROP_KEYBOARD = '<i class="fa fa-keyboard-o"></i>';
	const PROP_LANG = '<i class="fa fa-language"></i>';
	const PROP_LINK = '<i class="fa fa-link"></i>';
	const PROP_MAIL = '<i class="fa fa-envelope"></i>';
	const PROP_MASTER = '<i class="fa fa-sitemap"></i>';
	const PROP_META = '<i class="fa fa-plus-circle"></i>';
	const PROP_NAVI = '<i class="fa fa-compass"></i>';
	const PROP_NEWSLETTER = '<i class="fa fa-newspaper-o"></i>';
	const PROP_OTHER = '<i class="fa fa-magic"></i>';
	const PROP_PATH = '<i class="fa fa-folder"></i>';
	const PROP_REVREL = '<i class="fa fa-exchange"></i>';
	const PROP_SEO = '<i class="fa fa-search-plus"></i>';
	const PROP_SORT = '<i class="fa fa-sort"></i>';
	const PROP_STATISTIC = '<i class="fa fa-bar-chart"></i>';
	const PROP_TIME = '<i class="fa fa-clock-o"></i>';
	const PROP_UPLOAD = '<i class="fa fa-upload"></i>';
	const PROP_USER = '<i class="fa fa-user"></i>';
	const PROP_WINDOW = '<i class="fa fa-window-maximize"></i>';
	const PROP_WORKSPACE = '<i class="fa fa-gavel"></i>';
	//INFO
	const INFO_CALENDAR = '<i class="fa fa-calendar"></i>';
	const INFO_WORKFLOW = '<i class="fa fa-gears"></i>';
	const INFO_REFERENCES = '<i class="fa fa-shield"></i>';
	//CUSTOMER
	const CUSTOMER_FILTER = '<i class="fa fa-filter"></i>';

	/**
	 * @desc 	Get HTML-Code of the multibox
	 * @param	$name				string
	 * @param	$width				int
	 * @param	$content			array content is ['icon'=>'','headline'=>'','space'=>'','rightHeadline'=>false,'html'=>'','noline'=>false,'fold'=>false,'preline'=>false,'hide'=>false,'class'=>'']
	 * @param	$buttons			string
	 * @param	$foldAtNr			int
	 * @param	$foldRight			unknown
	 * @param	$foldDown			unknown
	 * @param	$displayAtStartup	bool
	 * @param	$headline			string
	 * @return	string
	 */
	public static function getHTML($name, array $content, $marginLeft = false, $buttons = '', $foldAtNr = -1, $foldRight = '', $foldDown = '', $displayAtStartup = false, $headline = '', $delegate = ''){
		$uniqname = $name ?: md5(uniqid(__FILE__, true));
		$fold = '';

		$out = $headline ?
			self::getBoxStartHeadline($name, $headline, $marginLeft) :
			self::getBoxStart($name);

		foreach($content as $i => $c){
			if($c === null || !empty($c['hide'])){
				continue;
			}
			$rightHeadline = (!empty($c['rightHeadline']));
			$icon = (empty($c['icon']) ? '' : $c['icon']);
			$headline = (empty($c['headline']) ? '' : $c['headline'] );
			$leftWidth = (empty($c['space']) ? ($icon ? self::SPACE_ICON : '') : $c['space'] );
			$leftContent = $icon ?: ($leftWidth && !$rightHeadline && $headline ? we_html_element::div(['class' => 'weMultiIconBoxHeadline'], $headline) : '');
			$rightHead = (($icon || !$leftContent || $rightHeadline) && $headline ? we_html_element::div(['class' => 'weMultiIconBoxHeadline'], $headline) : '');

			if($i === $foldAtNr && $foldAtNr < count($content)){// only if the folded items contain stuff.
				$fold = ' ';
				$foldSpace = $leftWidth;
			}


			$tmp = we_html_element::div(['class' => 'weMultiIconBoxContent' . (empty($c['preline']) ? '' : ' weMultiIconBoxPreLine') . ($i < (count($content) - 1) && (empty($c['noline'])) ? ' weMultiIconBoxLine' : '' ) . ($marginLeft ? ' withMargin' : '') . (empty($c['class']) ? '' : ' ' . $c['class']),
					'id' => 'div_' . $uniqname . '_' . $i], ($leftContent || $leftWidth ? we_html_element::div(['class' => 'multiiconleft leftSpace-' . $leftWidth], ((!$leftContent) && $leftWidth ? '' : $leftContent)) : '') .
					//right
					we_html_element::div(['class' => 'multiIconRight'], (empty($c['html']) ? '' :
						(empty($c['fold']) ? $rightHead . we_html_element::div([], $c['html']) :
						self::getFold(we_html_element::div([], $c['html']), $uniqname . '_' . $i, false, $rightHead ? $headline : '', $rightHead ? $headline : '')
						)
						)
					)
			);

			if($fold){
				$fold .= $tmp;
			} else {
				$out .= $tmp;
			}
		}

		if($fold){
			$out .= self::getFold($fold, $uniqname, $displayAtStartup, $foldDown, $foldRight, $delegate, $marginLeft, $foldSpace);
		}

		$out .= '</div>';

		return ($buttons ?
			//ignore height, replace by bottom:
			we_html_element::div(['class' => 'weMultiIconBoxWithFooter'], $out) .
			we_html_element::div(['class' => 'editfooter'], $buttons) :
			$out);
	}

	private static function getFold($content, $uniqname, $displayAtStartup, $foldDown = '', $foldRight = '', $moreJS = '', $marginLeft = '', $foldSpace = ''){
		static $pos = 0;
		$displayAtStartup = we_base_request::_(we_base_request::BOOL, 'fold', $displayAtStartup, $pos);

		$control = we_html_element::div(['class' => 'btn_direction_weMultibox_table' . ($marginLeft ? ' withMargin' : '')], self::getButton($uniqname, "weToggleBox('" . $uniqname . "');" . ($moreJS ?: '' ), ($displayAtStartup ? 'down' : 'right'), g_l('global', '[openCloseBox]')) .
				we_html_element::div(['class' => 'weMultiIconBoxHeadline toggleBox', 'id' => 'textRight_' . $uniqname, 'style' => 'display:' . ($displayAtStartup ? 'none' : '') . ';'], we_html_element::label([
						'for' => 'btn_direction' . $uniqname], $foldRight)) .
				we_html_element::div(['class' => 'weMultiIconBoxHeadline toggleBox', 'id' => 'textDown_' . $uniqname, 'style' => 'display:' . ($displayAtStartup ? '' : 'none') . ';'], we_html_element::label([
						'for' => 'btn_direction' . $uniqname], $foldDown))
			) .
			we_html_element::hidden('fold[' . $pos . ']', intval($displayAtStartup), 'hidden_' . $uniqname);

		$pos++;
		return ($foldSpace ? we_html_element::div(['class' => 'multiiconleft leftSpace-' . $foldSpace], '') . we_html_element::div(['class' => 'multiIconRight'], $control) : $control) .
			we_html_element::div(['class' => "default iconBoxTable", 'id' => 'table_' . $uniqname, 'style' => ($displayAtStartup ? '' : 'display:none')], $content);
	}

	//FIXME: is this really needed?
	static function getDynJS($uniqname = '', $marginLeft = 0){
		return we_html_element::jsScript(JS_DIR . 'multiIconBox.js', '', ['id' => 'loadVarMultiIconBox', 'data-iconbox' => setDynamicVar([
					'name' => $uniqname,
					'margin' => $marginLeft,
		])]);
	}

	public static function addBoxContainer($uniqname, $name, $data){
		return self::getBoxStart($name) . $data . '</div>';
	}

	private static function getBoxStartHeadline($name, $headline, $marginLeft = false){
		return self::getBoxStart($name) .
			we_html_element::div(['class' => 'weDialogHeadline' . ($marginLeft ? ' withMargin' : '')], $headline);
	}

	private static function getBoxStart($name = ''){
		return '<div class="default multiIcon defaultfont"' . ($name ? ' id="' . $name . '"' : '') . '>';
	}

	private static function getButton($name, $cmd, $state = "right", $title = ''){
		return we_html_button::create_button(($state == 'right' ? we_html_button::DIRRIGHT : we_html_button::DIRDOWN), "javascript:" . $cmd . ";", '', '', false, false, '', false, $title, '', 'btn_direction' . $name);
	}

}
