<?php

/**
 * webEdition CMS
 *
 * $Rev: 13ed029447f6 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-02-23 18:58 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_captcha extends we_weTag_tag{

	public static function parse(array $attribs, $content){
		if(($foo = self::attributeMissing($attribs, ['width' => false, 'height' => false], __CLASS__))){
			return $foo;
		}

		$width = self::getParserAttribute('width', $attribs, 100);
		$height = self::getParserAttribute('height', $attribs, 25);
		$path = self::getParserAttribute('path', $attribs, '/');

		$maxlength = self::getParserAttribute('maxlength', $attribs, 5);
		$type = self::getParserAttribute('type', $attribs, 'gif');

		$font = self::getParserAttribute('font', $attribs);
		$fontpath = self::getParserAttribute('fontpath', $attribs, '');
		$fontsize = self::getParserAttribute('fontsize', $attribs, 14);
		$fontcolor = self::getParserAttribute('fontcolor', $attribs, '#000000');

		$angle = self::getParserAttribute('angle', $attribs, 0);

		$subset = self::getParserAttribute('subset', $attribs, 'alphanum');
		$case = self::getParserAttribute('case', $attribs, 'mix');
		$skip = self::getParserAttribute('skip', $attribs, 'i,I,l,L,0,o,O,1,g,9');

		$valign = self::getParserAttribute('valign', $attribs, 'random');
		$align = self::getParserAttribute('align', $attribs, 'random');

		$bgcolor = self::getParserAttribute('bgcolor', $attribs, '#ffffff');
		$transparent = self::getParserAttribute('transparent', $attribs, false, true);

		$style = self::getParserAttribute('style', $attribs, '');
		$stylecolor = self::getParserAttribute('stylecolor', $attribs, '#cccccc');
		$stylenumber = self::getParserAttribute('stylenumber', $attribs, '5,10');

		// writing the temporary document
		$file = 'we_captcha_' . $GLOBALS['we_doc']->ID . '.php';
		$realPath = rtrim(realpath($_SERVER['DOCUMENT_ROOT'] . $path), '/') . '/' . $file;
		we_base_file::deleteLocalFile($realPath);


		$GLOBALS['DB_WE']->query('REPLACE INTO ' . CAPTCHADEF_TABLE . ' SET ' . we_database_base::arraySetter([
				'ID' => $GLOBALS['we_doc']->ID,
				'width' => $width,
				'height' => $height,
				'maxlength' => $maxlength,
				'fontpath' => $fontpath,
				'font' => $font,
				'fontsize' => $fontsize,
				'fontcolor' => $fontcolor,
				'subset' => $subset,
				'case' => $case,
				'skip' => $skip,
				'align' => $align,
				'valign' => $valign,
				'bgcolor' => $bgcolor,
				'transparent' => intval($bgcolor && $transparent),
				'style' => $style,
				'stylecolor' => $stylecolor,
				'stylenumber' => $stylenumber,
				'angle' => $angle,
				'type' => $bgcolor && $transparent ? 'gif' : $type
		]));

		// clean attribs
		$attribs = removeAttribs($attribs, [
			'path',
			'maxlength',
			'type',
			'font',
			'fontpath',
			'fontsize',
			'fontcolor',
			'angle',
			'subset',
			'case',
			'skip',
			'align',
			'valign',
			'bgcolor',
			'transparent',
			'style',
			'stylecolor',
			'stylenumber',
			'src'
		]);

		return '<?= ' . we_weTag_parser::printTag('captcha', $attribs) . ';?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		static $captcha = [];
		$tid = empty($GLOBALS['we_doc']) ? (empty($attribs['src']) ? 0 : intval($attribs['src'])) : $GLOBALS['we_doc']->TemplateID;
		if(!isset($attribs['alt'])){
			$attribs['alt'] = 'captcha';
		}

		if(!empty($GLOBALS['we_doc']) && !$GLOBALS['we_doc']->IsDynamic){
			//static documents link to dynamic source
			$attribs['src'] = WEBEDITION_DIR . 'showCaptcha.php?id=' . $tid;
			return getHtmlTag('img', $attribs);
		}

		if($tid){
			if(empty($captcha[$tid])){
				$attr = getHash('SELECT * FROM ' . CAPTCHADEF_TABLE . ' WHERE ID=' . $tid) ?: getHash('SELECT * FROM ' . CAPTCHADEF_TABLE . '  LIMIT 1');

				$image = new we_captcha_image($attr['width'], $attr['height'], $attr['maxlength']);
				if($attr['fontpath']){
					$image->setFontPath($attr['fontpath']);
				}
				$image->setFont($attr['font'], $attr['fontsize'], $attr['fontcolor']);
				$image->setCharacterSubset($attr['subset'], $attr['case'], $attr['skip']);
				$image->setAlign($attr['align']);
				$image->setVerticalAlign($attr['valign']);
				$image->setBackground($attr['bgcolor'], $attr['transparent']);
				$image->setStyle($attr['style'], $attr['stylecolor'], $attr['stylenumber']);
				$image->setAngleRange($attr['angle']);
				if(empty($GLOBALS['we_doc'])){ //called via we_showCaptcha
					return we_captcha_captcha::display($image, $attr['type']);
				}

				list($type, $data) = we_captcha_captcha::get($image, $attr['type']);
				$captcha[$tid] = $type . ';base64,' . base64_encode($data);
			}
			$attribs['src'] = 'data:' . $captcha[$tid];
			return getHtmlTag('img', $attribs);
		}
		return '';
	}

}

function we_tag_captcha(array $attribs){
	return we_tag_captcha::tag($attribs);
}
