<?php

/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_form extends we_weTag_tag{
	public static $name = null;

	public static function parse(array $attribs, $content){
		return '<?= ' . we_weTag_parser::printTag('form', array_change_key_case($attribs)) . ';
?>' . $content .
			'<?= ' . we_weTag_parser::printTag('form', ['_type' => 'stop']) . ';?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!empty($GLOBALS['we']['editmode'])){
			return'';
		}
		if(self::getAttribute('_type', $attribs, '', we_base_request::STRING) === 'stop'){
			unset($GLOBALS['WE_FORM']);
			self::$name = null;
			return we_tag('formToken') . '</form>';
		}
		$ret = '';
		$method = self::getAttribute('method', $attribs, 'post', we_base_request::STRING);
		$id = self::getAttribute('id', $attribs, 0, we_base_request::STRING);
		$action = self::getAttribute('action', $attribs, '', we_base_request::URL);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$parentid = self::getAttribute('parentid', $attribs, 0, we_base_request::INT);
		$doctype = self::getAttribute('doctype', $attribs, '', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
		$categories = self::getAttribute('categories', $attribs, '', we_base_request::WEFILELIST);
		$onsubmit = self::getAttribute('onsubmit', $attribs, '', we_base_request::JS);
		$remove = self::getAttribute('remove', $attribs, [], we_base_request::STRING_LIST);
		$xml = self::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);
		$formname = self::getAttribute('name', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
		$params = self::getAttribute('params', $attribs, '', we_base_request::STRING);
		if(array_key_exists('nameid', $attribs)){ // Bug #3153
			$formname = self::getAttribute('nameid', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
			$attribs['pass_id'] = $formname;
			unset($attribs['nameid']);
		}
		$captchaname = self::getAttribute('captchaname', $attribs, '', we_base_request::STRING);
		$enctype = self::getAttribute('enctype', $attribs, '', we_base_request::STRING);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$formAttribs = removeAttribs($attribs, [
			'onsubmit', 'name', 'method', 'xml', 'charset', 'id', 'action',
			'order', 'required', 'onsuccess', 'onerror', 'type', 'recipient', 'mimetype',
			'subject', 'onmailerror', 'preconfirm', 'postconfirm', 'from', 'confirmmail',
			'classid', 'doctype', 'remove', 'onrecipienterror', 'tid', 'forcefrom', 'categories',
			'params'
		]);

		$formAttribs['xml'] = $xml;
		$formAttribs['method'] = $method;

		$we_form_action = ($id ?
			($id === 'self' || ($id == 0 && defined('WE_REDIRECTED_SEO')) ? (defined('WE_REDIRECTED_SEO') ? WE_REDIRECTED_SEO : getScriptName()) : f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id))) :
			($action ?: getScriptName())) . $params;

		if($type != 'search'){
			$regs = [];
			if(preg_match('/^(.*)return (.+)$/i', $onsubmit, $regs)){
				$onsubmit = $regs[1] . ';return ' . $regs[2];
			} else {
				$onsubmit .= ';return true;';
			}
		}
		switch($type){
			case 'shopliste' :
				$formAttribs['action'] = $we_form_action;
				$articleID = (we_tag_listview::$current && we_tag_listview::$current->f(we_listview_base::ID_FIELD)) ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) : (isset($GLOBALS['we_obj']->ID) ? $GLOBALS['we_obj']->ID : $GLOBALS['we_doc']->ID);
				$formAttribs['name'] = 'form' . $articleID;
				if(empty($GLOBALS['we']['editmode'])){
					$ret = getHtmlTag('form', $formAttribs, '', false, true) .
						getHtmlTag('input', ['xml' => $xml, 'type' => 'hidden', 'name' => 'type',
							'value' => (
							(we_tag_listview::$current instanceof we_listview_objectBase) ?
							we_shop_shop::OBJECT :
							(we_tag_listview::$current && we_tag_listview::$current instanceof we_listview_document ?
							we_shop_shop::DOCUMENT :
							(isset($GLOBALS['we_obj']->ID) ?
							we_shop_shop::OBJECT :
							we_shop_shop::DOCUMENT
							)
							))
							]
						) .
						getHtmlTag('input', ['xml' => $xml, 'type' => 'hidden', 'name' => 'shop_artikelid',
							'value' => $articleID
						]) .
						getHtmlTag('input', [
							'xml' => $xml,
							'type' => 'hidden',
							'name' => we_base_variants::REQUEST,
							'value' => $GLOBALS['we_doc']->Variant,
						]) .
						getHtmlTag('input', ['xml' => $xml, 'type' => 'hidden', 'name' => 't', 'value' => time(),]);
				}
				break;
			case 'object' :
			case 'document' :
				if(!(isset($_REQUEST['edit_' . $type]) && empty($GLOBALS['we']['sessionStart'])) && isset($_SESSION)){
					unset($_SESSION['weS']['we_' . $type . '_session_' . $formname]);
				}

				$formAttribs['onsubmit'] = $onsubmit;
				$formAttribs['name'] = $formname;
				$formAttribs['action'] = $we_form_action;

				if($enctype){
					$formAttribs['enctype'] = $enctype;
				}
				if($target){
					$formAttribs['target'] = $target;
				}
				if($classid || $doctype){
					$GLOBALS['WE_FORM'] = self::$name = $formname;
					if(!$GLOBALS['we_doc']->InWebEdition){
						switch($type){
							case 'object':
								$id = we_base_request::_(we_base_request::INT, 'we_editObject_ID', 0);
								if(f('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classid))){
									we_objectFile::initObject($classid, $formname, $categories, intval($parentid), $id);
								}
								break;
							default:
								$id = we_base_request::_(we_base_request::INT, 'we_editDocument_ID', 0);
								we_webEditionDocument::initDocument($formname, $tid, $doctype, $categories, $id);
						}
					}
					$typetmp = ($type === 'object') ? 'Object' : 'Document';

					if(empty($GLOBALS['we']['editmode'])){
						$ret .= getHtmlTag('form', $formAttribs, '', false, true) .
							getHtmlTag('input', ['type' => 'hidden', 'name' => 'edit_' . $type, 'value' => 1, 'xml' => $xml]) .
							getHtmlTag('input', ['type' => 'hidden', 'name' => 'we_edit' . $typetmp . '_ID', 'xml' => $xml,
								'value' => we_base_request::_(we_base_request::INT, 'we_edit' . $typetmp . '_ID', 0),
						]);
					}
				} elseif(!isset($GLOBALS['we']['editmode']) || !$GLOBALS['we']['editmode']){
					$ret .= getHtmlTag('form', $formAttribs, '', false, true);
				}
				break;
			case 'formmail' :
				$confirmmail = self::getAttribute('confirmmail', $attribs, false, we_base_request::BOOL);
				$preconfirm = $confirmmail ? self::getAttribute('preconfirm', $attribs, '', we_base_request::STRING) : '';
				$postconfirm = $confirmmail ? self::getAttribute('postconfirm', $attribs, '', we_base_request::STRING) : '';
				$onsuccess = self::getAttribute('onsuccess', $attribs, 0, we_base_request::INT);
				$onerror = self::getAttribute('onerror', $attribs, 0, we_base_request::INT);
				$onmailerror = self::getAttribute('onmailerror', $attribs, 0, we_base_request::INT);
				$onrecipienterror = self::getAttribute('onrecipienterror', $attribs, 0, we_base_request::INT);
				$oncaptchaerror = self::getAttribute('oncaptchaerror', $attribs, 0, we_base_request::INT);
				$recipient = self::getAttribute('recipient', $attribs, '', we_base_request::EMAILLIST);

				if($enctype){
					$formAttribs['enctype'] = $enctype;
				}
				if($target){
					$formAttribs['target'] = $target;
				}

				$formAttribs['method'] = 'post'; //don't allow anything else
				$formAttribs['name'] = $formname;
				$formAttribs['onsubmit'] = $onsubmit;
				$formAttribs['action'] = ($id === 'self' ? (defined('WE_REDIRECTED_SEO') ? WE_REDIRECTED_SEO : getScriptName()) : ($id ? f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id)) : WEBEDITION_DIR . 'we_formmail.php'));


				//  now prepare all needed hidden-fields:
				if(empty($GLOBALS['we']['editmode'])){
					$recipientArray = explode(',', $recipient);
					foreach($recipientArray as $key => $val){
						$recipientArray[$key] = '"' . trim($val) . '"';
					}

					$ids = $GLOBALS['DB_WE']->getAllq('SELECT ID FROM ' . RECIPIENTS_TABLE . ' WHERE Email IN(' . implode(',', $recipientArray) . ')', true);

					$ret = getHtmlTag('form', $formAttribs, '', false, true);
					$data = array_filter([
						'order' => self::getAttribute('order', $attribs, '', we_base_request::STRING),
						'required' => self::getAttribute('required', $attribs, '', we_base_request::STRING),
						'subject' => self::getAttribute('subject', $attribs, '', we_base_request::STRING),
						'recipient' => ($ids ? implode(',', $ids) : ''),
						'mimetype' => self::getAttribute('mimetype', $attribs, '', we_base_request::STRING),
						'from' => self::getAttribute('from', $attribs, WE_DEFAULT_EMAIL, we_base_request::EMAIL),
						'error_page' => $onerror ?: 0,
						'mail_error_page' => $onmailerror ? $onmailerror : 0,
						'recipient_error_page' => $onrecipienterror ? $onrecipienterror : '',
						'ok_page' => $onsuccess ? $onsuccess : 0,
						'charset' => self::getAttribute('charset', $attribs, $GLOBALS['we']['Charset']['Backend'], we_base_request::STRING),
						'confirm_mail' => $confirmmail,
						'pre_confirm' => $GLOBALS['we_doc']->getElement($preconfirm),
						'post_confirm' => $GLOBALS['we_doc']->getElement($postconfirm),
						'we_remove' => $remove,
						'forcefrom' => self::getAttribute('forcefrom', $attribs, '', we_base_request::STRING),
						'captcha_error_page' => $oncaptchaerror ? $oncaptchaerror : 0,
						'captchaname' => $captchaname,
					]);
					$ret .= getHtmlTag('input', ['type' => 'hidden', 'name' => 'data-jwt', 'value' => we_helpers_jwt::encode(we_serialize($data, SERIALIZE_JSON), sha1(SECURITY_ENCRYPTION_KEY)),
						'xml' => $xml]);
				}
				break;
			default:
				if($enctype){
					$formAttribs['enctype'] = $enctype;
				}
				if($target){
					$formAttribs['target'] = $target;
				}
				$formAttribs['name'] = $formname;
				$formAttribs['onsubmit'] = $onsubmit;
				$formAttribs['action'] = $we_form_action;


				if(empty($GLOBALS['we']['editmode'])){
					$ret .= getHtmlTag('form', $formAttribs, '', false, true);
				}
		}
		return $ret;
	}

}

function we_tag_form(array $attribs){
	return we_tag_form::tag($attribs);
}
