<?php

/**
 * webEdition CMS
 *
 * $Rev: 3e18469368b7 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2018-11-20 23:00 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_include extends we_weTag_tag{

	public static function parse(array $attribs, $content){
		$type = self::getParserAttribute('type', $attribs, 'document');
		$once = (self::getParserAttribute('once', $attribs, false, true) ? '_once' : '');
		if($type === 'template'){
			$attribs['_parsed'] = 'true';
		}

		return ($type === 'template' ?
			'<?php if(($we_inc=' . we_weTag_parser::printTag('include', $attribs) . ')){include' . $once . '($we_inc);}; ?>' : //include templates of ID's
			'<?php if(($we_inc=' . we_weTag_parser::printTag('include', $attribs) . ')){we_tag_include::setBackVar(we_newBackVarID());include' . $once . '($we_inc);we_tag_include::resetBackVar(-1);}?>' //include documents
			);
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$id = intval(self::getAttribute('id', $attribs, 0, we_base_request::INT));
		$path = self::getAttribute('path', $attribs, '', we_base_request::FILE);

		if(self::getAttribute('type', $attribs, '', we_base_request::STRING) === 'template'){
			if(!isset($attribs['_parsed'])){
				echo 'cannot use we:include with type="template" dynamically';
				return '';
			}
			$ret = rtrim(preg_replace('/.tmpl$/i', '.php', ($id ? id_to_path($id, TEMPLATES_TABLE) : str_replace('..', '', $path))), '/'); //filter rel. paths
			return (empty($ret) ? '' : TEMPLATES_PATH . $ret);
		}

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$gethttp = self::getAttribute('gethttp', $attribs, false, we_base_request::BOOL);
		$seeMode = self::getAttribute((isset($attribs['seem']) ? 'seem' : 'seeMode'), $attribs, true, we_base_request::BOOL);
		$once = self::getAttribute('once', $attribs, false, we_base_request::BOOL);

		if((!$id) && (!$path) && (!$name)){
			t_e('we:include - missing id, path or name');
			echo '<!-- we:include - missing id, path or name !!-->';
			return '';
		}

		if(we_tag('ifEditmode')){
			if($name && !($id || $path)){
				$type = self::getAttribute('kind', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
				$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
				$description = self::getAttribute('description', $attribs, g_l('weTag', '[include][include_file]'), we_base_request::RAW);

				echo '<table class="we we_include weEditTable"><tr><td>' . $description . '</td></tr><tr><td>' .
				we_tag('href', ['name' => $nameOrig, 'rootdir' => self::getAttribute('rootdir', $attribs, '/', we_base_request::FILE), 'startid' => self::getAttribute('startid', $attribs, 0, we_base_request::INT),
					'type' => $type, 'size' => self::getAttribute('size', $attribs, 50, we_base_request::UNIT)]) .
				'</td></tr></table>';
				return '';
			}
		} else //notEditmode
		if($name && !($id || $path)){
			$type = self::getAttribute('kind', $attribs, we_base_link::TYPE_ALL, we_base_request::STRING);
			$nameOrig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
			$nint = $name . we_base_link::MAGIC_INT_LINK;
			$int = intval($GLOBALS['we_doc']->getElement($nint, we_contents_base::ETYPE_NUMBER));
			if($int && ($id = intval($GLOBALS['we_doc']->getElement($nint . 'ID', we_contents_base::ETYPE_NUMBER)))){
				$path = '';
			} else {
				$path = we_tag('href', ['name' => $nameOrig, 'hidedirindex' => 'false', 'type' => $type, 'isInternal' => 1]);
			}
		}

		if(
			(!$id && !$path) ||
			(!empty($GLOBALS['we']['main']['id']) && $GLOBALS['we']['main']['id'] == $id)//don't include same id
		){
			return '';
		}

		if($id){
			switch($GLOBALS['we_doc']->ContentType){
				case we_base_ContentTypes::WEDOCUMENT:
					break; //don't include any unknown document
				case we_base_ContentTypes::TEMPLATE:
					if($GLOBALS['we_doc']->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW ||
						$GLOBALS['we_doc']->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE){
						break;
					}
				default:
					return '';
			}
			$tmp = getHash('SELECT Path,ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id) . ' AND Published>0', null);
			$realPath = $tmp ? $tmp['Path'] : '';
			$ct = $tmp ? $tmp['ContentType'] : '';
			switch($ct){
				case we_base_ContentTypes::WEDOCUMENT:
				case we_base_ContentTypes::JS:
				case we_base_ContentTypes::CSS:
				case we_base_ContentTypes::HTML:
				case we_base_ContentTypes::TEXT:
				case we_base_ContentTypes::XML:
					break;
				default:
					return '';
			}
		} else {
			$realPath = $path;
		}
		if(!$realPath){
			return '';
		}

		$isSeemode = (we_tag('ifSeeMode'));
		// check early if there is a document - if not the rest is never needed
		if($gethttp){
			$we_unique = we_newBackVarID();
			self::setBackVar($we_unique);
			$st = 0;
			echo ($isSeemode && $seeMode && ($id || $path) ? we_gui_SEEM::getSeemAnchors(($id ?: path_to_id($path, FILE_TABLE, $GLOBALS['DB_WE'])), 'include') : '') .
			getHTTP(getServerUrl(true), $realPath, $st);
			self::resetBackVar($we_unique);
			return '';
		}

		$realPath = WEBEDITION_PATH . '..' . $realPath; //(symlink) webEdition always points to the REAL DOC-Root!
		if(!file_exists($realPath) || !is_file($realPath)){
//		t_e('include of', 'id:' . $id . ',path:' . $path . ',name:' . $name, ' doesn\'t exist');
			return '';
		}
		//check Customer-Filter on static documents
		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && $id){
			$filter = we_customer_documentFilter::getFilterByIdAndTable($id, FILE_TABLE, $GLOBALS['DB_WE']);

			if(is_object($filter)){
				if($filter->accessForVisitor($id, $ct, true) != we_customer_documentFilter::ACCESS){
					return '';
				}
			}
		}

		if($isSeemode && $seeMode && ($id || $path)){
			echo we_gui_SEEM::getSeemAnchors(($id ?: path_to_id($path, FILE_TABLE, $GLOBALS['DB_WE'])), 'include');
		}

		return $realPath;
	}

	public static function setBackVar($we_unique){
		$GLOBALS['we']['backVars'][$we_unique] = [
			'we_doc' => clone($GLOBALS['we_doc']),
			'GLOBAL' => [
				'WE_IS_DYN' => isset($GLOBALS['WE_IS_DYN']) ? 1 : 0,
				'WE_DOC_ID' => $GLOBALS['WE_DOC_ID'],
				'we_cmd' => we_base_request::_(we_base_request::RAW, 'we_cmd', ''),
				'we_transaction' => isset($GLOBALS['we_transaction']) ? $GLOBALS['we_transaction'] : '',
				'we_ContentType' => isset($GLOBALS['we_ContentType']) ? $GLOBALS['we_ContentType'] : we_base_ContentTypes::WEDOCUMENT,
			],
			'REQUEST' => [
				'we_cmd' => we_base_request::_(we_base_request::RAW, 'we_cmd', ''),
			],
			'we' => [
				'postTagName' => isset($GLOBALS['we']['postTagName']) ? $GLOBALS['we']['postTagName'] : '',
				'editmode' => isset($GLOBALS['we']['editmode']) ? $GLOBALS['we']['editmode'] : null,
			]
		];

		if(isset($GLOBALS['WE_IS_DYN'])){
			unset($GLOBALS['WE_IS_DYN']);
		}
		if(isset($GLOBALS['we']['postTagName'])){
			unset($GLOBALS['we']['postTagName']);
		}
	}

	/**
	 * function of we:include
	 * @param string $index
	 */
	public static function resetBackVar($we_unique = -1){
		$keys = array_keys($GLOBALS['we']['backVars']);
		$index = ($we_unique === -1 ? end($keys) : $we_unique);
		$GLOBALS['we_doc'] = clone($GLOBALS['we']['backVars'][$index]['we_doc']);
		foreach($GLOBALS['we']['backVars'][$index]['GLOBAL'] as $key => $val){
			$GLOBALS[$key] = $val;
		}
		foreach($GLOBALS['we']['backVars'][$index]['REQUEST'] as $key => $val){
			$_REQUEST[$key] = $val;
		}
		foreach($GLOBALS['we']['backVars'][$index]['we'] as $key => $val){
			$GLOBALS['we'][$key] = $val;
		}

		if($GLOBALS['we']['backVars'][$index]['GLOBAL']['WE_IS_DYN']){
			$GLOBALS['WE_IS_DYN'] = 1;
		} else if(isset($GLOBALS['WE_IS_DYN'])){
			unset($GLOBALS['WE_IS_DYN']);
		}
		unset($GLOBALS['we']['backVars'][$index]);
	}

}

function we_setBackVar($we_unique){
	we_tag_include::setBackVar($we_unique);
}

function we_tag_include(array $attribs){//FIXME: include doesn't work in editmode - check funktionen
	return we_tag_include::tag($attribs);
}
