<?php

/**
 * webEdition CMS
 *
 * $Rev: a533000e58bf $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-02-01 09:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_sessionStart extends we_weTag_tag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$GLOBALS['we']['sessionStart'] = true;
		if((!empty($GLOBALS['we']['main']['doc']->InWebEdition)) || !empty($GLOBALS['we']['editmode']) || !empty($GLOBALS['we']['Scheduler_active']) || !empty($GLOBALS['we']['Versioning_active'])){
			return '';
		}

		if(!isset($_SESSION)){
			new we_base_sessionHandler();
		}

		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return '';
		}

		if(!empty($_REQUEST['we_webUser_logout'])){
			if(!empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID'])){
				/* (isset($_REQUEST['s']['AutoLogin']) && !$_REQUEST['s']['AutoLogin']) || (isset($_SESSION['webuser']['AutoLogin']) && !$_SESSION['webuser']['AutoLogin'])) && */
				if(!empty($_SESSION['webuser']['AutoLoginID'])){
					$GLOBALS['DB_WE']->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE AutoLoginID=x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\'');
					setcookie('_we_autologin', '', (time() - 3600), '/');
				}
				//we want to know who was logged out, so keep this info
				$GLOBALS['we']['logout'] = $_SESSION['webuser']['ID'];
			}
			unset($_SESSION['s'], $_REQUEST['s']);
			if(SECURITY_DELETE_SESSION){
				we_base_sessionHandler::makeNewID(true);
			}
			$_SESSION['webuser'] = ['registered' => false];
			return '';
		}

		$SessionAutologin = 0;

		if(!isset($_SESSION['webuser'])){
			$_SESSION['webuser'] = ['registered' => false];
		}
		$persistentlogins = self::getAttribute('persistentlogins', $attribs, false, we_base_request::BOOL);
		if(!$_SESSION['webuser']['registered'] && isset($_REQUEST['s']['Username']) && isset($_REQUEST['s']['Password']) && !(isset($_REQUEST['s']['ID'])) && !isset($_REQUEST['s']['Password2'])//if set, we assume it is a password reset or use of an forgotten password routine, so we don't try to do an login
		){
			$GLOBALS['DB_WE']->query('DELETE FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND LoginDate<(NOW() - INTERVAL ' . we_base_constants::LOGIN_FAILED_HOLDTIME . ' DAY)');
			$checkPassword = true;
			$hook = new we_hook_base('customer_preLogin', '', ['customer' => &$_REQUEST['s'], 'type' => 'normal', 'tagname' => 'sessionStart', 'checkPassword' => &$checkPassword,]);
			$hook->executeHook();

			if(!self::startdoLogin($persistentlogins, $SessionAutologin, !$checkPassword)){
				self::handleFailedLogin();
			} else {
				$GLOBALS['DB_WE']->query('UPDATE ' . FAILED_LOGINS_TABLE . ' SET isValid="false" WHERE UserTable="tblWebUser" AND Username="' . $GLOBALS['DB_WE']->escape($_REQUEST['s']['Username']) . '"');
				//change session ID to prevent session
				we_base_sessionHandler::makeNewID();
				$hook = new we_hook_base('customer_Login', '', ['customer' => &$_SESSION['webuser'], 'type' => 'normal', 'tagname' => 'sessionStart']);
				$hook->executeHook();
			}
			unset($_REQUEST['s']['Password']);
		}
		if($persistentlogins && (!$_SESSION['webuser']['registered']) && isset($_COOKIE['_we_autologin'])){
			if(!self::startdoAutoLogin()){
				//in case login with cookie failed, remove the cookie
				setcookie('_we_autologin', '', (time() - 3600), '/');
				self::handleFailedLogin();
			} else {
				we_base_sessionHandler::makeNewID();
			}
		}

		if(!empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID']) && !empty($_SESSION['webuser']['Username'])){
			if($_SESSION['webuser']['LastAccess'] + 60 < time()){
				$GLOBALS['DB_WE']->query('UPDATE ' . CUSTOMER_TABLE . ' SET LastAccess=UNIX_TIMESTAMP() WHERE ID=' . intval($_SESSION['webuser']['ID']));
				$_SESSION['webuser']['LastAccess'] = time();
			}
		}

		if(!empty($_SESSION['webuser']['registered']) && self::getAttribute('onlinemonitor', $attribs, false, we_base_request::BOOL)){
			$GLOBALS['DB_WE']->query('DELETE FROM ' . CUSTOMER_SESSION_TABLE . ' WHERE LastAccess<(NOW() - INTERVAL 1 HOUR)');
			$monitorgroupfield = self::getAttribute('monitorgroupfield', $attribs, '', we_base_request::STRING);
			$doc = self::getDocForTag(self::getAttribute('monitordoc', $attribs, '', we_base_request::STRING), false);

			$WebUserID = ($_SESSION['webuser']['registered'] ? $_SESSION['webuser']['ID'] : 0);
			$WebUserGroup = ($_SESSION['webuser']['registered'] && $monitorgroupfield && $_SESSION['webuser'][$monitorgroupfield] ? $_SESSION['webuser'][$monitorgroupfield] : 'we_guest');
			$sessID = session_id();
			$sessID = (preg_match('|^([a-f0-9]){32,40}$|', $sessID) ? str_pad($sessID, 40, '0') : '00'/* sessid is invalid */);

			$GLOBALS['DB_WE']->query('INSERT INTO ' . CUSTOMER_SESSION_TABLE . ' SET ' .
				we_database_base::arraySetter([
					'SessionID' => sql_function('x\'' . $sessID . '\''),
					'SessionIp' => inet_pton(strstr($_SERVER['REMOTE_ADDR'], ':') ? $_SERVER['REMOTE_ADDR'] : '::ffff:' . $_SERVER['REMOTE_ADDR']),
					'WebUserID' => $WebUserID,
					'WebUserGroup' => $WebUserGroup,
					'Browser' => isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '',
					'Referrer' => isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '',
					'LastLogin' => sql_function('NOW()'),
					'PageID' => $doc->ID,
					'SessionAutologin' => $SessionAutologin
				]) . ' ON DUPLICATE KEY UPDATE ' . we_database_base::arraySetter([
					'PageID' => $doc->ID,
					'WebUserID' => $WebUserID,
					'WebUserGroup' => $WebUserGroup,
			]));
		}
		//remove sessions consisting only of webuser[registered]
		if(!empty($_SESSION['webuser']) && count($_SESSION['webuser']) == 1){
			unset($_SESSION['webuser']);
		}
		return '';
	}

	//keep this public
	public static function handleFailedLogin(){
		$_SESSION['webuser'] = ['registered' => false, 'loginfailed' => we_users_user::INVALID_CREDENTIALS];
		if(!isset($GLOBALS['WE_LOGIN_DENIED'])){
			we_users_user::logLoginFailed('tblWebUser', $_REQUEST['s']['Username']);
		}
		sleep(SECURITY_DELAY_FAILED_LOGIN);
		//make it harder to guess salt/password
		usleep(rand(0, 1000000));


		if(
			intval(f('SELECT COUNT(1)  FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND Username="' . $GLOBALS['DB_WE']->escape($_REQUEST['s']['Username']) . '" AND isValid="true" AND LoginDate >(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour)')) >= intval(SECURITY_LIMIT_CUSTOMER_NAME) ||
			intval(f('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND IP="' . $_SERVER['REMOTE_ADDR'] . '" AND LoginDate >(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_IP_HOURS) . ' hour)')) >= intval(SECURITY_LIMIT_CUSTOMER_IP)
		){
			//don't serve user
			if(SECURITY_LIMIT_CUSTOMER_REDIRECT){
				$_SESSION['webuser']['loginfailed'] = we_users_user::MAX_LOGIN_COUNT_REACHED;
				unset($_REQUEST['s']);
				if(($path = id_to_path(SECURITY_LIMIT_CUSTOMER_REDIRECT, FILE_TABLE))){
					include(WEBEDITION_PATH . '../' . $path);
					exit();
				}
			}
			echo CheckAndConvertISOfrontend('Dear customer, our service is currently not available. Please try again later. Thank you.<br/>' .
				'Sehr geehrter Kunde, aus Sicherheitsgründen ist ein Login derzeit nicht möglich! Bitte probieren Sie es später noch ein mal. Vielen Dank');

			exit();
		}
	}

	/* Function made available to check externally if userlogin is denied */

	private static function startCheckDenied(){
		if(
			intval(f('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND Username="' . $GLOBALS['DB_WE']->escape($_REQUEST['s']['Username']) . '" AND isValid="true" AND LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour)')) >= intval(SECURITY_LIMIT_CUSTOMER_NAME) ||
			intval(f('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND IP="' . $_SERVER['REMOTE_ADDR'] . '" AND LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_IP_HOURS) . ' hour)')) >= intval(SECURITY_LIMIT_CUSTOMER_IP)
		){
			$GLOBALS['WE_LOGIN_DENIED'] = true;
			return true;
		}
		return false;
	}

	//keep this public!
	public static function startdoLogin($persistentlogins, &$SessionAutologin, $externalPasswordCheck = false){ //FIXME: check for last time =>(cuncurrent logins)
		if($_REQUEST['s']['Username'] && $_REQUEST['s']['Password']){
			if(self::startCheckDenied()){
				return false;
			}
			$wasRegistered = !empty($_SESSION['webuser']['registered']) && $_SESSION['webuser']['registered'];
			$u = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE Password!="" ' . ($externalPasswordCheck ? '' : 'AND LoginDenied=0') . ' AND Username="' . $GLOBALS['DB_WE']->escape($_REQUEST['s']['Username']) . '"', null, MYSQLI_ASSOC);
			if($u && ($externalPasswordCheck || we_customer_customer::comparePassword($u['Password'], $_REQUEST['s']['Password'], $u['ID']))){
				if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
					unset($u['Password']);
				}

				$_SESSION['webuser'] = array_merge($u, we_customer_customer::getEncryptedFields());
				//keep Password if known
				if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
					$_SESSION['webuser']['_Password'] = $_REQUEST['s']['Password'];
				}
				$_SESSION['webuser']['registered'] = true;
				$GLOBALS['DB_WE']->query('UPDATE ' . CUSTOMER_TABLE . ' SET LastLogin=UNIX_TIMESTAMP() WHERE ID=' . intval($_SESSION['webuser']['ID']));

				if($persistentlogins && !empty($_REQUEST['s']['AutoLogin']) && $_SESSION['webuser']['AutoLoginDenied'] != 1){
					$_SESSION['webuser']['AutoLoginID'] = uniqid(substr(session_id(), 0, 8), true);
					$GLOBALS['DB_WE']->query('INSERT INTO ' . CUSTOMER_AUTOLOGIN_TABLE . ' SET ' . we_database_base::arraySetter([
							'AutoLoginID' => sql_function('x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\''),
							'WebUserID' => $_SESSION['webuser']['ID'],
							'LastIp' => $_SERVER['REMOTE_ADDR']
					]));

					$cookie = session_get_cookie_params();
					setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), '/', '', $cookie['secure'], true);
					$GLOBALS['DB_WE']->query('UPDATE ' . CUSTOMER_TABLE . ' SET AutoLogin=1 WHERE ID=' . intval($_SESSION['webuser']['ID']));
					$_SESSION['webuser']['AutoLogin'] = 1;
					$SessionAutologin = 1;
				}
				$GLOBALS['WE_LOGIN'] = !$wasRegistered;
				return true;
			}
		}
		return false;
	}

	private static function startdoAutoLogin(){
		$autologinSeek = $_COOKIE['_we_autologin'];
		if(!empty($autologinSeek)){
			$hook = new we_hook_base('customer_preLogin', '', ['customer' => &$_REQUEST['s'], 'type' => 'autoLogin', 'tagname' => 'sessionStart']);
			$hook->executeHook();

			$wasRegistered = $_SESSION['webuser']['registered'];
			$u = getHash('SELECT u.* FROM ' . CUSTOMER_TABLE . ' u JOIN ' . CUSTOMER_AUTOLOGIN_TABLE . ' c ON u.ID=c.WebUserID WHERE u.LoginDenied=0 AND u.AutoLoginDenied=0 AND u.Password!="" AND c.AutoLoginID=x\'' . sha1($autologinSeek) . '\'', null, MYSQLI_ASSOC);
			if($u){
				if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
					unset($u['Password']);
				}

				$_SESSION['webuser'] = array_merge($u, we_customer_customer::getEncryptedFields());
				//try to decrypt password if possible
				if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
					$_SESSION['webuser']['_Password'] = we_customer_customer::decryptData($_SESSION['webuser']['Password']);
				}
				$_SESSION['webuser']['registered'] = true;
				$_SESSION['webuser']['AutoLoginID'] = uniqid(substr(session_id(), 0, 8), true);
				$GLOBALS['DB_WE']->query('UPDATE ' . CUSTOMER_AUTOLOGIN_TABLE . ' SET ' . we_database_base::arraySetter([
						'AutoLoginID' => sql_function('x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\''),
						'LastIp' => $_SERVER['REMOTE_ADDR'],
					]) . ' WHERE WebUserID=' . intval($_SESSION['webuser']['ID']) . ' AND AutoLoginID=x\'' . sha1($autologinSeek) . '\''
				);

				$cookie = session_get_cookie_params();
				setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), '/', '', $cookie['secure'], true);
				$GLOBALS['WE_LOGIN'] = $wasRegistered;
				$hook = new we_hook_base('customer_Login', '', ['customer' => &$_SESSION['webuser'], 'type' => 'autoLogin', 'tagname' => 'sessionStart']);
				$hook->executeHook();
				return true;
			}
		}

		return false;
	}

}

function we_tag_sessionStart(array $attribs){
	return we_tag_sessionStart::tag($attribs);
}
