<?php

/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Document Definition base class
 */
abstract class we_workflow_base{

	var $uid;
	var $db;
	var $persistents = [];
	var $table = '';
	var $ClassName = __CLASS__;

	function __construct(){
		$this->uid = 'wf_' . md5(uniqid(__FILE__, true));
		$this->db = new DB_WE();
	}

	function load(){
		$data = getHash('SELECT * FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->ID));
		if(!$data){
			return;
		}
		foreach($data as $key => $value){
			if(isset($this->persistents[$key])){
				switch($this->persistents[$key]){
					case we_base_request::INTLISTA:
						$this->$key = ($value === '' ? [] : explode(',', trim($value, ',')));
						break;
					case we_base_request::DATE_TIME:
						$this->$key = ($value ? null : new DateTime($value));
						break;
					default:
						$this->$key = $value;
				}
			}
		}
	}

	function save(){
		$sets = $wheres = [];
		foreach(array_keys($this->persistents) as $val){
			if($val === 'ID'){
				$wheres[] = 'ID=' . intval($this->{$val});
			} else {
				$sets[$val] = is_array($this->{$val}) ? implode(',', array_unique($this->{$val})) : $this->{$val};
			}
		}
		$where = implode(',', $wheres);
		$set = we_database_base::arraySetter($sets);

		if($this->ID == 0){
			$this->db->query('INSERT INTO ' . $this->db->escape($this->table) . ' SET ' . $set);
			# get ID #
			$this->ID = $this->db->getInsertId();
		} else {
			$this->db->query('UPDATE ' . $this->db->escape($this->table) . ' SET ' . $set . ' WHERE ' . $where);
		}
	}

	function delete(){
		if(!$this->ID){
			return false;
		}
		$this->db->query('DELETE FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->ID));
		return true;
	}

	protected static function getCurrentUser($field = ''){
		$ret = getHash('SELECT CONCAT(First," ",Second) AS Name,IF(Email,CONCAT("\"",First," ",Second,"\" <",Email,">"),"") AS Mail FROM ' . USER_TABLE . ' WHERE ID=' . intval($_SESSION['user']['ID']), $GLOBALS['DB_WE']);
		return $field ? $ret[$field] : $ret;
	}

	function sendMail($userID, $subject, $description){
		$reciepient = f('SELECT Email FROM ' . USER_TABLE . ' WHERE ID=' . intval($userID), '', $this->db);
		if($reciepient && we_check_email($reciepient)){
			$this_user = self::getCurrentUser('Mail');
			we_mail($reciepient, we_base_util::convertDisallowedChars($subject), $description, '', $this_user);
		}
	}

}
