<?php

/**
 * webEdition CMS
 *
 * $Rev: 05d72ed1885d $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2018-10-19 20:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
function we_include_tag_file($filename){
	// as default: all tag_functions are in this file.
	if(class_exists($filename, false) && method_exists($filename, 'tag') || function_exists($filename)){
		// do noting
		return true;
	}
	if(file_exists(($file = WE_INCLUDES_PATH . 'we_classes/we_tags/' . $filename . '.class.php'))){
		require_once ($file);
		return true;
	}
	//error check is only required for custom tags
	if(file_exists(($file = WE_INCLUDES_PATH . 'we_tags/custom_tags/' . $filename . '.class.php')) ||
		file_exists(($file = WE_INCLUDES_PATH . 'we_tags/custom_tags/' . $filename . '.inc.php'))){
		require_once ($file);
		return class_exists($filename, false) && method_exists($filename, 'tag') || function_exists($filename) ? true : we_weTag_tag::parseError(sprintf(g_l('parser', '[tag_not_known]'), str_replace('we_tag_', '', $filename)));
	}

	if(substr($filename, 0, 2) === 'if'){ // this ifTag does not exist
		echo we_weTag_tag::parseError(sprintf(g_l('parser', '[tag_not_known]'), str_replace('we_tag_', '', $filename)));
		return false;
	}
	return we_weTag_tag::parseError(sprintf(g_l('parser', '[tag_not_known]'), str_replace('we_tag_', '', $filename)));
}

/**
 * get the full name of an Attribute with applied postTagName if set
 * @param string $var
 * @return string
 */
function we_tag_getPostName($var){//fixme: remove
	return we_weTag_tag::getPostName($var);
}

function we_profiler($start = true){
	if($start){
		if(isset($_SERVER['REQUEST_TIME_FLOAT'])){
			define('WE_PROFILER', (float) $_SERVER['REQUEST_TIME_FLOAT']);
		} else {
			define('WE_PROFILER', microtime(true));
		}
		$GLOBALS['we_profile'] = [];
	} else {
		echo implode(',', array_keys($GLOBALS['we_profile'][0])) . "<br/>\n";
		foreach($GLOBALS['we_profile'] as $line){
			echo implode(',', $line) . "<br/>\n";
		}
	}
}

function we_tag($name, array $attribs = [], $content = '', $internal = false){
	//keep track of editmode
	$edMerk = isset($GLOBALS['we']['editmode']) ? $GLOBALS['we']['editmode'] : '';
	//FIXME: do we support this????
	$user = we_weTag_tag::getAttribute('user', $attribs, [], we_base_request::STRING_LIST);
	if(defined('WE_PROFILER')){
		static $last = WE_PROFILER;
		$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
		$cur = microtime(true);
		$prof = [
			'tag' => $name,
			'line' => $bt[0]['line'],
			'file' => $bt[0]['file'],
			'relTime' => round(($cur - $last) * 100, 2) . ' ms',
			'absTime' => round(($cur - WE_PROFILER) * 100, 2) . ' ms',
			'mem' => round(((memory_get_usage() / 1024))) . ' kB',
			''
		];
		$last = $cur;
		$GLOBALS['we_profile'][] = $prof;
		//annotate tag, if possible
		$attribs['data-time'] = $prof['relTime'];
		$attribs['data-mem'] = $prof['mem'];
		$attribs['data-line'] = $prof['line'];
		$attribs['data-file'] = $prof['file'];
	}
	//make sure comment attribute is never shown
	switch($name){
		case 'setVar':
		case 'xmlnode'://special handling inside tag setVar and xmlnode
			$attribs = removeAttribs($attribs, ['cachelifetime', 'comment', 'user']);
			$nameTo = '';
			$to = 'screen';
			break;
		default:
			$to = we_weTag_tag::getAttribute('to', $attribs, 'screen');
			$nameTo = we_weTag_tag::getAttribute('nameto', $attribs, isset($attribs['name']) ? $attribs['name'] : '');
			$attribs = removeAttribs($attribs, ['cachelifetime', 'comment', 'to', 'nameto', 'user']);

			/* if to attribute is set, output of the tag is redirected to a variable
			 * this makes only sense if tag output is equal to non-editmode */
			if($to != 'screen'){
				$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = false;
			}
	}

	//make a copy of the name - this copy is never touched even not inside blocks/listviews etc.
	if(isset($attribs['name'])){
		$attribs['_name_orig'] = $attribs['name'];
		$attribs['name'] = we_weTag_tag::getPostName($attribs['name']);
		if(!empty($GLOBALS['we']['editmode']) && ($GLOBALS['we_doc'] instanceof we_webEditionDocument)){
			$GLOBALS['we_doc']->addUsedElement($name, $attribs['name']);
		}
	}

	if($edMerk && $user && (!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR))){
		if(!in_array($_SESSION['user']['Username'], $user)){
			$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = false;
		}
	}

	$fn = 'we_tag_' . $name;
	if(($foo = we_include_tag_file($fn)) !== true){
		return $foo;
	}

	switch($name){
		case 'setVar':
			we_tag_setVar::tag($attribs);
			//nothing more to do don't waste time
			return '';
		default:
			$foo = class_exists($fn, false) && method_exists($fn, 'tag') ? $fn::tag($attribs, $content, $internal) : $fn($attribs, $content, $internal);
			$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = $edMerk;
			return we_weTag_tag::redirectTagOutput($foo, $nameTo, $to);
	}
}

//tag utility functions

function getArrayValue($var, $name, $arrayIndex, $isset = false){
	we_weTag_tag::getArrayValue($var, $name, $arrayIndex, $isset);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getParserAttribute($name, $attribs, $default = '', $type = we_base_request::RAW, $isDeprecated = false){
	return we_weTag_tag::getAttribute($name, $attribs, $default, $type, $isDeprecated, false);
}

/**
 * get an attribute from $attribs, and return its value according to default
 * @param string $name attributes name
 * @param array $attribs array containg the attributes
 * @param mixed $default default value
 * @param bool $useGlobal check if attribute value is a php-variable and is found in $GLOBALS
 * @return mixed returns the attributes value or default if not set
 */
function weTag_getAttribute($name, $attribs, $default = '', $type = we_base_request::RAW, $isDeprecated = false, $useGlobal = true){
	return we_weTag_tag::getAttribute($name, $attribs, $default, $type, $isDeprecated, $useGlobal);
}

/**
 * @deprecated since version 8.0.4
 */
function we_getDocForTag($docAttr, $maindefault = false){
	return we_weTag_tag::getDocForTag($docAttr, $maindefault);
}

/**
 * @deprecated since version 8.0.4
 */
function modulFehltError($modul, $tag){
	return we_weTag_tag::modulMissingError($modul, $tag);
}

/**
 * @deprecated since version 8.0.4
 */
function parseError($text, $extra = ''){
	return we_weTag_tag::parseError($text, $extra);
}

/**
 * @deprecated since version 8.0.4
 */
function attributFehltError($attribs, $attrs, $tag, $canBeEmpty = false){
	return we_weTag_tag::attributeMissing($attribs, $attrs, $tag, $canBeEmpty);
}

/**
 * @return array
 * @param array $atts
 * @param array $ignore
 * @desc only uses the attribs given in the array use
 */
function useAttribs(array $atts, array $use){
	return array_intersect_key($atts, array_flip($use));
}

function we_getInputRadioField($name, $value, $itsValue, $atts){
	return we_weTag_tag::inputRadioField($name, $value, $itsValue, $atts);
}

function we_getTextareaField($name, $value, array $atts){
	return we_weTag_tag::textareaField($name, $value, $atts);
}

function we_getInputTextInputField($name, $value, array $atts){
	return we_weTag_tag::inputTextInputField($name, $value, $atts);
}

function we_getInputCheckboxField($name, $value, array $attr){
	return we_weTag_tag::inputCheckboxField($name, $value, $attr);
}

function we_getSelectField($name, $value, array $values, array $attribs = [], $addMissing = true){
	return we_weTag_tag::selectField($name, $value, $values, $attribs, $addMissing);
}

/**
 * @deprecated since version 8.0.4
 */
function we_pre_tag_listview($newLV){
	we_weTag_tag::pre_listview($newLV);
}

/**
 * @deprecated since version 8.0.4
 */
function we_post_tag_listview(){
	we_weTag_tag::post_listview();
}

/**
 * @deprecated since version 8.0.4
 */
function getFieldOutLang(array $attribs, $fallBackBackend = false){
	return we_weTag_tag::getOutLang($attribs, $fallBackBackend);
}
